@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * System-dependent configurations of Tokyo Cabinet
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -27,67 +27,67 @@
 #if defined(__linux__)
 
 #define _SYS_LINUX_
-#define TCSYSNAME  "Linux"
+#define TCSYSNAME   "Linux"
 
 #elif defined(__FreeBSD__)
 
 #define _SYS_FREEBSD_
-#define TCSYSNAME  "FreeBSD"
+#define TCSYSNAME   "FreeBSD"
 
 #elif defined(__NetBSD__)
 
 #define _SYS_NETBSD_
-#define TCSYSNAME  "NetBSD"
+#define TCSYSNAME   "NetBSD"
 
 #elif defined(__OpenBSD__)
 
 #define _SYS_OPENBSD_
-#define TCSYSNAME  "OpenBSD"
+#define TCSYSNAME   "OpenBSD"
 
 #elif defined(__sun__)
 
 #define _SYS_SUNOS_
-#define TCSYSNAME  "SunOS"
+#define TCSYSNAME   "SunOS"
 
 #elif defined(__hpux)
 
 #define _SYS_HPUX_
-#define TCSYSNAME  "HP-UX"
+#define TCSYSNAME   "HP-UX"
 
 #elif defined(__osf)
 
 #define _SYS_TRU64_
-#define TCSYSNAME  "Tru64"
+#define TCSYSNAME   "Tru64"
 
 #elif defined(_AIX)
 
 #define _SYS_AIX_
-#define TCSYSNAME  "AIX"
+#define TCSYSNAME   "AIX"
 
 #elif defined(__APPLE__) && defined(__MACH__)
 
 #define _SYS_MACOSX_
-#define TCSYSNAME  "Mac OS X"
+#define TCSYSNAME   "Mac OS X"
 
 #elif defined(_MSC_VER)
 
 #define _SYS_MSVC_
-#define TCSYSNAME  "Windows (VC++)"
+#define TCSYSNAME   "Windows (VC++)"
 
 #elif defined(_WIN32)
 
 #define _SYS_MINGW_
-#define TCSYSNAME  "Windows (MinGW)"
+#define TCSYSNAME   "Windows (MinGW)"
 
 #elif defined(__CYGWIN__)
 
 #define _SYS_CYGWIN_
-#define TCSYSNAME  "Windows (Cygwin)"
+#define TCSYSNAME   "Windows (Cygwin)"
 
 #else
 
 #define _SYS_GENERIC_
-#define TCSYSNAME  "Generic"
+#define TCSYSNAME   "Generic"
 
 #endif
 
@@ -153,6 +153,13 @@
 #define TCITOHLL(TC_num)  (TC_num)
 #endif
 
+#if defined(_SYS_LINUX_) || defined(_SYS_FREEBSD_) || defined(_SYS_NETBSD_) || \
+  defined(_SYS_MACOSX_) || defined(_SYS_SUNOS_)
+#define TCUBCACHE      1
+#else
+#define TCUBCACHE      0
+#endif
+
 #if defined(_MYNOZLIB)
 #define TCUSEZLIB      0
 #else
@@ -222,11 +229,13 @@
 #include <stdint.h>
 
 #include <unistd.h>
+#include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <sys/time.h>
 #include <sys/times.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <dirent.h>
 #include <regex.h>
@@ -234,6 +243,9 @@
 
 #if TCUSEPTHREAD
 #include <pthread.h>
+#if defined(_POSIX_PRIORITY_SCHEDULING)
+#include <sched.h>
+#endif
 #endif
 
 
@@ -245,7 +257,23 @@
 
 #define sizeof(a)      ((int)sizeof(a))
 
+#if defined(_SYS_FREEBSD_) || defined(_SYS_NETBSD_) || defined(_SYS_OPENBSD_)
+#define nan(TC_a)      strtod("nan", NULL)
+#endif
+
+#if ! defined(PATH_MAX)
+#if defined(MAXPATHLEN)
+#define PATH_MAX       MAXPATHLEN
+#else
+#define PATH_MAX       4096
+#endif
+#endif
+
+
+extern int _tc_dummy_cnt;
+
 int _tc_dummyfunc(void);
+
 int _tc_dummyfuncv(int a, ...);
 
 
@@ -313,6 +341,8 @@ void *_tc_recdecode(const void *ptr, int size, int *sp, void *op);
 
 #if ! TCUSEPTHREAD
 
+#define pthread_t                        intptr_t
+
 #define pthread_once_t                   intptr_t
 #undef PTHREAD_ONCE_INIT
 #define PTHREAD_ONCE_INIT                0
@@ -342,6 +372,7 @@ void *_tc_recdecode(const void *ptr, int size, int *sp, void *op);
 #define pthread_rwlock_wrlock(TC_a)      _tc_dummyfuncv((intptr_t)(TC_a))
 #define pthread_rwlock_unlock(TC_a)      _tc_dummyfuncv((intptr_t)(TC_a))
 
+#define pthread_key_t                    intptr_t
 #define pthread_key_create(TC_a, TC_b)   _tc_dummyfuncv((intptr_t)(TC_a), (TC_b))
 #define pthread_key_delete(TC_a)         _tc_dummyfuncv((intptr_t)(TC_a))
 #define pthread_setspecific(TC_a, TC_b)  _tc_dummyfuncv((intptr_t)(TC_a))
@@ -349,19 +380,18 @@ void *_tc_recdecode(const void *ptr, int size, int *sp, void *op);
 
 #define pthread_create(TC_th, TC_attr, TC_func, TC_arg) \
   (*(TC_th) = 0, (TC_func)(TC_arg), 0)
-#define pthread_join(TC_th, TC_rv) \
-  (*(TC_rv) = NULL, 0)
-#define pthread_detach(TC_th)         0
+#define pthread_join(TC_th, TC_rv)       (*(TC_rv) = NULL, 0)
+#define pthread_detach(TC_th)            0
+#define sched_yield()                    _tc_dummyfunc()
 
 #endif
 
 #if TCUSEPTHREAD && TCMICROYIELD
 #define TCTESTYIELD() \
   do { \
-    static uint32_t cnt = 0; \
-    if(((++cnt) & (0x20 - 1)) == 0){ \
-      pthread_yield(); \
-      if(cnt > 0x1000) cnt = (uint32_t)time(NULL) % 0x1000; \
+    if(((++_tc_dummy_cnt) & (0x20 - 1)) == 0){ \
+      sched_yield(); \
+      if(_tc_dummy_cnt > 0x1000) _tc_dummy_cnt = (uint32_t)time(NULL) % 0x1000; \
     } \
   } while(false)
 #undef assert
@@ -373,10 +403,18 @@ void *_tc_recdecode(const void *ptr, int size, int *sp, void *op);
     } \
     TCTESTYIELD(); \
   } while(false)
+#define if(TC_cond) \
+  if((((++_tc_dummy_cnt) & (0x100 - 1)) != 0 || (sched_yield() * 0) == 0) && (TC_cond))
+#define while(TC_cond) \
+  while((((++_tc_dummy_cnt) & (0x100 - 1)) != 0 || (sched_yield() * 0) == 0) && (TC_cond))
 #else
 #define TCTESTYIELD() \
   do { \
-  } while(false);
+  } while(false)
+#endif
+
+#if !defined(_POSIX_PRIORITY_SCHEDULING) && TCUSEPTHREAD
+#define sched_yield()                    usleep(1000 * 20)
 #endif
 
 
@@ -468,7 +506,12 @@ void *_tc_recdecode(const void *ptr, int size, int *sp, void *op);
     (TC_step) = _TC_i + 1; \
   } while(false)
 
-/* Compare keys of two records by lexical order. */
+/* calculate the size of a buffer for a variable length number */
+#define TCCALCVNUMSIZE(TC_num) \
+  ((TC_num) < 0x80 ? 1 : (TC_num) < 0x4000 ? 2 : (TC_num) < 0x200000 ? 3 : \
+   (TC_num) < 0x10000000 ? 4 : 5)
+
+/* compare keys of two records by lexical order */
 #define TCCMPLEXICAL(TC_rv, TC_aptr, TC_asiz, TC_bptr, TC_bsiz) \
   do { \
     (TC_rv) = 0; \
