@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The command line utility of the abstract database API
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -15,6 +15,7 @@
 
 
 #include <tcutil.h>
+#include <tcbdb.h>
 #include <tcadb.h>
 #include "myconf.h"
 
@@ -27,22 +28,31 @@ const char *g_progname;                  // program name
 int main(int argc, char **argv);
 static void usage(void);
 static void printerr(TCADB *adb);
-static int printdata(const char *ptr, int size, bool px);
-static char *hextoobj(const char *str, int *sp);
+static int sepstrtochr(const char *str);
+static char *strtozsv(const char *str, int sep, int *sp);
+static int printdata(const char *ptr, int size, bool px, int sep);
+static bool mapbdbproc(void *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
+                       void *op);
 static int runcreate(int argc, char **argv);
 static int runinform(int argc, char **argv);
 static int runput(int argc, char **argv);
 static int runout(int argc, char **argv);
 static int runget(int argc, char **argv);
 static int runlist(int argc, char **argv);
+static int runoptimize(int argc, char **argv);
+static int runmisc(int argc, char **argv);
+static int runmap(int argc, char **argv);
 static int runversion(int argc, char **argv);
 static int proccreate(const char *name);
 static int procinform(const char *name);
 static int procput(const char *name, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
                    int dmode);
 static int procout(const char *name, const char *kbuf, int ksiz);
-static int procget(const char *name, const char *kbuf, int ksiz, bool px, bool pz);
-static int proclist(const char *name, int max, bool pv, bool px, const char *fmstr);
+static int procget(const char *name, const char *kbuf, int ksiz, int sep, bool px, bool pz);
+static int proclist(const char *name, int sep, int max, bool pv, bool px, const char *fmstr);
+static int procoptimize(const char *name, const char *params);
+static int procmisc(const char *name, const char *func, const TCLIST *args, int sep, bool px);
+static int procmap(const char *name, const char *dest, const char *fmstr);
 static int procversion(void);
 
 
@@ -63,6 +73,12 @@ int main(int argc, char **argv){
     rv = runget(argc, argv);
   } else if(!strcmp(argv[1], "list")){
     rv = runlist(argc, argv);
+  } else if(!strcmp(argv[1], "optimize")){
+    rv = runoptimize(argc, argv);
+  } else if(!strcmp(argv[1], "misc")){
+    rv = runmisc(argc, argv);
+  } else if(!strcmp(argv[1], "map")){
+    rv = runmap(argc, argv);
   } else if(!strcmp(argv[1], "version") || !strcmp(argv[1], "--version")){
     rv = runversion(argc, argv);
   } else {
@@ -79,29 +95,61 @@ static void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s create name\n", g_progname);
   fprintf(stderr, "  %s inform name\n", g_progname);
-  fprintf(stderr, "  %s put [-sx] [-dk|-dc] name key value\n", g_progname);
-  fprintf(stderr, "  %s out [-sx] name key\n", g_progname);
-  fprintf(stderr, "  %s get [-sx] [-px] [-pz] name key\n", g_progname);
-  fprintf(stderr, "  %s list [-m num] [-pv] [-px] [-fm str] name\n", g_progname);
+  fprintf(stderr, "  %s put [-sx] [-sep chr] [-dk|-dc|-dai|-dad] name key value\n", g_progname);
+  fprintf(stderr, "  %s out [-sx] [-sep chr] name key\n", g_progname);
+  fprintf(stderr, "  %s get [-sx] [-sep chr] [-px] [-pz] name key\n", g_progname);
+  fprintf(stderr, "  %s list [-sep chr] [-m num] [-pv] [-px] [-fm str] name\n", g_progname);
+  fprintf(stderr, "  %s optimize name [params]\n", g_progname);
+  fprintf(stderr, "  %s misc [-sx] [-sep chr] [-px] name func [arg...]\n", g_progname);
+  fprintf(stderr, "  %s map [-fm str] name dest\n", g_progname);
   fprintf(stderr, "  %s version\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
 }
 
 
+/* get the character of separation string */
+static int sepstrtochr(const char *str){
+  if(!strcmp(str, "\\t")) return '\t';
+  if(!strcmp(str, "\\r")) return '\r';
+  if(!strcmp(str, "\\n")) return '\n';
+  return *(unsigned char *)str;
+}
+
+
+/* encode a string as a zero separaterd string */
+static char *strtozsv(const char *str, int sep, int *sp){
+ int size = strlen(str);
+ char *buf = tcmemdup(str, size);
+ for(int i = 0; i < size; i++){
+   if(buf[i] == sep) buf[i] = '\0';
+ }
+ *sp = size;
+ return buf;
+}
+
+
 /* print error information */
 static void printerr(TCADB *adb){
-  fprintf(stderr, "%s: error\n", g_progname);
+  const char *path = tcadbpath(adb);
+  fprintf(stderr, "%s: %s: error\n", g_progname, path ? path : "-");
 }
 
 
 /* print record data */
-static int printdata(const char *ptr, int size, bool px){
+static int printdata(const char *ptr, int size, bool px, int sep){
   int len = 0;
   while(size-- > 0){
     if(px){
       if(len > 0) putchar(' ');
       len += printf("%02X", *(unsigned char *)ptr);
+    } else if(sep > 0){
+      if(*ptr == '\0'){
+        putchar(sep);
+      } else {
+        putchar(*ptr);
+      }
+      len++;
     } else {
       putchar(*ptr);
       len++;
@@ -112,24 +160,12 @@ static int printdata(const char *ptr, int size, bool px){
 }
 
 
-/* create a binary object from a hexadecimal string */
-static char *hextoobj(const char *str, int *sp){
-  int len = strlen(str);
-  char *buf = tcmalloc(len + 1);
-  int j = 0;
-  for(int i = 0; i < len; i += 2){
-    while(strchr(" \n\r\t\f\v", str[i])){
-      i++;
-    }
-    char mbuf[3];
-    if((mbuf[0] = str[i]) == '\0') break;
-    if((mbuf[1] = str[i+1]) == '\0') break;
-    mbuf[2] = '\0';
-    buf[j++] = (char)strtol(mbuf, NULL, 16);
-  }
-  buf[j] = '\0';
-  *sp = j;
-  return buf;
+/* mapping function */
+static bool mapbdbproc(void *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
+                       void *op){
+  bool err = false;
+  if(!tcadbmapbdbemit(map, kbuf, ksiz, vbuf, vsiz)) err = true;
+  return !err;
 }
 
 
@@ -178,14 +214,22 @@ static int runput(int argc, char **argv){
   char *value = NULL;
   int dmode = 0;
   bool sx = false;
+  int sep = -1;
   for(int i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-dk")){
         dmode = -1;
       } else if(!strcmp(argv[i], "-dc")){
         dmode = 1;
+      } else if(!strcmp(argv[i], "-dai")){
+        dmode = 10;
+      } else if(!strcmp(argv[i], "-dad")){
+        dmode = 11;
       } else if(!strcmp(argv[i], "-sx")){
         sx = true;
+      } else if(!strcmp(argv[i], "-sep")){
+        if(++i >= argc) usage();
+        sep = sepstrtochr(argv[i]);
       } else {
         usage();
       }
@@ -200,11 +244,14 @@ static int runput(int argc, char **argv){
     }
   }
   if(!name || !key || !value) usage();
-  int ksiz, vsiz;
   char *kbuf, *vbuf;
+  int ksiz, vsiz;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
-    vbuf = hextoobj(value, &vsiz);
+    kbuf = tchexdecode(key, &ksiz);
+    vbuf = tchexdecode(value, &vsiz);
+  } else if(sep > 0){
+    kbuf = strtozsv(key, sep, &ksiz);
+    vbuf = strtozsv(value, sep, &vsiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
@@ -223,10 +270,14 @@ static int runout(int argc, char **argv){
   char *name = NULL;
   char *key = NULL;
   bool sx = false;
+  int sep = -1;
   for(int i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-sx")){
         sx = true;
+      } else if(!strcmp(argv[i], "-sep")){
+        if(++i >= argc) usage();
+        sep = sepstrtochr(argv[i]);
       } else {
         usage();
       }
@@ -242,7 +293,9 @@ static int runout(int argc, char **argv){
   int ksiz;
   char *kbuf;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
+    kbuf = tchexdecode(key, &ksiz);
+  } else if(sep > 0){
+    kbuf = strtozsv(key, sep, &ksiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
@@ -258,12 +311,16 @@ static int runget(int argc, char **argv){
   char *name = NULL;
   char *key = NULL;
   bool sx = false;
+  int sep = -1;
   bool px = false;
   bool pz = false;
   for(int i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-sx")){
         sx = true;
+      } else if(!strcmp(argv[i], "-sep")){
+        if(++i >= argc) usage();
+        sep = sepstrtochr(argv[i]);
       } else if(!strcmp(argv[i], "-px")){
         px = true;
       } else if(!strcmp(argv[i], "-pz")){
@@ -283,13 +340,15 @@ static int runget(int argc, char **argv){
   int ksiz;
   char *kbuf;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
+    kbuf = tchexdecode(key, &ksiz);
+  } else if(sep > 0){
+    kbuf = strtozsv(key, sep, &ksiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
   }
   name = tcsprintf("%s#mode=r", name);
-  int rv = procget(name, kbuf, ksiz, px, pz);
+  int rv = procget(name, kbuf, ksiz, sep, px, pz);
   tcfree(name);
   tcfree(kbuf);
   return rv;
@@ -299,15 +358,19 @@ static int runget(int argc, char **argv){
 /* parse arguments of list command */
 static int runlist(int argc, char **argv){
   char *name = NULL;
+  int sep = -1;
   int max = -1;
   bool pv = false;
   bool px = false;
   char *fmstr = NULL;
   for(int i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-m")){
+      if(!strcmp(argv[i], "-sep")){
         if(++i >= argc) usage();
-        max = atoi(argv[i]);
+        sep = sepstrtochr(argv[i]);
+      } else if(!strcmp(argv[i], "-m")){
+        if(++i >= argc) usage();
+        max = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-pv")){
         pv = true;
       } else if(!strcmp(argv[i], "-px")){
@@ -326,7 +389,103 @@ static int runlist(int argc, char **argv){
   }
   if(!name) usage();
   name = tcsprintf("%s#mode=r", name);
-  int rv = proclist(name, max, pv, px, fmstr);
+  int rv = proclist(name, sep, max, pv, px, fmstr);
+  tcfree(name);
+  return rv;
+}
+
+
+/* parse arguments of optimize command */
+static int runoptimize(int argc, char **argv){
+  char *name = NULL;
+  char *params = NULL;
+  for(int i = 2; i < argc; i++){
+    if(!name && argv[i][0] == '-'){
+      usage();
+    } else if(!name){
+      name = argv[i];
+    } else if(!params){
+      params = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!name) usage();
+  int rv = procoptimize(name, params);
+  return rv;
+}
+
+
+/* parse arguments of misc command */
+static int runmisc(int argc, char **argv){
+  char *name = NULL;
+  char *func = NULL;
+  TCLIST *args = tcmpoollistnew(tcmpoolglobal());
+  bool sx = false;
+  int sep = -1;
+  bool px = false;
+  for(int i = 2; i < argc; i++){
+    if(!name && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-sx")){
+        sx = true;
+      } else if(!strcmp(argv[i], "-sep")){
+        if(++i >= argc) usage();
+        sep = sepstrtochr(argv[i]);
+      } else if(!strcmp(argv[i], "-px")){
+        px = true;
+      } else {
+        usage();
+      }
+    } else if(!name){
+      name = argv[i];
+    } else if(!func){
+      func = argv[i];
+    } else {
+      if(sx){
+        int size;
+        char *buf = tchexdecode(argv[i], &size);
+        tclistpush(args, buf, size);
+        tcfree(buf);
+      } else if(sep > 0){
+        int size;
+        char *buf = strtozsv(argv[i], sep, &size);
+        tclistpush(args, buf, size);
+        tcfree(buf);
+      } else {
+        tclistpush2(args, argv[i]);
+      }
+    }
+  }
+  if(!name || !func) usage();
+  int rv = procmisc(name, func, args, sep, px);
+  return rv;
+}
+
+
+/* parse arguments of map command */
+static int runmap(int argc, char **argv){
+  char *name = NULL;
+  char *dest = NULL;
+  char *fmstr = NULL;
+  for(int i = 2; i < argc; i++){
+    if(!name && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-fm")){
+        if(++i >= argc) usage();
+        fmstr = argv[i];
+      } else {
+        usage();
+      }
+    } else if(!name){
+      name = argv[i];
+    } else if(!dest){
+      dest = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!name || !dest) usage();
+  name = tcsprintf("%s#mode=r", name);
+  int rv = procmap(name, dest, fmstr);
   tcfree(name);
   return rv;
 }
@@ -366,6 +525,20 @@ static int procinform(const char *name){
     return 1;
   }
   bool err = false;
+  const char *path = tcadbpath(adb);
+  if(!path) path = "(unknown)";
+  printf("path: %s\n", path);
+  const char *type = "(unknown)";
+  switch(tcadbomode(adb)){
+  case ADBOVOID: type = "not opened"; break;
+  case ADBOMDB: type = "on-memory hash database"; break;
+  case ADBONDB: type = "on-memory tree database"; break;
+  case ADBOHDB: type = "hash database"; break;
+  case ADBOBDB: type = "B+ tree database"; break;
+  case ADBOFDB: type = "fixed-length database"; break;
+  case ADBOTDB: type = "table database"; break;
+  }
+  printf("database type: %s\n", type);
   printf("record number: %llu\n", (unsigned long long)tcadbrnum(adb));
   printf("size: %llu\n", (unsigned long long)tcadbsize(adb));
   if(!tcadbclose(adb)){
@@ -400,6 +573,18 @@ static int procput(const char *name, const char *kbuf, int ksiz, const char *vbu
       err = true;
     }
     break;
+  case 10:
+    if(tcadbaddint(adb, kbuf, ksiz, tcatoi(vbuf)) == INT_MIN){
+      printerr(adb);
+      err = true;
+    }
+    break;
+  case 11:
+    if(isnan(tcadbadddouble(adb, kbuf, ksiz, tcatof(vbuf)))){
+      printerr(adb);
+      err = true;
+    }
+    break;
   default:
     if(!tcadbput(adb, kbuf, ksiz, vbuf, vsiz)){
       printerr(adb);
@@ -439,7 +624,7 @@ static int procout(const char *name, const char *kbuf, int ksiz){
 
 
 /* perform get command */
-static int procget(const char *name, const char *kbuf, int ksiz, bool px, bool pz){
+static int procget(const char *name, const char *kbuf, int ksiz, int sep, bool px, bool pz){
   TCADB *adb = tcadbnew();
   if(!tcadbopen(adb, name)){
     printerr(adb);
@@ -450,7 +635,7 @@ static int procget(const char *name, const char *kbuf, int ksiz, bool px, bool p
   int vsiz;
   char *vbuf = tcadbget(adb, kbuf, ksiz, &vsiz);
   if(vbuf){
-    printdata(vbuf, vsiz, px);
+    printdata(vbuf, vsiz, px, sep);
     if(!pz) putchar('\n');
     tcfree(vbuf);
   } else {
@@ -467,7 +652,7 @@ static int procget(const char *name, const char *kbuf, int ksiz, bool px, bool p
 
 
 /* perform list command */
-static int proclist(const char *name, int max, bool pv, bool px, const char *fmstr){
+static int proclist(const char *name, int sep, int max, bool pv, bool px, const char *fmstr){
   TCADB *adb = tcadbnew();
   if(!tcadbopen(adb, name)){
     printerr(adb);
@@ -480,13 +665,13 @@ static int proclist(const char *name, int max, bool pv, bool px, const char *fms
     for(int i = 0; i < tclistnum(keys); i++){
       int ksiz;
       const char *kbuf = tclistval(keys, i, &ksiz);
-      printdata(kbuf, ksiz, px);
+      printdata(kbuf, ksiz, px, sep);
       if(pv){
         int vsiz;
         char *vbuf = tcadbget(adb, kbuf, ksiz, &vsiz);
         if(vbuf){
           putchar('\t');
-          printdata(vbuf, vsiz, px);
+          printdata(vbuf, vsiz, px, sep);
           tcfree(vbuf);
         }
       }
@@ -502,13 +687,13 @@ static int proclist(const char *name, int max, bool pv, bool px, const char *fms
     char *kbuf;
     int cnt = 0;
     while((kbuf = tcadbiternext(adb, &ksiz)) != NULL){
-      printdata(kbuf, ksiz, px);
+      printdata(kbuf, ksiz, px, sep);
       if(pv){
         int vsiz;
         char *vbuf = tcadbget(adb, kbuf, ksiz, &vsiz);
         if(vbuf){
           putchar('\t');
-          printdata(vbuf, vsiz, px);
+          printdata(vbuf, vsiz, px, sep);
           tcfree(vbuf);
         }
       }
@@ -526,10 +711,107 @@ static int proclist(const char *name, int max, bool pv, bool px, const char *fms
 }
 
 
+/* perform optimize command */
+static int procoptimize(const char *name, const char *params){
+  TCADB *adb = tcadbnew();
+  if(!tcadbopen(adb, name)){
+    printerr(adb);
+    tcadbdel(adb);
+    return 1;
+  }
+  bool err = false;
+  if(!tcadboptimize(adb, params)){
+    printerr(adb);
+    err = true;
+  }
+  if(!tcadbclose(adb)){
+    if(!err) printerr(adb);
+    err = true;
+  }
+  tcadbdel(adb);
+  return err ? 1 : 0;
+}
+
+
+/* perform misc command */
+static int procmisc(const char *name, const char *func, const TCLIST *args, int sep, bool px){
+  TCADB *adb = tcadbnew();
+  if(!tcadbopen(adb, name)){
+    printerr(adb);
+    tcadbdel(adb);
+    return 1;
+  }
+  bool err = false;
+  TCLIST *res = tcadbmisc(adb, func, args);
+  if(res){
+    for(int i = 0; i < tclistnum(res); i++){
+      int rsiz;
+      const char *rbuf = tclistval(res, i, &rsiz);
+      printdata(rbuf, rsiz, px, sep);
+      printf("\n");
+    }
+    tclistdel(res);
+  } else {
+    printerr(adb);
+    err = true;
+  }
+  if(!tcadbclose(adb)){
+    if(!err) printerr(adb);
+    err = true;
+  }
+  tcadbdel(adb);
+  return err ? 1 : 0;
+}
+
+
+/* perform map command */
+static int procmap(const char *name, const char *dest, const char *fmstr){
+  TCADB *adb = tcadbnew();
+  if(!tcadbopen(adb, name)){
+    printerr(adb);
+    tcadbdel(adb);
+    return 1;
+  }
+  bool err = false;
+  TCBDB *bdb = tcbdbnew();
+  if(!tcbdbopen(bdb, dest, BDBOWRITER | BDBOCREAT | BDBOTRUNC)){
+    printerr(adb);
+    tcbdbdel(bdb);
+    tcadbdel(adb);
+    return 1;
+  }
+  if(fmstr){
+    TCLIST *keys = tcadbfwmkeys2(adb, fmstr, -1);
+    if(!tcadbmapbdb(adb, keys, bdb, mapbdbproc, NULL, -1)){
+      printerr(adb);
+      err = true;
+    }
+    tclistdel(keys);
+  } else {
+    if(!tcadbmapbdb(adb, NULL, bdb, mapbdbproc, NULL, -1)){
+      printerr(adb);
+      err = true;
+    }
+  }
+  if(!tcbdbclose(bdb)){
+    printerr(adb);
+    err = true;
+  }
+  tcbdbdel(bdb);
+  if(!tcadbclose(adb)){
+    printerr(adb);
+    err = true;
+  }
+  tcadbdel(adb);
+  return err ? 1 : 0;
+}
+
+
 /* perform version command */
 static int procversion(void){
-  printf("Tokyo Cabinet version %s (%d:%s)\n", tcversion, _TC_LIBVER, _TC_FORMATVER);
-  printf("Copyright (C) 2006-2008 Mikio Hirabayashi\n");
+  printf("Tokyo Cabinet version %s (%d:%s) for %s\n",
+         tcversion, _TC_LIBVER, _TC_FORMATVER, TCSYSNAME);
+  printf("Copyright (C) 2006-2009 Mikio Hirabayashi\n");
   return 0;
 }
 
