@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The B+ tree database API of Tokyo Cabinet
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -31,6 +31,8 @@ __TCBDB_CLINKAGEBEGIN
 #include <stdbool.h>
 #include <stdint.h>
 #include <time.h>
+#include <limits.h>
+#include <math.h>
 #include <tcutil.h>
 #include <tchdb.h>
 
@@ -41,30 +43,9 @@ __TCBDB_CLINKAGEBEGIN
  *************************************************************************************************/
 
 
-/* type of the pointer to a comparison function.
-   `aptr' specifies the pointer to the region of one key.
-   `asiz' specifies the size of the region of one key.
-   `bptr' specifies the pointer to the region of the other key.
-   `bsiz' specifies the size of the region of the other key.
-   `op' specifies the pointer to the optional opaque object.
-   The return value is positive if the former is big, negative if the latter is big, 0 if both
-   are equivalent. */
-typedef int (*BDBCMP)(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
-
-/* type of the pointer to a encoding or decoding function.
-   `ptr' specifies the pointer to the region.
-   `size' specifies the size of the region.
-   `sp' specifies the pointer to the variable into which the size of the region of the return
-   value is assigned.
-   `op' specifies the pointer to the optional opaque object.
-   If successful, the return value is the pointer to the result object allocated with `malloc'
-   call, else, it is `NULL'. */
-typedef void *(*BDBCODEC)(const void *ptr, int size, int *sp, void *op);
-
 typedef struct {                         /* type of structure for a B+ tree database */
   void *mmtx;                            /* mutex for method */
   void *cmtx;                            /* mutex for cache */
-  void *tmtx;                            /* mutex for transaction */
   TCHDB *hdb;                            /* internal database object */
   char *opaque;                          /* opaque buffer */
   bool open;                             /* whether the internal database is opened */
@@ -80,11 +61,11 @@ typedef struct {                         /* type of structure for a B+ tree data
   uint64_t rnum;                         /* number of records */
   TCMAP *leafc;                          /* cache for leaves */
   TCMAP *nodec;                          /* cache for nodes */
-  BDBCMP cmp;                            /* pointer to the comparison function */
+  TCCMP cmp;                             /* pointer to the comparison function */
   void *cmpop;                           /* opaque object for the comparison function */
-  uint32_t lcnum;                        /* max number of cached leaves */
-  uint32_t ncnum;                        /* max number of cached nodes */
-  uint32_t lsmax;                        /* max size of each leaf */
+  uint32_t lcnum;                        /* maximum number of cached leaves */
+  uint32_t ncnum;                        /* maximum number of cached nodes */
+  uint32_t lsmax;                        /* maximum size of each leaf */
   uint32_t lschk;                        /* counter for leaf size checking */
   uint64_t capnum;                       /* capacity number of records */
   uint64_t *hist;                        /* history array of visited nodes */
@@ -93,6 +74,7 @@ typedef struct {                         /* type of structure for a B+ tree data
   uint64_t lleaf;                        /* ID number of the last visited leaf */
   bool tran;                             /* whether in the transaction */
   char *rbopaque;                        /* opaque for rollback */
+  uint64_t clock;                        /* logical clock */
   int64_t cnt_saveleaf;                  /* tesing counter for leaf save times */
   int64_t cnt_loadleaf;                  /* tesing counter for leaf load times */
   int64_t cnt_killleaf;                  /* tesing counter for leaf kill times */
@@ -121,11 +103,13 @@ enum {                                   /* enumeration for open modes */
   BDBOCREAT = 1 << 2,                    /* writer creating */
   BDBOTRUNC = 1 << 3,                    /* writer truncating */
   BDBONOLCK = 1 << 4,                    /* open without locking */
-  BDBOLCKNB = 1 << 5                     /* lock without blocking */
+  BDBOLCKNB = 1 << 5,                    /* lock without blocking */
+  BDBOTSYNC = 1 << 6                     /* synchronize every transaction */
 };
 
 typedef struct {                         /* type of structure for a B+ tree cursor */
   TCBDB *bdb;                            /* database object */
+  uint64_t clock;                        /* logical clock */
   uint64_t id;                           /* ID number of the leaf */
   int32_t kidx;                          /* number of the key */
   int32_t vidx;                          /* number of the value */
@@ -159,7 +143,7 @@ void tcbdbdel(TCBDB *bdb);
 /* Get the last happened error code of a B+ tree database object.
    `bdb' specifies the B+ tree database object.
    The return value is the last happened error code.
-   The following error code is defined: `TCESUCCESS' for success, `TCETHREAD' for threading
+   The following error codes are defined: `TCESUCCESS' for success, `TCETHREAD' for threading
    error, `TCEINVALID' for invalid operation, `TCENOFILE' for file not found, `TCENOPERM' for no
    permission, `TCEMETA' for invalid meta data, `TCERHEAD' for invalid record header, `TCEOPEN'
    for open error, `TCECLOSE' for close error, `TCETRUNC' for trunc error, `TCESYNC' for sync
@@ -181,16 +165,20 @@ bool tcbdbsetmutex(TCBDB *bdb);
 
 /* Set the custom comparison function of a B+ tree database object.
    `bdb' specifies the B+ tree database object which is not opened.
-   `cmp' specifies the pointer to the custom comparison function.
+   `cmp' specifies the pointer to the custom comparison function.  It receives five parameters.
+   The first parameter is the pointer to the region of one key.  The second parameter is the size
+   of the region of one key.  The third parameter is the pointer to the region of the other key.
+   The fourth parameter is the size of the region of the other key.  The fifth parameter is the
+   pointer to the optional opaque object.  It returns positive if the former is big, negative if
+   the latter is big, 0 if both are equivalent.
    `cmpop' specifies an arbitrary pointer to be given as a parameter of the comparison function.
    If it is not needed, `NULL' can be specified.
    If successful, the return value is true, else, it is false.
    The default comparison function compares keys of two records by lexical order.  The functions
-   `tcbdbcmplexical' (dafault), `tcbdbcmpdecimal', `tcbdbcmpint32', and `tcbdbcmpint64' are
-   built-in.  Note that the comparison function should be set before the database is opened.
-   Moreover, user-defined comparison functions should be set every time the database is being
-   opened. */
-bool tcbdbsetcmpfunc(TCBDB *bdb, BDBCMP cmp, void *cmpop);
+   `tctccmplexical' (dafault), `tctccmpdecimal', `tctccmpint32', and `tctccmpint64' are built-in.
+   Note that the comparison function should be set before the database is opened.  Moreover,
+   user-defined comparison functions should be set every time the database is being opened. */
+bool tcbdbsetcmpfunc(TCBDB *bdb, TCCMP cmp, void *cmpop);
 
 
 /* Set the tuning parameters of a B+ tree database object.
@@ -200,13 +188,13 @@ bool tcbdbsetcmpfunc(TCBDB *bdb, BDBCMP cmp, void *cmpop);
    `nmemb' specifies the number of members in each non-leaf page.  If it is not more than 0, the
    default value is specified.  The default value is 256.
    `bnum' specifies the number of elements of the bucket array.  If it is not more than 0, the
-   default value is specified.  The default value is 16381.  Suggested size of the bucket array
+   default value is specified.  The default value is 32749.  Suggested size of the bucket array
    is about from 1 to 4 times of the number of all pages to be stored.
    `apow' specifies the size of record alignment by power of 2.  If it is negative, the default
    value is specified.  The default value is 8 standing for 2^8=256.
    `fpow' specifies the maximum number of elements of the free block pool by power of 2.  If it
    is negative, the default value is specified.  The default value is 10 standing for 2^10=1024.
-   `opts' specifies options by bitwise or: `BDBTLARGE' specifies that the size of the database
+   `opts' specifies options by bitwise-or: `BDBTLARGE' specifies that the size of the database
    can be larger than 2GB by using 64-bit bucket array, `BDBTDEFLATE' specifies that each page
    is compressed with Deflate encoding, `BDBTBZIP' specifies that each page is compressed with
    BZIP2 encoding, `BDBTTCBS' specifies that each page is compressed with TCBS encoding.
@@ -227,14 +215,33 @@ bool tcbdbtune(TCBDB *bdb, int32_t lmemb, int32_t nmemb,
 bool tcbdbsetcache(TCBDB *bdb, int32_t lcnum, int32_t ncnum);
 
 
+/* Set the size of the extra mapped memory of a B+ tree database object.
+   `bdb' specifies the B+ tree database object which is not opened.
+   `xmsiz' specifies the size of the extra mapped memory.  If it is not more than 0, the extra
+   mapped memory is disabled.  It is disabled by default.
+   If successful, the return value is true, else, it is false.
+   Note that the mapping parameters should be set before the database is opened. */
+bool tcbdbsetxmsiz(TCBDB *bdb, int64_t xmsiz);
+
+
+/* Set the unit step number of auto defragmentation of a B+ tree database object.
+   `bdb' specifies the B+ tree database object which is not opened.
+   `dfunit' specifie the unit step number.  If it is not more than 0, the auto defragmentation
+   is disabled.  It is disabled by default.
+   If successful, the return value is true, else, it is false.
+   Note that the defragmentation parameter should be set before the database is opened. */
+bool tcbdbsetdfunit(TCBDB *bdb, int32_t dfunit);
+
+
 /* Open a database file and connect a B+ tree database object.
    `bdb' specifies the B+ tree database object which is not opened.
    `path' specifies the path of the database file.
    `omode' specifies the connection mode: `BDBOWRITER' as a writer, `BDBOREADER' as a reader.
-   If the mode is `BDBOWRITER', the following may be added by bitwise or: `BDBOCREAT', which
-   means it creates a new database if not exist, `BDBOTRUNC', which means it creates a new database
-   regardless if one exists.  Both of `BDBOREADER' and `BDBOWRITER' can be added to by
-   bitwise or: `BDBONOLCK', which means it opens the database file without file locking, or
+   If the mode is `BDBOWRITER', the following may be added by bitwise-or: `BDBOCREAT', which
+   means it creates a new database if not exist, `BDBOTRUNC', which means it creates a new
+   database regardless if one exists, `BDBOTSYNC', which means every transaction synchronizes
+   updated contents with the device.  Both of `BDBOREADER' and `BDBOWRITER' can be added to by
+   bitwise-or: `BDBONOLCK', which means it opens the database file without file locking, or
    `BDBOLCKNB', which means locking is performed without blocking.
    If successful, the return value is true, else, it is false. */
 bool tcbdbopen(TCBDB *bdb, const char *path, int omode);
@@ -518,6 +525,28 @@ TCLIST *tcbdbfwmkeys(TCBDB *bdb, const void *pbuf, int psiz, int max);
 TCLIST *tcbdbfwmkeys2(TCBDB *bdb, const char *pstr, int max);
 
 
+/* Add an integer to a record in a B+ tree database object.
+   `bdb' specifies the B+ tree database object connected as a writer.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `num' specifies the additional value.
+   If successful, the return value is the summation value, else, it is `INT_MIN'.
+   If the corresponding record exists, the value is treated as an integer and is added to.  If no
+   record corresponds, a new record of the additional value is stored. */
+int tcbdbaddint(TCBDB *bdb, const void *kbuf, int ksiz, int num);
+
+
+/* Add a real number to a record in a B+ tree database object.
+   `bdb' specifies the B+ tree database object connected as a writer.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `num' specifies the additional value.
+   If successful, the return value is the summation value, else, it is Not-a-Number.
+   If the corresponding record exists, the value is treated as a real number and is added to.  If
+   no record corresponds, a new record of the additional value is stored. */
+double tcbdbadddouble(TCBDB *bdb, const void *kbuf, int ksiz, double num);
+
+
 /* Synchronize updated contents of a B+ tree database object with the file and the device.
    `bdb' specifies the B+ tree database object connected as a writer.
    If successful, the return value is true, else, it is false.
@@ -537,7 +566,7 @@ bool tcbdbsync(TCBDB *bdb);
    setting is not changed.
    `fpow' specifies the maximum number of elements of the free block pool by power of 2.  If it
    is negative, the current setting is not changed.
-   `opts' specifies options by bitwise or: `BDBTLARGE' specifies that the size of the database
+   `opts' specifies options by bitwise-or: `BDBTLARGE' specifies that the size of the database
    can be larger than 2GB by using 64-bit bucket array, `BDBTDEFLATE' specifies that each record
    is compressed with Deflate encoding, `BDBTBZIP' specifies that each page is compressed with
    BZIP2 encoding, `BDBTTCBS' specifies that each page is compressed with TCBS encoding.  If it
@@ -705,7 +734,7 @@ bool tcbdbcurput(BDBCUR *cur, const void *vbuf, int vsiz, int cpmode);
 bool tcbdbcurput2(BDBCUR *cur, const char *vstr, int cpmode);
 
 
-/* Delete the record where a cursor object is.
+/* Remove the record where a cursor object is.
    `cur' specifies the cursor object of writer connection.
    If successful, the return value is true, else, it is false.  False is returned when the cursor
    is at invalid position.
@@ -723,7 +752,7 @@ bool tcbdbcurout(BDBCUR *cur);
    the return value can be treated as a character string.  Because the region of the return
    value is allocated with the `malloc' call, it should be released with the `free' call when
    it is no longer in use. */
-char *tcbdbcurkey(BDBCUR *cur, int *sp);
+void *tcbdbcurkey(BDBCUR *cur, int *sp);
 
 
 /* Get the key string of the record where the cursor object is.
@@ -745,7 +774,7 @@ char *tcbdbcurkey2(BDBCUR *cur);
    the return value can be treated as a character string.  Because the region of the return value
    is volatile and it may be spoiled by another operation of the database, the data should be
    copied into another involatile buffer immediately. */
-const char *tcbdbcurkey3(BDBCUR *cur, int *sp);
+const void *tcbdbcurkey3(BDBCUR *cur, int *sp);
 
 
 /* Get the value of the record where the cursor object is.
@@ -758,7 +787,7 @@ const char *tcbdbcurkey3(BDBCUR *cur, int *sp);
    the return value can be treated as a character string.  Because the region of the return
    value is allocated with the `malloc' call, it should be released with the `free' call when
    it is no longer in use. */
-char *tcbdbcurval(BDBCUR *cur, int *sp);
+void *tcbdbcurval(BDBCUR *cur, int *sp);
 
 
 /* Get the value string of the record where the cursor object is.
@@ -780,7 +809,7 @@ char *tcbdbcurval2(BDBCUR *cur);
    the return value can be treated as a character string.  Because the region of the return value
    is volatile and it may be spoiled by another operation of the database, the data should be
    copied into another involatile buffer immediately. */
-const char *tcbdbcurval3(BDBCUR *cur, int *sp);
+const void *tcbdbcurval3(BDBCUR *cur, int *sp);
 
 
 /* Get the key and the value of the record where the cursor object is.
@@ -819,6 +848,12 @@ void tcbdbsetdbgfd(TCBDB *bdb, int fd);
 int tcbdbdbgfd(TCBDB *bdb);
 
 
+/* Check whether mutual exclusion control is set to a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   If mutual exclusion control is set, it is true, else it is false. */
+bool tcbdbhasmutex(TCBDB *bdb);
+
+
 /* Synchronize updating contents on memory of a B+ tree database object.
    `bdb' specifies the B+ tree database object connected as a writer.
    `phys' specifies whether to synchronize physically.
@@ -835,7 +870,7 @@ bool tcbdbcacheclear(TCBDB *bdb);
 /* Get the comparison function of a B+ tree database object.
    `bdb' specifies the B+ tree database object.
    The return value is the pointer to the comparison function. */
-BDBCMP tcbdbcmpfunc(TCBDB *bdb);
+TCCMP tcbdbcmpfunc(TCBDB *bdb);
 
 
 /* Get the opaque object for the comparison function of a B+ tree database object.
@@ -893,15 +928,15 @@ uint32_t tcbdbfbpmax(TCBDB *bdb);
 
 /* Get the inode number of the database file of a B+ tree database object.
    `bdb' specifies the B+ tree database object.
-   The return value is the inode number of the database file or 0 the object does not connect to
-   any database file. */
+   The return value is the inode number of the database file or 0 if the object does not connect
+   to any database file. */
 uint64_t tcbdbinode(TCBDB *bdb);
 
 
 /* Get the modification time of the database file of a B+ tree database object.
    `bdb' specifies the B+ tree database object.
-   The return value is the inode number of the database file or 0 the object does not connect to
-   any database file. */
+   The return value is the inode number of the database file or 0 if the object does not connect
+   to any database file. */
 time_t tcbdbmtime(TCBDB *bdb);
 
 
@@ -932,8 +967,8 @@ uint64_t tcbdbbnumused(TCBDB *bdb);
 
 /* Set the maximum size of each leaf node.
    `bdb' specifies the B+ tree database object which is not opened.
-   `lsmax' specifies the maximum size of each leaf node.  If it is not more than 0, the maximum
-   size is unlimited.
+   `lsmax' specifies the maximum size of each leaf node.  If it is not more than 0, the default
+   value is specified.  The default value is 16386.
    If successful, the return value is true, else, it is false.
    Note that the tuning parameters of the database should be set before the database is opened. */
 bool tcbdbsetlsmax(TCBDB *bdb, uint32_t lsmax);
@@ -951,7 +986,12 @@ bool tcbdbsetcapnum(TCBDB *bdb, uint64_t capnum);
 
 /* Set the custom codec functions of a B+ tree database object.
    `bdb' specifies the B+ tree database object.
-   `enc' specifies the pointer to the custom encoding function.
+   `enc' specifies the pointer to the custom encoding function.  It receives four parameters.
+   The first parameter is the pointer to the region.  The second parameter is the size of the
+   region.  The third parameter is the pointer to the variable into which the size of the region
+   of the return value is assigned.  The fourth parameter is the pointer to the optional opaque
+   object.  It returns the pointer to the result object allocated with `malloc' call if
+   successful, else, it returns `NULL'.
    `encop' specifies an arbitrary pointer to be given as a parameter of the encoding function.
    If it is not needed, `NULL' can be specified.
    `dec' specifies the pointer to the custom decoding function.
@@ -960,7 +1000,21 @@ bool tcbdbsetcapnum(TCBDB *bdb, uint64_t capnum);
    If successful, the return value is true, else, it is false.
    Note that the custom codec functions should be set before the database is opened and should be
    set every time the database is being opened. */
-bool tcbdbsetcodecfunc(TCBDB *bdb, BDBCODEC enc, void *encop, BDBCODEC dec, void *decop);
+bool tcbdbsetcodecfunc(TCBDB *bdb, TCCODEC enc, void *encop, TCCODEC dec, void *decop);
+
+
+/* Get the unit step number of auto defragmentation of a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   The return value is the unit step number of auto defragmentation. */
+uint32_t tcbdbdfunit(TCBDB *bdb);
+
+
+/* Perform dynamic defragmentation of a B+ tree database object.
+   `bdb' specifies the B+ tree database object connected as a writer.
+   `step' specifie the number of steps.  If it is not more than 0, the whole file is defragmented
+   gradually without keeping a continuous lock.
+   If successful, the return value is true, else, it is false. */
+bool tcbdbdefrag(TCBDB *bdb, int64_t step);
 
 
 /* Store a new record into a B+ tree database object with backward duplication.
@@ -985,6 +1039,29 @@ bool tcbdbputdupback(TCBDB *bdb, const void *kbuf, int ksiz, const void *vbuf, i
 bool tcbdbputdupback2(TCBDB *bdb, const char *kstr, const char *vstr);
 
 
+/* Store a record into a B+ tree database object with a duplication handler.
+   `bdb' specifies the B+ tree database object connected as a writer.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `vbuf' specifies the pointer to the region of the value.  `NULL' means that record addition is
+   ommited if there is no corresponding record.
+   `vsiz' specifies the size of the region of the value.
+   `proc' specifies the pointer to the callback function to process duplication.  It receives
+   four parameters.  The first parameter is the pointer to the region of the value.  The second
+   parameter is the size of the region of the value.  The third parameter is the pointer to the
+   variable into which the size of the region of the return value is assigned.  The fourth
+   parameter is the pointer to the optional opaque object.  It returns the pointer to the result
+   object allocated with `malloc'.  It is released by the caller.  If it is `NULL', the record is
+   not modified.  If it is `(void *)-1', the record is removed.
+   `op' specifies an arbitrary pointer to be given as a parameter of the callback function.  If
+   it is not needed, `NULL' can be specified.
+   If successful, the return value is true, else, it is false.
+   Note that the callback function can not perform any database operation because the function
+   is called in the critical section guarded by the same locks of database operations. */
+bool tcbdbputproc(TCBDB *bdb, const void *kbuf, int ksiz, const void *vbuf, int vsiz,
+                  TCPDPROC proc, void *op);
+
+
 /* Move a cursor object to the rear of records corresponding a key.
    `cur' specifies the cursor object.
    `kbuf' specifies the pointer to the region of the key.
@@ -1006,52 +1083,20 @@ bool tcbdbcurjumpback(BDBCUR *cur, const void *kbuf, int ksiz);
 bool tcbdbcurjumpback2(BDBCUR *cur, const char *kstr);
 
 
-/* Compare two keys by lexical order.
-   `aptr' specifies the pointer to the region of one key.
-   `asiz' specifies the size of the region of one key.
-   `bptr' specifies the pointer to the region of the other key.
-   `bsiz' specifies the size of the region of the other key.
-   `op' specifies the pointer to the optional opaque object.
-   The return value is positive if the former is big, negative if the latter is big, 0 if both
-   are equivalent. */
-int tcbdbcmplexical(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
-
-
-/* Compare two keys as decimal strings of real numbers.
-   `aptr' specifies the pointer to the region of one key.
-   `asiz' specifies the size of the region of one key.
-   `bptr' specifies the pointer to the region of the other key.
-   `bsiz' specifies the size of the region of the other key.
-   `op' is ignored.
-   The return value is positive if the former is big, negative if the latter is big, 0 if both
-   are equivalent. */
-int tcbdbcmpdecimal(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
-
-
-/* Compare two keys as 32-bit integers in the native byte order.
-   `aptr' specifies the pointer to the region of one key.
-   `asiz' specifies the size of the region of one key.
-   `bptr' specifies the pointer to the region of the other key.
-   `bsiz' specifies the size of the region of the other key.
-   `op' is ignored.
-   The return value is positive if the former is big, negative if the latter is big, 0 if both
-   are equivalent. */
-int tcbdbcmpint32(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
-
-
-/* Compare two keys as 64-bit integers in the native byte order.
-   `aptr' specifies the pointer to the region of one key.
-   `asiz' specifies the size of the region of one key.
-   `bptr' specifies the pointer to the region of the other key.
-   `bsiz' specifies the size of the region of the other key.
-   `op' is ignored.
-   The return value is positive if the former is big, negative if the latter is big, 0 if both
-   are equivalent. */
-int tcbdbcmpint64(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
-
-
-/* tricks for backward compatibility */
-#define tcbdbrange3 tcbdbfwmkeys2
+/* Process each record atomically of a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   `iter' specifies the pointer to the iterator function called for each record.  It receives
+   five parameters.  The first parameter is the pointer to the region of the key.  The second
+   parameter is the size of the region of the key.  The third parameter is the pointer to the
+   region of the value.  The fourth parameter is the size of the region of the value.  The fifth
+   parameter is the pointer to the optional opaque object.  It returns true to continue iteration
+   or false to stop iteration.
+   `op' specifies an arbitrary pointer to be given as a parameter of the iterator function.  If
+   it is not needed, `NULL' can be specified.
+   If successful, the return value is true, else, it is false.
+   Note that the callback function can not perform any database operation because the function
+   is called in the critical section guarded by the same locks of database operations. */
+bool tcbdbforeach(TCBDB *bdb, TCITER iter, void *op);
 
 
 
