@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The command line utility of the B+ tree database API
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -29,7 +29,6 @@ int main(int argc, char **argv);
 static void usage(void);
 static void printerr(TCBDB *bdb);
 static int printdata(const char *ptr, int size, bool px);
-static char *hextoobj(const char *str, int *sp);
 static char *mygetline(FILE *ifp);
 static int mycmpfunc(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
 static int runcreate(int argc, char **argv);
@@ -42,17 +41,17 @@ static int runoptimize(int argc, char **argv);
 static int runimporttsv(int argc, char **argv);
 static int runversion(int argc, char **argv);
 static int proccreate(const char *path, int lmemb, int nmemb,
-                      int bnum, int apow, int fpow, BDBCMP cmp, int opts);
+                      int bnum, int apow, int fpow, TCCMP cmp, int opts);
 static int procinform(const char *path, int omode);
 static int procput(const char *path, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
-                   BDBCMP cmp, int omode, int dmode);
-static int procout(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int omode);
-static int procget(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int omode,
+                   TCCMP cmp, int omode, int dmode);
+static int procout(const char *path, const char *kbuf, int ksiz, TCCMP cmp, int omode);
+static int procget(const char *path, const char *kbuf, int ksiz, TCCMP cmp, int omode,
                    bool px, bool pz);
-static int proclist(const char *path, BDBCMP cmp, int omode, int max, bool pv, bool px, bool bk,
+static int proclist(const char *path, TCCMP cmp, int omode, int max, bool pv, bool px, bool bk,
                     const char *jstr, const char *bstr, const char *estr, const char *fmstr);
 static int procoptimize(const char *path, int lmemb, int nmemb,
-                        int bnum, int apow, int fpow, BDBCMP cmp, int opts, int omode);
+                        int bnum, int apow, int fpow, TCCMP cmp, int opts, int omode, bool df);
 static int procimporttsv(const char *path, const char *file, int omode, bool sc);
 static int procversion(void);
 
@@ -62,7 +61,7 @@ int main(int argc, char **argv){
   g_progname = argv[0];
   g_dbgfd = -1;
   const char *ebuf = getenv("TCDBGFD");
-  if(ebuf) g_dbgfd = atoi(ebuf);
+  if(ebuf) g_dbgfd = tcatoix(ebuf);
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "create")){
@@ -98,14 +97,14 @@ static void usage(void){
   fprintf(stderr, "  %s create [-cd|-ci|-cj] [-tl] [-td|-tb|-tt|-tx] path"
           " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "  %s inform [-nl|-nb] path\n", g_progname);
-  fprintf(stderr, "  %s put [-cd|-ci|-cj] [-nl|-nb] [-sx] [-dk|-dc|-dd|-db] path"
+  fprintf(stderr, "  %s put [-cd|-ci|-cj] [-nl|-nb] [-sx] [-dk|-dc|-dd|-db|-dai|-dad] path"
           " key value\n", g_progname);
   fprintf(stderr, "  %s out [-cd|-ci|-cj] [-nl|-nb] [-sx] path key\n", g_progname);
   fprintf(stderr, "  %s get [-cd|-ci|-cj] [-nl|-nb] [-sx] [-px] [-pz] path key\n", g_progname);
   fprintf(stderr, "  %s list [-cd|-ci|-cj] [-nl|-nb] [-m num] [-bk] [-pv] [-px] [-j str]"
           " [-rb bkey ekey] [-fm str] path\n", g_progname);
-  fprintf(stderr, "  %s optimize [-cd|-ci|-cj] [-tl] [-td|-tb|-tt|-tx] [-tz] [-nl|-nb] path"
-          " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
+  fprintf(stderr, "  %s optimize [-cd|-ci|-cj] [-tl] [-td|-tb|-tt|-tx] [-tz] [-nl|-nb] [-df]"
+          " path [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "  %s importtsv [-nl|-nb] [-sc] path [file]\n", g_progname);
   fprintf(stderr, "  %s version\n", g_progname);
   fprintf(stderr, "\n");
@@ -138,27 +137,6 @@ static int printdata(const char *ptr, int size, bool px){
 }
 
 
-/* create a binary object from a hexadecimal string */
-static char *hextoobj(const char *str, int *sp){
-  int len = strlen(str);
-  char *buf = tcmalloc(len + 1);
-  int j = 0;
-  for(int i = 0; i < len; i += 2){
-    while(strchr(" \n\r\t\f\v", str[i])){
-      i++;
-    }
-    char mbuf[3];
-    if((mbuf[0] = str[i]) == '\0') break;
-    if((mbuf[1] = str[i+1]) == '\0') break;
-    mbuf[2] = '\0';
-    buf[j++] = (char)strtol(mbuf, NULL, 16);
-  }
-  buf[j] = '\0';
-  *sp = j;
-  return buf;
-}
-
-
 /* read a line from a file descriptor */
 static char *mygetline(FILE *ifp){
   int len = 0;
@@ -200,16 +178,16 @@ static int runcreate(int argc, char **argv){
   char *bstr = NULL;
   char *astr = NULL;
   char *fstr = NULL;
-  BDBCMP cmp = NULL;
+  TCCMP cmp = NULL;
   int opts = 0;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-cd")){
-        cmp = tcbdbcmpdecimal;
+        cmp = tccmpdecimal;
       } else if(!strcmp(argv[i], "-ci")){
-        cmp = tcbdbcmpint32;
+        cmp = tccmpint32;
       } else if(!strcmp(argv[i], "-cj")){
-        cmp = tcbdbcmpint64;
+        cmp = tccmpint64;
       } else if(!strcmp(argv[i], "-tl")){
         opts |= BDBTLARGE;
       } else if(!strcmp(argv[i], "-td")){
@@ -240,11 +218,11 @@ static int runcreate(int argc, char **argv){
     }
   }
   if(!path) usage();
-  int lmemb = lmstr ? atoi(lmstr) : -1;
-  int nmemb = nmstr ? atoi(nmstr) : -1;
-  int bnum = bstr ? atoi(bstr) : -1;
-  int apow = astr ? atoi(astr) : -1;
-  int fpow = fstr ? atoi(fstr) : -1;
+  int lmemb = lmstr ? tcatoix(lmstr) : -1;
+  int nmemb = nmstr ? tcatoix(nmstr) : -1;
+  int bnum = bstr ? tcatoix(bstr) : -1;
+  int apow = astr ? tcatoix(astr) : -1;
+  int fpow = fstr ? tcatoix(fstr) : -1;
   int rv = proccreate(path, lmemb, nmemb, bnum, apow, fpow, cmp, opts);
   return rv;
 }
@@ -280,18 +258,18 @@ static int runput(int argc, char **argv){
   char *path = NULL;
   char *key = NULL;
   char *value = NULL;
-  BDBCMP cmp = NULL;
+  TCCMP cmp = NULL;
   int omode = 0;
   int dmode = 0;
   bool sx = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-cd")){
-        cmp = tcbdbcmpdecimal;
+        cmp = tccmpdecimal;
       } else if(!strcmp(argv[i], "-ci")){
-        cmp = tcbdbcmpint32;
+        cmp = tccmpint32;
       } else if(!strcmp(argv[i], "-cj")){
-        cmp = tcbdbcmpint64;
+        cmp = tccmpint64;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -304,6 +282,10 @@ static int runput(int argc, char **argv){
         dmode = 2;
       } else if(!strcmp(argv[i], "-db")){
         dmode = 3;
+      } else if(!strcmp(argv[i], "-dai")){
+        dmode = 10;
+      } else if(!strcmp(argv[i], "-dad")){
+        dmode = 11;
       } else if(!strcmp(argv[i], "-sx")){
         sx = true;
       } else {
@@ -320,11 +302,11 @@ static int runput(int argc, char **argv){
     }
   }
   if(!path || !key || !value) usage();
-  int ksiz, vsiz;
   char *kbuf, *vbuf;
+  int ksiz, vsiz;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
-    vbuf = hextoobj(value, &vsiz);
+    kbuf = tchexdecode(key, &ksiz);
+    vbuf = tchexdecode(value, &vsiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
@@ -342,17 +324,17 @@ static int runput(int argc, char **argv){
 static int runout(int argc, char **argv){
   char *path = NULL;
   char *key = NULL;
-  BDBCMP cmp = NULL;
+  TCCMP cmp = NULL;
   int omode = 0;
   bool sx = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-cd")){
-        cmp = tcbdbcmpdecimal;
+        cmp = tccmpdecimal;
       } else if(!strcmp(argv[i], "-ci")){
-        cmp = tcbdbcmpint32;
+        cmp = tccmpint32;
       } else if(!strcmp(argv[i], "-cj")){
-        cmp = tcbdbcmpint64;
+        cmp = tccmpint64;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -374,7 +356,7 @@ static int runout(int argc, char **argv){
   int ksiz;
   char *kbuf;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
+    kbuf = tchexdecode(key, &ksiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
@@ -389,7 +371,7 @@ static int runout(int argc, char **argv){
 static int runget(int argc, char **argv){
   char *path = NULL;
   char *key = NULL;
-  BDBCMP cmp = NULL;
+  TCCMP cmp = NULL;
   int omode = 0;
   bool sx = false;
   bool px = false;
@@ -397,11 +379,11 @@ static int runget(int argc, char **argv){
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-cd")){
-        cmp = tcbdbcmpdecimal;
+        cmp = tccmpdecimal;
       } else if(!strcmp(argv[i], "-ci")){
-        cmp = tcbdbcmpint32;
+        cmp = tccmpint32;
       } else if(!strcmp(argv[i], "-cj")){
-        cmp = tcbdbcmpint64;
+        cmp = tccmpint64;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -427,7 +409,7 @@ static int runget(int argc, char **argv){
   int ksiz;
   char *kbuf;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
+    kbuf = tchexdecode(key, &ksiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
@@ -441,7 +423,7 @@ static int runget(int argc, char **argv){
 /* parse arguments of list command */
 static int runlist(int argc, char **argv){
   char *path = NULL;
-  BDBCMP cmp = NULL;
+  TCCMP cmp = NULL;
   int omode = 0;
   int max = -1;
   bool pv = false;
@@ -454,18 +436,18 @@ static int runlist(int argc, char **argv){
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-cd")){
-        cmp = tcbdbcmpdecimal;
+        cmp = tccmpdecimal;
       } else if(!strcmp(argv[i], "-ci")){
-        cmp = tcbdbcmpint32;
+        cmp = tccmpint32;
       } else if(!strcmp(argv[i], "-cj")){
-        cmp = tcbdbcmpint64;
+        cmp = tccmpint64;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
         omode |= BDBOLCKNB;
       } else if(!strcmp(argv[i], "-m")){
         if(++i >= argc) usage();
-        max = atoi(argv[i]);
+        max = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-bk")){
         bk = true;
       } else if(!strcmp(argv[i], "-pv")){
@@ -506,17 +488,18 @@ static int runoptimize(int argc, char **argv){
   char *bstr = NULL;
   char *astr = NULL;
   char *fstr = NULL;
-  BDBCMP cmp = NULL;
+  TCCMP cmp = NULL;
   int opts = UINT8_MAX;
   int omode = 0;
+  bool df = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-cd")){
-        cmp = tcbdbcmpdecimal;
+        cmp = tccmpdecimal;
       } else if(!strcmp(argv[i], "-ci")){
-        cmp = tcbdbcmpint32;
+        cmp = tccmpint32;
       } else if(!strcmp(argv[i], "-cj")){
-        cmp = tcbdbcmpint64;
+        cmp = tccmpint64;
       } else if(!strcmp(argv[i], "-tl")){
         if(opts == UINT8_MAX) opts = 0;
         opts |= BDBTLARGE;
@@ -538,6 +521,8 @@ static int runoptimize(int argc, char **argv){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
         omode |= BDBOLCKNB;
+      } else if(!strcmp(argv[i], "-df")){
+        df = true;
       } else {
         usage();
       }
@@ -558,12 +543,12 @@ static int runoptimize(int argc, char **argv){
     }
   }
   if(!path) usage();
-  int lmemb = lmstr ? atoi(lmstr) : -1;
-  int nmemb = nmstr ? atoi(nmstr) : -1;
-  int bnum = bstr ? atoi(bstr) : -1;
-  int apow = astr ? atoi(astr) : -1;
-  int fpow = fstr ? atoi(fstr) : -1;
-  int rv = procoptimize(path, lmemb, nmemb, bnum, apow, fpow, cmp, opts, omode);
+  int lmemb = lmstr ? tcatoix(lmstr) : -1;
+  int nmemb = nmstr ? tcatoix(nmstr) : -1;
+  int bnum = bstr ? tcatoix(bstr) : -1;
+  int apow = astr ? tcatoix(astr) : -1;
+  int fpow = fstr ? tcatoix(fstr) : -1;
+  int rv = procoptimize(path, lmemb, nmemb, bnum, apow, fpow, cmp, opts, omode, df);
   return rv;
 }
 
@@ -608,7 +593,7 @@ static int runversion(int argc, char **argv){
 
 /* perform create command */
 static int proccreate(const char *path, int lmemb, int nmemb,
-                      int bnum, int apow, int fpow, BDBCMP cmp, int opts){
+                      int bnum, int apow, int fpow, TCCMP cmp, int opts){
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
@@ -654,15 +639,15 @@ static int procinform(const char *path, int omode){
   if(flags & BDBFOPEN) printf(" open");
   if(flags & BDBFFATAL) printf(" fatal");
   printf("\n");
-  BDBCMP cmp = tcbdbcmpfunc(bdb);
+  TCCMP cmp = tcbdbcmpfunc(bdb);
   printf("comparison function: ");
-  if(cmp == tcbdbcmplexical){
+  if(cmp == tccmplexical){
     printf("lexical");
-  } else if(cmp == tcbdbcmpdecimal){
+  } else if(cmp == tccmpdecimal){
     printf("decimal");
-  } else if(cmp == tcbdbcmpint32){
+  } else if(cmp == tccmpint32){
     printf("int32");
-  } else if(cmp == tcbdbcmpint64){
+  } else if(cmp == tccmpint64){
     printf("int64");
   } else {
     printf("custom");
@@ -702,7 +687,7 @@ static int procinform(const char *path, int omode){
 
 /* perform put command */
 static int procput(const char *path, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
-                   BDBCMP cmp, int omode, int dmode){
+                   TCCMP cmp, int omode, int dmode){
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
@@ -738,6 +723,18 @@ static int procput(const char *path, const char *kbuf, int ksiz, const char *vbu
       err = true;
     }
     break;
+  case 10:
+    if(tcbdbaddint(bdb, kbuf, ksiz, tcatoi(vbuf)) == INT_MIN){
+      printerr(bdb);
+      err = true;
+    }
+    break;
+  case 11:
+    if(isnan(tcbdbadddouble(bdb, kbuf, ksiz, tcatof(vbuf)))){
+      printerr(bdb);
+      err = true;
+    }
+    break;
   default:
     if(!tcbdbput(bdb, kbuf, ksiz, vbuf, vsiz)){
       printerr(bdb);
@@ -755,7 +752,7 @@ static int procput(const char *path, const char *kbuf, int ksiz, const char *vbu
 
 
 /* perform out command */
-static int procout(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int omode){
+static int procout(const char *path, const char *kbuf, int ksiz, TCCMP cmp, int omode){
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
@@ -780,7 +777,7 @@ static int procout(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int
 
 
 /* perform get command */
-static int procget(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int omode,
+static int procget(const char *path, const char *kbuf, int ksiz, TCCMP cmp, int omode,
                    bool px, bool pz){
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
@@ -812,7 +809,7 @@ static int procget(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int
 
 
 /* perform list command */
-static int proclist(const char *path, BDBCMP cmp, int omode, int max, bool pv, bool px, bool bk,
+static int proclist(const char *path, TCCMP cmp, int omode, int max, bool pv, bool px, bool bk,
                     const char *jstr, const char *bstr, const char *estr, const char *fmstr){
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
@@ -921,7 +918,7 @@ static int proclist(const char *path, BDBCMP cmp, int omode, int max, bool pv, b
 
 /* perform optimize command */
 static int procoptimize(const char *path, int lmemb, int nmemb,
-                        int bnum, int apow, int fpow, BDBCMP cmp, int opts, int omode){
+                        int bnum, int apow, int fpow, TCCMP cmp, int opts, int omode, bool df){
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
@@ -932,9 +929,16 @@ static int procoptimize(const char *path, int lmemb, int nmemb,
     return 1;
   }
   bool err = false;
-  if(!tcbdboptimize(bdb, lmemb, nmemb, bnum, apow, fpow, opts)){
-    printerr(bdb);
-    err = true;
+  if(df){
+    if(!tcbdbdefrag(bdb, INT64_MAX)){
+      printerr(bdb);
+      err = true;
+    }
+  } else {
+    if(!tcbdboptimize(bdb, lmemb, nmemb, bnum, apow, fpow, opts)){
+      printerr(bdb);
+      err = true;
+    }
   }
   if(!tcbdbclose(bdb)){
     if(!err) printerr(bdb);
@@ -972,7 +976,7 @@ static int procimporttsv(const char *path, const char *file, int omode, bool sc)
     }
     *pv = '\0';
     if(sc) tcstrtolower(line);
-    if(!tcbdbputdup2(bdb, line, pv + 1) && tcbdbecode(bdb) != TCEKEEP){
+    if(!tcbdbputdup2(bdb, line, pv + 1)){
       printerr(bdb);
       err = true;
     }
@@ -997,8 +1001,9 @@ static int procimporttsv(const char *path, const char *file, int omode, bool sc)
 
 /* perform version command */
 static int procversion(void){
-  printf("Tokyo Cabinet version %s (%d:%s)\n", tcversion, _TC_LIBVER, _TC_FORMATVER);
-  printf("Copyright (C) 2006-2008 Mikio Hirabayashi\n");
+  printf("Tokyo Cabinet version %s (%d:%s) for %s\n",
+         tcversion, _TC_LIBVER, _TC_FORMATVER, TCSYSNAME);
+  printf("Copyright (C) 2006-2009 Mikio Hirabayashi\n");
   return 0;
 }
 
