@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The test cases of the B+ tree database API
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -18,7 +18,7 @@
 #include <tcbdb.h>
 #include "myconf.h"
 
-#define RECBUFSIZ      32                // buffer for records
+#define RECBUFSIZ      48                // buffer for records
 
 typedef struct {                         // type of structure for write thread
   TCBDB *bdb;
@@ -58,9 +58,16 @@ typedef struct {                         // type of structure for typical thread
   int id;
 } TARGTYPICAL;
 
+typedef struct {                         // type of structure for race thread
+  TCBDB *bdb;
+  int rnum;
+  int id;
+} TARGRACE;
+
 
 /* global variables */
 const char *g_progname;                  // program name
+unsigned int g_randseed;                 // random seed
 int g_dbgfd;                             // debugging output
 
 
@@ -68,36 +75,47 @@ int g_dbgfd;                             // debugging output
 int main(int argc, char **argv);
 static void usage(void);
 static void iprintf(const char *format, ...);
+static void iputchar(int c);
 static void eprint(TCBDB *bdb, const char *func);
 static void mprint(TCBDB *bdb);
+static void sysprint(void);
 static int myrand(int range);
 static int myrandnd(int range);
+static bool iterfunc(const void *kbuf, int ksiz, const void *vbuf, int vsiz, void *op);
 static int runwrite(int argc, char **argv);
 static int runread(int argc, char **argv);
 static int runremove(int argc, char **argv);
 static int runwicked(int argc, char **argv);
 static int runtypical(int argc, char **argv);
+static int runrace(int argc, char **argv);
 static int procwrite(const char *path, int tnum, int rnum, int lmemb, int nmemb,
-                     int bnum, int apow, int fpow, int opts, int omode, bool rnd);
-static int procread(const char *path, int tnum, int omode, bool wb, bool rnd);
-static int procremove(const char *path, int tnum, int omode, bool rnd);
+                     int bnum, int apow, int fpow, int opts, int xmsiz, int dfunit, int omode,
+                     bool rnd);
+static int procread(const char *path, int tnum, int xmsiz, int dfunit, int omode,
+                    bool wb, bool rnd);
+static int procremove(const char *path, int tnum, int xmsiz, int dfunit, int omode, bool rnd);
 static int procwicked(const char *path, int tnum, int rnum, int opts, int omode, bool nc);
 static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmemb,
-                       int bnum, int apow, int fpow, int opts, int omode, bool nc, int rratio);
+                       int bnum, int apow, int fpow, int opts, int xmsiz, int dfunit, int omode,
+                       bool nc, int rratio);
+static int procrace(const char *path, int tnum, int rnum, int lmemb, int nmemb,
+                    int bnum, int apow, int fpow, int opts, int xmsiz, int dfunit, int omode);
 static void *threadwrite(void *targ);
 static void *threadread(void *targ);
 static void *threadremove(void *targ);
 static void *threadwicked(void *targ);
 static void *threadtypical(void *targ);
+static void *threadrace(void *targ);
 
 
 /* main routine */
 int main(int argc, char **argv){
   g_progname = argv[0];
-  g_dbgfd = -1;
-  const char *ebuf = getenv("TCDBGFD");
-  if(ebuf) g_dbgfd = atoi(ebuf);
-  srand((unsigned int)(tctime() * 1000) % UINT_MAX);
+  const char *ebuf = getenv("TCRNDSEED");
+  g_randseed = ebuf ? tcatoix(ebuf) : tctime() * 1000;
+  srand(g_randseed);
+  ebuf = getenv("TCDBGFD");
+  g_dbgfd = ebuf ? tcatoix(ebuf) : UINT16_MAX;
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "write")){
@@ -110,9 +128,18 @@ int main(int argc, char **argv){
     rv = runwicked(argc, argv);
   } else if(!strcmp(argv[1], "typical")){
     rv = runtypical(argc, argv);
+  } else if(!strcmp(argv[1], "race")){
+    rv = runrace(argc, argv);
   } else {
     usage();
   }
+  if(rv != 0){
+    printf("FAILED:");
+    for(int i = 0; i < argc; i++){
+      printf(" %s", argv[i]);
+    }
+    printf("\n\n");
+  }
   return rv;
 }
 
@@ -122,14 +149,17 @@ static void usage(void){
   fprintf(stderr, "%s: test cases of the B+ tree database API of Tokyo Cabinet\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s write [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-rnd] path tnum rnum"
-          " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
-  fprintf(stderr, "  %s read [-nl|-nb] [-wb] [-rnd] path tnum\n", g_progname);
-  fprintf(stderr, "  %s remove [-nl|-nb] [-rnd] path tnum\n", g_progname);
+  fprintf(stderr, "  %s write [-tl] [-td|-tb|-tt|-tx] [-xm num] [-df num] [-nl|-nb] [-rnd]"
+          " path tnum rnum [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
+  fprintf(stderr, "  %s read [-xm num] [-df num] [-nl|-nb] [-wb] [-rnd] path tnum\n",
+          g_progname);
+  fprintf(stderr, "  %s remove [-xm num] [-df num] [-nl|-nb] [-rnd] path tnum\n", g_progname);
   fprintf(stderr, "  %s wicked [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-nc] path tnum rnum\n",
           g_progname);
-  fprintf(stderr, "  %s typical [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-nc] [-rr] path tnum rnum"
-          " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
+  fprintf(stderr, "  %s typical [-tl] [-td|-tb|-tt|-tx] [-xm num] [-df num] [-nl|-nb]"
+          " [-nc] [-rr num] path tnum rnum [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
+  fprintf(stderr, "  %s race [-tl] [-td|-tb|-tt|-tx] [-xm num] [-df num] [-nl|-nb]"
+          " path tnum rnum [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -145,6 +175,13 @@ static void iprintf(const char *format, ...){
 }
 
 
+/* print a character and flush the buffer */
+static void iputchar(int c){
+  putchar(c);
+  fflush(stdout);
+}
+
+
 /* print error message of hash database */
 static void eprint(TCBDB *bdb, const char *func){
   const char *path = tcbdbpath(bdb);
@@ -184,12 +221,33 @@ static void mprint(TCBDB *bdb){
   iprintf("cnt_deferdrp: %lld\n", (long long)bdb->hdb->cnt_deferdrp);
   iprintf("cnt_flushdrp: %lld\n", (long long)bdb->hdb->cnt_flushdrp);
   iprintf("cnt_adjrecc: %lld\n", (long long)bdb->hdb->cnt_adjrecc);
+  iprintf("cnt_defrag: %lld\n", (long long)bdb->hdb->cnt_defrag);
+  iprintf("cnt_shiftrec: %lld\n", (long long)bdb->hdb->cnt_shiftrec);
+  iprintf("cnt_trunc: %lld\n", (long long)bdb->hdb->cnt_trunc);
+}
+
+
+/* print system information */
+static void sysprint(void){
+  TCMAP *info = tcsysinfo();
+  if(info){
+    tcmapiterinit(info);
+    const char *kbuf;
+    while((kbuf = tcmapiternext2(info)) != NULL){
+      iprintf("sys_%s: %s\n", kbuf, tcmapiterval2(kbuf));
+    }
+    tcmapdel(info);
+  }
 }
 
 
 /* get a random number */
 static int myrand(int range){
-  return (int)((double)range * rand() / (RAND_MAX + 1.0));
+  if(range < 2) return 0;
+  int high = (unsigned int)rand() >> 4;
+  int low = range * (rand() / (RAND_MAX + 1.0));
+  low &= (unsigned int)INT_MAX >> 4;
+  return (high + low) % range;
 }
 
 
@@ -200,6 +258,19 @@ static int myrandnd(int range){
 }
 
 
+/* iterator function */
+static bool iterfunc(const void *kbuf, int ksiz, const void *vbuf, int vsiz, void *op){
+  unsigned int sum = 0;
+  while(--ksiz >= 0){
+    sum += ((char *)kbuf)[ksiz];
+  }
+  while(--vsiz >= 0){
+    sum += ((char *)vbuf)[vsiz];
+  }
+  return myrand(100 + (sum & 0xff)) > 0;
+}
+
+
 /* parse arguments of write command */
 static int runwrite(int argc, char **argv){
   char *path = NULL;
@@ -211,6 +282,8 @@ static int runwrite(int argc, char **argv){
   char *astr = NULL;
   char *fstr = NULL;
   int opts = 0;
+  int xmsiz = -1;
+  int dfunit = 0;
   int omode = 0;
   bool rnd = false;
   for(int i = 2; i < argc; i++){
@@ -225,6 +298,12 @@ static int runwrite(int argc, char **argv){
         opts |= BDBTTCBS;
       } else if(!strcmp(argv[i], "-tx")){
         opts |= BDBTEXCODEC;
+      } else if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -255,15 +334,16 @@ static int runwrite(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
-  int lmemb = lmstr ? atoi(lmstr) : -1;
-  int nmemb = nmstr ? atoi(nmstr) : -1;
-  int bnum = bstr ? atoi(bstr) : -1;
-  int apow = astr ? atoi(astr) : -1;
-  int fpow = fstr ? atoi(fstr) : -1;
-  int rv = procwrite(path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, omode, rnd);
+  int lmemb = lmstr ? tcatoix(lmstr) : -1;
+  int nmemb = nmstr ? tcatoix(nmstr) : -1;
+  int bnum = bstr ? tcatoix(bstr) : -1;
+  int apow = astr ? tcatoix(astr) : -1;
+  int fpow = fstr ? tcatoix(fstr) : -1;
+  int rv = procwrite(path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, xmsiz, dfunit,
+                     omode, rnd);
   return rv;
 }
 
@@ -272,12 +352,20 @@ static int runwrite(int argc, char **argv){
 static int runread(int argc, char **argv){
   char *path = NULL;
   char *tstr = NULL;
+  int xmsiz = -1;
+  int dfunit = 0;
   int omode = 0;
   bool rnd = false;
   bool wb = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-nl")){
+      if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
         omode |= BDBOLCKNB;
@@ -297,9 +385,9 @@ static int runread(int argc, char **argv){
     }
   }
   if(!path || !tstr) usage();
-  int tnum = atoi(tstr);
+  int tnum = tcatoix(tstr);
   if(tnum < 1) usage();
-  int rv = procread(path, tnum, omode, wb, rnd);
+  int rv = procread(path, tnum, xmsiz, dfunit, omode, wb, rnd);
   return rv;
 }
 
@@ -308,11 +396,19 @@ static int runread(int argc, char **argv){
 static int runremove(int argc, char **argv){
   char *path = NULL;
   char *tstr = NULL;
+  int xmsiz = -1;
+  int dfunit = 0;
   int omode = 0;
   bool rnd = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-nl")){
+      if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
         omode |= BDBOLCKNB;
@@ -330,9 +426,9 @@ static int runremove(int argc, char **argv){
     }
   }
   if(!path || !tstr) usage();
-  int tnum = atoi(tstr);
+  int tnum = tcatoix(tstr);
   if(tnum < 1) usage();
-  int rv = procremove(path, tnum, omode, rnd);
+  int rv = procremove(path, tnum, xmsiz, dfunit, omode, rnd);
   return rv;
 }
 
@@ -377,8 +473,8 @@ static int runwicked(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
   int rv = procwicked(path, tnum, rnum, opts, omode, nc);
   return rv;
@@ -396,6 +492,8 @@ static int runtypical(int argc, char **argv){
   char *astr = NULL;
   char *fstr = NULL;
   int opts = 0;
+  int xmsiz = -1;
+  int dfunit = 0;
   int omode = 0;
   int rratio = -1;
   bool nc = false;
@@ -411,6 +509,12 @@ static int runtypical(int argc, char **argv){
         opts |= BDBTTCBS;
       } else if(!strcmp(argv[i], "-tx")){
         opts |= BDBTEXCODEC;
+      } else if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -419,7 +523,7 @@ static int runtypical(int argc, char **argv){
         nc = true;
       } else if(!strcmp(argv[i], "-rr")){
         if(++i >= argc) usage();
-        rratio = atoi(argv[i]);
+        rratio = tcatoix(argv[i]);
       } else {
         usage();
       }
@@ -444,25 +548,102 @@ static int runtypical(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
-  int lmemb = lmstr ? atoi(lmstr) : -1;
-  int nmemb = nmstr ? atoi(nmstr) : -1;
-  int bnum = bstr ? atoi(bstr) : -1;
-  int apow = astr ? atoi(astr) : -1;
-  int fpow = fstr ? atoi(fstr) : -1;
-  int rv = proctypical(path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, omode, nc, rratio);
+  int lmemb = lmstr ? tcatoix(lmstr) : -1;
+  int nmemb = nmstr ? tcatoix(nmstr) : -1;
+  int bnum = bstr ? tcatoix(bstr) : -1;
+  int apow = astr ? tcatoix(astr) : -1;
+  int fpow = fstr ? tcatoix(fstr) : -1;
+  int rv = proctypical(path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, xmsiz, dfunit,
+                       omode, nc, rratio);
+  return rv;
+}
+
+
+/* parse arguments of race command */
+static int runrace(int argc, char **argv){
+  char *path = NULL;
+  char *tstr = NULL;
+  char *rstr = NULL;
+  char *lmstr = NULL;
+  char *nmstr = NULL;
+  char *bstr = NULL;
+  char *astr = NULL;
+  char *fstr = NULL;
+  int opts = 0;
+  int xmsiz = -1;
+  int dfunit = 0;
+  int omode = 0;
+  for(int i = 2; i < argc; i++){
+    if(!path && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-tl")){
+        opts |= BDBTLARGE;
+      } else if(!strcmp(argv[i], "-td")){
+        opts |= BDBTDEFLATE;
+      } else if(!strcmp(argv[i], "-tb")){
+        opts |= BDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
+        opts |= BDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= BDBTEXCODEC;
+      } else if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-nl")){
+        omode |= BDBONOLCK;
+      } else if(!strcmp(argv[i], "-nb")){
+        omode |= BDBOLCKNB;
+      } else {
+        usage();
+      }
+    } else if(!path){
+      path = argv[i];
+    } else if(!tstr){
+      tstr = argv[i];
+    } else if(!rstr){
+      rstr = argv[i];
+    } else if(!lmstr){
+      lmstr = argv[i];
+    } else if(!nmstr){
+      nmstr = argv[i];
+    } else if(!bstr){
+      bstr = argv[i];
+    } else if(!astr){
+      astr = argv[i];
+    } else if(!fstr){
+      fstr = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!path || !tstr || !rstr) usage();
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
+  if(tnum < 1 || rnum < 1) usage();
+  int lmemb = lmstr ? tcatoix(lmstr) : -1;
+  int nmemb = nmstr ? tcatoix(nmstr) : -1;
+  int bnum = bstr ? tcatoix(bstr) : -1;
+  int apow = astr ? tcatoix(astr) : -1;
+  int fpow = fstr ? tcatoix(fstr) : -1;
+  int rv = procrace(path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, xmsiz, dfunit,
+                    omode);
   return rv;
 }
 
 
 /* perform write command */
 static int procwrite(const char *path, int tnum, int rnum, int lmemb, int nmemb,
-                     int bnum, int apow, int fpow, int opts, int omode, bool rnd){
-  iprintf("<Writing Test>\n  path=%s  tnum=%d  rnum=%d  lmemb=%d  nmemb=%d"
-          "  bnum=%d  apow=%d  fpow=%d  opts=%d  omode=%d  rnd=%d\n\n",
-          path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, omode, rnd);
+                     int bnum, int apow, int fpow, int opts, int xmsiz, int dfunit, int omode,
+                     bool rnd){
+  iprintf("<Writing Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  lmemb=%d  nmemb=%d"
+          "  bnum=%d  apow=%d  fpow=%d  opts=%d  xmsiz=%d  dfunit=%d  omode=%d  rnd=%d\n\n",
+          g_randseed, path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, xmsiz, dfunit,
+          omode, rnd);
   bool err = false;
   double stime = tctime();
   TCBDB *bdb = tcbdbnew();
@@ -479,6 +660,14 @@ static int procwrite(const char *path, int tnum, int rnum, int lmemb, int nmemb,
     eprint(bdb, "tcbdbtune");
     err = true;
   }
+  if(xmsiz >= 0 && !tcbdbsetxmsiz(bdb, xmsiz)){
+    eprint(bdb, "tcbdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tcbdbsetdfunit(bdb, dfunit)){
+    eprint(bdb, "tcbdbsetdfunit");
+    err = true;
+  }
   if(!tcbdbopen(bdb, path, BDBOWRITER | BDBOCREAT | BDBOTRUNC | omode)){
     eprint(bdb, "tcbdbopen");
     err = true;
@@ -517,6 +706,7 @@ static int procwrite(const char *path, int tnum, int rnum, int lmemb, int nmemb,
   iprintf("record number: %llu\n", (unsigned long long)tcbdbrnum(bdb));
   iprintf("size: %llu\n", (unsigned long long)tcbdbfsiz(bdb));
   mprint(bdb);
+  sysprint();
   if(!tcbdbclose(bdb)){
     eprint(bdb, "tcbdbclose");
     err = true;
@@ -529,9 +719,10 @@ static int procwrite(const char *path, int tnum, int rnum, int lmemb, int nmemb,
 
 
 /* perform read command */
-static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
-  iprintf("<Reading Test>\n  path=%s  tnum=%d  omode=%d  wb=%d  rnd=%d\n\n",
-          path, tnum, omode, wb, rnd);
+static int procread(const char *path, int tnum, int xmsiz, int dfunit, int omode,
+                    bool wb, bool rnd){
+  iprintf("<Reading Test>\n  seed=%u  path=%s  tnum=%d  xmsiz=%d  dfunit=%d  omode=%d"
+          "  wb=%d  rnd=%d\n\n", g_randseed, path, tnum, xmsiz, dfunit, omode, wb, rnd);
   bool err = false;
   double stime = tctime();
   TCBDB *bdb = tcbdbnew();
@@ -544,6 +735,14 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
     eprint(bdb, "tcbdbsetcodecfunc");
     err = true;
   }
+  if(xmsiz >= 0 && !tcbdbsetxmsiz(bdb, xmsiz)){
+    eprint(bdb, "tcbdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tcbdbsetdfunit(bdb, dfunit)){
+    eprint(bdb, "tcbdbsetdfunit");
+    err = true;
+  }
   if(!tcbdbopen(bdb, path, BDBOREADER | omode)){
     eprint(bdb, "tcbdbopen");
     err = true;
@@ -585,6 +784,7 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
   iprintf("record number: %llu\n", (unsigned long long)tcbdbrnum(bdb));
   iprintf("size: %llu\n", (unsigned long long)tcbdbfsiz(bdb));
   mprint(bdb);
+  sysprint();
   if(!tcbdbclose(bdb)){
     eprint(bdb, "tcbdbclose");
     err = true;
@@ -597,8 +797,9 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
 
 
 /* perform remove command */
-static int procremove(const char *path, int tnum, int omode, bool rnd){
-  iprintf("<Removing Test>\n  path=%s  tnum=%d  omode=%d  rnd=%d\n\n", path, tnum, omode, rnd);
+static int procremove(const char *path, int tnum, int xmsiz, int dfunit, int omode, bool rnd){
+  iprintf("<Removing Test>\n  seed=%u  path=%s  tnum=%d  xmsiz=%d  dfunit=%d  omode=%d"
+          "  rnd=%d\n\n", g_randseed, path, tnum, xmsiz, dfunit, omode, rnd);
   bool err = false;
   double stime = tctime();
   TCBDB *bdb = tcbdbnew();
@@ -611,6 +812,14 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
     eprint(bdb, "tcbdbsetcodecfunc");
     err = true;
   }
+  if(xmsiz >= 0 && !tcbdbsetxmsiz(bdb, xmsiz)){
+    eprint(bdb, "tcbdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tcbdbsetdfunit(bdb, dfunit)){
+    eprint(bdb, "tcbdbsetdfunit");
+    err = true;
+  }
   if(!tcbdbopen(bdb, path, BDBOWRITER | omode)){
     eprint(bdb, "tcbdbopen");
     err = true;
@@ -650,6 +859,7 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
   iprintf("record number: %llu\n", (unsigned long long)tcbdbrnum(bdb));
   iprintf("size: %llu\n", (unsigned long long)tcbdbfsiz(bdb));
   mprint(bdb);
+  sysprint();
   if(!tcbdbclose(bdb)){
     eprint(bdb, "tcbdbclose");
     err = true;
@@ -663,8 +873,8 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
 
 /* perform wicked command */
 static int procwicked(const char *path, int tnum, int rnum, int opts, int omode, bool nc){
-  iprintf("<Writing Test>\n  path=%s  tnum=%d  rnum=%d  opts=%d  omode=%d  nc=%d\n\n",
-          path, tnum, rnum, opts, omode, nc);
+  iprintf("<Writing Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  opts=%d  omode=%d  nc=%d\n\n",
+          g_randseed, path, tnum, rnum, opts, omode, nc);
   bool err = false;
   double stime = tctime();
   TCBDB *bdb = tcbdbnew();
@@ -737,7 +947,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
       int rsiz;
       char *rbuf = tcbdbget(bdb, kbuf, ksiz, &rsiz);
       if(vbuf){
-        putchar('.');
+        iputchar('.');
         if(!rbuf){
           eprint(bdb, "tcbdbget");
           err = true;
@@ -746,7 +956,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
           err = true;
         }
       } else {
-        putchar('*');
+        iputchar('*');
         if(rbuf || tcbdbecode(bdb) != TCENOREC){
           eprint(bdb, "(validation)");
           err = true;
@@ -761,6 +971,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
   iprintf("record number: %llu\n", (unsigned long long)tcbdbrnum(bdb));
   iprintf("size: %llu\n", (unsigned long long)tcbdbfsiz(bdb));
   mprint(bdb);
+  sysprint();
   if(!tcbdbclose(bdb)){
     eprint(bdb, "tcbdbclose");
     err = true;
@@ -774,10 +985,13 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
 
 /* perform typical command */
 static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmemb,
-                       int bnum, int apow, int fpow, int opts, int omode, bool nc, int rratio){
-  iprintf("<Typical Access Test>\n  path=%s  tnum=%d  rnum=%d  lmemb=%d  nmemb=%d"
-          "  bnum=%d  apow=%d  fpow=%d  opts=%d  omode=%d  nc=%d  rratio=%d\n\n",
-          path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, omode, nc, rratio);
+                       int bnum, int apow, int fpow, int opts, int xmsiz, int dfunit, int omode,
+                       bool nc, int rratio){
+  iprintf("<Typical Access Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  lmemb=%d  nmemb=%d"
+          "  bnum=%d  apow=%d  fpow=%d  opts=%d  xmsiz=%d  dfunit=%d  omode=%d"
+          "  nc=%d  rratio=%d\n\n",
+          g_randseed, path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, xmsiz, dfunit,
+          omode, nc, rratio);
   bool err = false;
   double stime = tctime();
   TCBDB *bdb = tcbdbnew();
@@ -794,6 +1008,14 @@ static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmem
     eprint(bdb, "tcbdbtune");
     err = true;
   }
+  if(xmsiz >= 0 && !tcbdbsetxmsiz(bdb, xmsiz)){
+    eprint(bdb, "tcbdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tcbdbsetdfunit(bdb, dfunit)){
+    eprint(bdb, "tcbdbsetdfunit");
+    err = true;
+  }
   if(!tcbdbopen(bdb, path, BDBOWRITER | BDBOCREAT | BDBOTRUNC | omode)){
     eprint(bdb, "tcbdbopen");
     err = true;
@@ -834,6 +1056,86 @@ static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmem
   iprintf("record number: %llu\n", (unsigned long long)tcbdbrnum(bdb));
   iprintf("size: %llu\n", (unsigned long long)tcbdbfsiz(bdb));
   mprint(bdb);
+  sysprint();
+  if(!tcbdbclose(bdb)){
+    eprint(bdb, "tcbdbclose");
+    err = true;
+  }
+  tcbdbdel(bdb);
+  iprintf("time: %.3f\n", tctime() - stime);
+  iprintf("%s\n\n", err ? "error" : "ok");
+  return err ? 1 : 0;
+}
+
+
+/* perform race command */
+static int procrace(const char *path, int tnum, int rnum, int lmemb, int nmemb,
+                    int bnum, int apow, int fpow, int opts, int xmsiz, int dfunit, int omode){
+  iprintf("<Race Condition Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  lmemb=%d  nmemb=%d"
+          "  bnum=%d  apow=%d  fpow=%d  opts=%d  xmsiz=%d  dfunit=%d  omode=%d\n\n",
+          g_randseed, path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts,
+          xmsiz, dfunit, omode);
+  bool err = false;
+  double stime = tctime();
+  TCBDB *bdb = tcbdbnew();
+  if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
+  if(!tcbdbsetmutex(bdb)){
+    eprint(bdb, "tcbdbsetmutex");
+    err = true;
+  }
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(bdb, "tcbdbsetcodecfunc");
+    err = true;
+  }
+  if(!tcbdbtune(bdb, lmemb, nmemb, bnum, apow, fpow, opts)){
+    eprint(bdb, "tcbdbtune");
+    err = true;
+  }
+  if(xmsiz >= 0 && !tcbdbsetxmsiz(bdb, xmsiz)){
+    eprint(bdb, "tcbdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tcbdbsetdfunit(bdb, dfunit)){
+    eprint(bdb, "tcbdbsetdfunit");
+    err = true;
+  }
+  if(!tcbdbopen(bdb, path, BDBOWRITER | BDBOCREAT | BDBOTRUNC | omode)){
+    eprint(bdb, "tcbdbopen");
+    err = true;
+  }
+  TARGRACE targs[tnum];
+  pthread_t threads[tnum];
+  if(tnum == 1){
+    targs[0].bdb = bdb;
+    targs[0].rnum = rnum;
+    targs[0].id = 0;
+    if(threadrace(targs) != NULL) err = true;
+  } else {
+    for(int i = 0; i < tnum; i++){
+      targs[i].bdb = bdb;
+      targs[i].rnum = rnum;
+      targs[i].id = i;
+      if(pthread_create(threads + i, NULL, threadrace, targs + i) != 0){
+        eprint(bdb, "pthread_create");
+        targs[i].id = -1;
+        err = true;
+      }
+    }
+    for(int i = 0; i < tnum; i++){
+      if(targs[i].id == -1) continue;
+      void *rv;
+      if(pthread_join(threads[i], &rv) != 0){
+        eprint(bdb, "pthread_join");
+        err = true;
+      } else if(rv){
+        err = true;
+      }
+    }
+  }
+  iprintf("record number: %llu\n", (unsigned long long)tcbdbrnum(bdb));
+  iprintf("size: %llu\n", (unsigned long long)tcbdbfsiz(bdb));
+  mprint(bdb);
+  sysprint();
   if(!tcbdbclose(bdb)){
     eprint(bdb, "tcbdbclose");
     err = true;
@@ -862,8 +1164,7 @@ static void *threadwrite(void *targ){
       break;
     }
     if(id <= 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -900,8 +1201,7 @@ static void *threadread(void *targ){
       tcfree(vbuf);
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -926,8 +1226,7 @@ static void *threadremove(void *targ){
       break;
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -955,7 +1254,7 @@ static void *threadwicked(void *targ){
     if(!nc) tcglobalmutexlock();
     switch(myrand(16)){
     case 0:
-      if(id == 0) putchar('0');
+      if(id == 0) iputchar('0');
       if(!tcbdbput(bdb, kbuf, ksiz, vbuf, vsiz)){
         eprint(bdb, "tcbdbput");
         err = true;
@@ -963,7 +1262,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapput(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 1:
-      if(id == 0) putchar('1');
+      if(id == 0) iputchar('1');
       if(!tcbdbput2(bdb, kbuf, vbuf)){
         eprint(bdb, "tcbdbput2");
         err = true;
@@ -971,7 +1270,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapput2(map, kbuf, vbuf);
       break;
     case 2:
-      if(id == 0) putchar('2');
+      if(id == 0) iputchar('2');
       if(!tcbdbputkeep(bdb, kbuf, ksiz, vbuf, vsiz) && tcbdbecode(bdb) != TCEKEEP){
         eprint(bdb, "tcbdbputkeep");
         err = true;
@@ -979,7 +1278,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputkeep(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 3:
-      if(id == 0) putchar('3');
+      if(id == 0) iputchar('3');
       if(!tcbdbputkeep2(bdb, kbuf, vbuf) && tcbdbecode(bdb) != TCEKEEP){
         eprint(bdb, "tcbdbputkeep2");
         err = true;
@@ -987,7 +1286,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputkeep2(map, kbuf, vbuf);
       break;
     case 4:
-      if(id == 0) putchar('4');
+      if(id == 0) iputchar('4');
       if(!tcbdbputcat(bdb, kbuf, ksiz, vbuf, vsiz)){
         eprint(bdb, "tcbdbputcat");
         err = true;
@@ -995,7 +1294,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputcat(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 5:
-      if(id == 0) putchar('5');
+      if(id == 0) iputchar('5');
       if(!tcbdbputcat2(bdb, kbuf, vbuf)){
         eprint(bdb, "tcbdbputcat2");
         err = true;
@@ -1003,7 +1302,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputcat2(map, kbuf, vbuf);
       break;
     case 6:
-      if(id == 0) putchar('6');
+      if(id == 0) iputchar('6');
       if(nc){
         if(!tcbdbputdup(bdb, kbuf, ksiz, vbuf, vsiz)){
           eprint(bdb, "tcbdbputdup");
@@ -1012,7 +1311,7 @@ static void *threadwicked(void *targ){
       }
       break;
     case 7:
-      if(id == 0) putchar('7');
+      if(id == 0) iputchar('7');
       if(nc){
         if(!tcbdbputdup2(bdb, kbuf, vbuf)){
           eprint(bdb, "tcbdbputdup2");
@@ -1021,8 +1320,8 @@ static void *threadwicked(void *targ){
       }
       break;
     case 8:
-      if(id == 0) putchar('8');
-      if(myrand(10) == 0){
+      if(id == 0) iputchar('8');
+      if(myrand(2) == 0){
         if(!tcbdbout(bdb, kbuf, ksiz) && tcbdbecode(bdb) != TCENOREC){
           eprint(bdb, "tcbdbout");
           err = true;
@@ -1031,8 +1330,8 @@ static void *threadwicked(void *targ){
       }
       break;
     case 9:
-      if(id == 0) putchar('9');
-      if(myrand(10) == 0){
+      if(id == 0) iputchar('9');
+      if(myrand(2) == 0){
         if(!tcbdbout2(bdb, kbuf) && tcbdbecode(bdb) != TCENOREC){
           eprint(bdb, "tcbdbout2");
           err = true;
@@ -1041,7 +1340,7 @@ static void *threadwicked(void *targ){
       }
       break;
     case 10:
-      if(id == 0) putchar('A');
+      if(id == 0) iputchar('A');
       if(!(rbuf = tcbdbget(bdb, kbuf, ksiz, &vsiz))){
         if(tcbdbecode(bdb) != TCENOREC){
           eprint(bdb, "tcbdbget");
@@ -1064,7 +1363,7 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 11:
-      if(id == 0) putchar('B');
+      if(id == 0) iputchar('B');
       if(!(rbuf = tcbdbget(bdb, kbuf, ksiz, &vsiz)) && tcbdbecode(bdb) != TCENOREC){
         eprint(bdb, "tcbdbget");
         err = true;
@@ -1072,7 +1371,7 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 12:
-      if(id == 0) putchar('C');
+      if(id == 0) iputchar('C');
       if(!(rbuf = tcbdbget2(bdb, kbuf)) && tcbdbecode(bdb) != TCENOREC){
         eprint(bdb, "tcbdbget2");
         err = true;
@@ -1080,14 +1379,14 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 13:
-      if(id == 0) putchar('D');
+      if(id == 0) iputchar('D');
       if(!tcbdbget3(bdb, kbuf, ksiz, &vsiz) && tcbdbecode(bdb) != TCENOREC){
         eprint(bdb, "tcbdbget3");
         err = true;
       }
       break;
     case 14:
-      if(id == 0) putchar('E');
+      if(id == 0) iputchar('E');
       if(myrand(rnum / 50) == 0){
         switch(myrand(5)){
         case 0:
@@ -1098,7 +1397,7 @@ static void *threadwicked(void *targ){
           break;
         case 1:
           if(!tcbdbcurlast(cur) && tcbdbecode(bdb) != TCENOREC){
-            eprint(bdb, "tcbdbcurfirst");
+            eprint(bdb, "tcbdbcurlast");
             err = true;
           }
           break;
@@ -1153,7 +1452,42 @@ static void *threadwicked(void *targ){
       tcxstrdel(ikey);
       break;
     default:
-      if(id == 0) putchar('@');
+      if(id == 0) iputchar('@');
+      if(tcbdbtranbegin(bdb)){
+        if(myrand(2) == 0){
+          if(!tcbdbput(bdb, kbuf, ksiz, vbuf, vsiz)){
+            eprint(bdb, "tcbdbput");
+            err = true;
+          }
+          if(!nc) tcmapput(map, kbuf, ksiz, vbuf, vsiz);
+        } else {
+          if(!tcbdbout(bdb, kbuf, ksiz) && tcbdbecode(bdb) != TCENOREC){
+            eprint(bdb, "tcbdbout");
+            err = true;
+          }
+          if(!nc) tcmapout(map, kbuf, ksiz);
+        }
+        if(nc && myrand(2) == 0){
+          if(!tcbdbtranabort(bdb)){
+            eprint(bdb, "tcbdbtranabort");
+            err = true;
+          }
+        } else {
+          if(!tcbdbtrancommit(bdb)){
+            eprint(bdb, "tcbdbtrancommit");
+            err = true;
+          }
+        }
+      } else {
+        eprint(bdb, "tcbdbtranbegin");
+        err = true;
+      }
+      if(myrand(1000) == 0){
+        if(!tcbdbforeach(bdb, iterfunc, NULL)){
+          eprint(bdb, "tcbdbforeach");
+          err = true;
+        }
+      }
       if(myrand(10000) == 0) srand((unsigned int)(tctime() * 1000) % UINT_MAX);
       break;
     }
@@ -1161,11 +1495,11 @@ static void *threadwicked(void *targ){
     if(id == 0){
       if(i % 50 == 0) iprintf(" (%08d)\n", i);
       if(id == 0 && i == rnum / 4){
-        if(!tcbdboptimize(bdb, -1, -1, -1, -1, -1, -1)){
+        if(!tcbdboptimize(bdb, -1, -1, -1, -1, -1, -1) && tcbdbecode(bdb) != TCEINVALID){
           eprint(bdb, "tcbdboptimize");
           err = true;
         }
-        if(!tcbdbcurfirst(cur)){
+        if(!tcbdbcurfirst(cur) && tcbdbecode(bdb) != TCENOREC){
           eprint(bdb, "tcbdbcurfirst");
           err = true;
         }
@@ -1320,8 +1654,7 @@ static void *threadtypical(void *targ){
       }
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -1352,5 +1685,126 @@ static void *threadtypical(void *targ){
 }
 
 
+/* thread the race function */
+static void *threadrace(void *targ){
+  TCBDB *bdb = ((TARGRACE *)targ)->bdb;
+  int rnum = ((TARGRACE *)targ)->rnum;
+  int id = ((TARGRACE *)targ)->id;
+  bool err = false;
+  int mid = rnum * 2;
+  for(int i = 1; !err && i <= rnum; i++){
+    char buf[RECBUFSIZ];
+    int len = sprintf(buf, "%d", myrandnd(i));
+    int rnd = myrand(100);
+    if(rnd < 10){
+      if(!tcbdbputkeep(bdb, buf, len, buf, len) && tcbdbecode(bdb) != TCEKEEP){
+        eprint(bdb, "tcbdbputkeep");
+        err = true;
+      }
+    } else if(rnd < 20){
+      if(!tcbdbputcat(bdb, buf, len, buf, len)){
+        eprint(bdb, "tcbdbputcat");
+        err = true;
+      }
+    } else if(rnd < 30){
+      if(!tcbdbputdup(bdb, buf, len, buf, len)){
+        eprint(bdb, "tcbdbputdup");
+        err = true;
+      }
+    } else if(rnd < 40){
+      if(!tcbdbputdupback(bdb, buf, len, buf, len)){
+        eprint(bdb, "tcbdbputdupback");
+        err = true;
+      }
+    } else if(rnd < 50){
+      if(!tcbdbout(bdb, buf, len) && tcbdbecode(bdb) != TCENOREC){
+        eprint(bdb, "tcbdbout");
+        err = true;
+      }
+    } else {
+      if(myrand(10) == 0){
+        int rsiz = myrand(1024);
+        char *rbuf = tcmalloc(rsiz + 1);
+        for(int j = 0; j < rsiz; j++){
+          rbuf[j] = myrand('z' - 'a') + 'a';
+        }
+        if(myrand(2) == 0){
+          if(!tcbdbput(bdb, buf, len, rbuf, rsiz)){
+            eprint(bdb, "tcbdbputcat");
+            err = true;
+          }
+        } else {
+          if(!tcbdbputcat(bdb, buf, len, rbuf, rsiz)){
+            eprint(bdb, "tcbdbputcat");
+            err = true;
+          }
+        }
+        tcfree(rbuf);
+      } else {
+        if(!tcbdbput(bdb, buf, len, buf, len)){
+          eprint(bdb, "tcbdbput");
+          err = true;
+        }
+      }
+    }
+    if(id == 0){
+      if(myrand(mid) == 0){
+        iprintf("[v]");
+        if(!tcbdbvanish(bdb)){
+          eprint(bdb, "tcbdbvanish");
+          err = true;
+        }
+      }
+      if(myrand(mid) == 0){
+        iprintf("[o]");
+        if(!tcbdboptimize(bdb, -1, -1, myrand(rnum) + 1, myrand(10), myrand(10), 0)){
+          eprint(bdb, "tcbdbvanish");
+          err = true;
+        }
+      }
+      if(myrand(mid) == 0){
+        iprintf("[d]");
+        if(!tcbdbdefrag(bdb, -1)){
+          eprint(bdb, "tcbdbdefrag");
+          err = true;
+        }
+      }
+      if(myrand(mid) == 0){
+        iprintf("[i]");
+        BDBCUR *cur = tcbdbcurnew(bdb);
+        if(!tcbdbcurfirst(cur) && tcbdbecode(bdb) != TCENOREC){
+          eprint(bdb, "tcbdbcurfirst");
+          err = true;
+        }
+        char *kbuf;
+        int ksiz;
+        while((kbuf = tcbdbcurkey(cur, &ksiz)) != NULL){
+          int vsiz;
+          char *vbuf = tcbdbcurval(cur, &vsiz);
+          if(vbuf){
+            tcfree(vbuf);
+          } else if(tcbdbecode(bdb) != TCENOREC){
+            eprint(bdb, "tcbdbget");
+            err = true;
+          }
+          tcfree(kbuf);
+          tcbdbcurnext(cur);
+        }
+        if(tcbdbecode(bdb) != TCENOREC){
+          eprint(bdb, "(validation)");
+          err = true;
+        }
+        tcbdbcurdel(cur);
+      }
+      if(rnum > 250 && i % (rnum / 250) == 0){
+        iputchar('.');
+        if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
+      }
+    }
+  }
+  return err ? "error" : NULL;
+}
+
+
 
 // END OF FILE
