@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The fixed-length database API of Tokyo Cabinet
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -31,6 +31,8 @@ __TCFDB_CLINKAGEBEGIN
 #include <stdbool.h>
 #include <stdint.h>
 #include <time.h>
+#include <limits.h>
+#include <math.h>
 #include <tcutil.h>
 
 
@@ -44,6 +46,8 @@ typedef struct {                         /* type of structure for a fixed-length
   void *mmtx;                            /* mutex for method */
   void *amtx;                            /* mutex for attribute */
   void *rmtxs;                           /* mutexes for records */
+  void *tmtx;                            /* mutex for transaction */
+  void *wmtx;                            /* mutex for write ahead logging */
   void *eckey;                           /* key for thread specific error code */
   uint8_t type;                          /* database type */
   uint8_t flags;                         /* additional flags */
@@ -54,7 +58,7 @@ typedef struct {                         /* type of structure for a fixed-length
   uint64_t limid;                        /* limit ID number */
   char *path;                            /* path of the database file */
   int fd;                                /* file descriptor of the database file */
-  uint32_t omode;                        /* connection mode */
+  uint32_t omode;                        /* open mode */
   uint64_t rnum;                         /* number of the records */
   uint64_t fsiz;                         /* size of the database file */
   uint64_t min;                          /* minimum ID number */
@@ -66,6 +70,9 @@ typedef struct {                         /* type of structure for a fixed-length
   bool fatal;                            /* whether a fatal error occured */
   uint64_t inode;                        /* inode number */
   time_t mtime;                          /* modification time */
+  bool tran;                             /* whether in the transaction */
+  int walfd;                             /* file descriptor of write ahead logging */
+  uint64_t walend;                       /* end offset of write ahead logging */
   int dbgfd;                             /* file descriptor for debugging */
   int64_t cnt_writerec;                  /* tesing counter for record write times */
   int64_t cnt_readrec;                   /* tesing counter for record read times */
@@ -83,7 +90,8 @@ enum {                                   /* enumeration for open modes */
   FDBOCREAT = 1 << 2,                    /* writer creating */
   FDBOTRUNC = 1 << 3,                    /* writer truncating */
   FDBONOLCK = 1 << 4,                    /* open without locking */
-  FDBOLCKNB = 1 << 5                     /* lock without blocking */
+  FDBOLCKNB = 1 << 5,                    /* lock without blocking */
+  FDBOTSYNC = 1 << 6                     /* synchronize every transaction */
 };
 
 enum {                                   /* enumeration for ID constants */
@@ -115,7 +123,7 @@ void tcfdbdel(TCFDB *fdb);
 /* Get the last happened error code of a fixed-length database object.
    `fdb' specifies the fixed-length database object.
    The return value is the last happened error code.
-   The following error code is defined: `TCESUCCESS' for success, `TCETHREAD' for threading
+   The following error codes are defined: `TCESUCCESS' for success, `TCETHREAD' for threading
    error, `TCEINVALID' for invalid operation, `TCENOFILE' for file not found, `TCENOPERM' for no
    permission, `TCEMETA' for invalid meta data, `TCERHEAD' for invalid record header, `TCEOPEN'
    for open error, `TCECLOSE' for close error, `TCETRUNC' for trunc error, `TCESYNC' for sync
@@ -150,10 +158,11 @@ bool tcfdbtune(TCFDB *fdb, int32_t width, int64_t limsiz);
    `fdb' specifies the fixed-length database object which is not opened.
    `path' specifies the path of the database file.
    `omode' specifies the connection mode: `FDBOWRITER' as a writer, `FDBOREADER' as a reader.
-   If the mode is `FDBOWRITER', the following may be added by bitwise or: `FDBOCREAT', which
+   If the mode is `FDBOWRITER', the following may be added by bitwise-or: `FDBOCREAT', which
    means it creates a new database if not exist, `FDBOTRUNC', which means it creates a new
-   database regardless if one exists.  Both of `FDBOREADER' and `FDBOWRITER' can be added to by
-   bitwise or: `FDBONOLCK', which means it opens the database file without file locking, or
+   database regardless if one exists, `FDBOTSYNC', which means every transaction synchronizes
+   updated contents with the device.  Both of `FDBOREADER' and `FDBOWRITER' can be added to by
+   bitwise-or: `FDBONOLCK', which means it opens the database file without file locking, or
    `FDBOLCKNB', which means locking is performed without blocking.
    If successful, the return value is true, else, it is false. */
 bool tcfdbopen(TCFDB *fdb, const char *path, int omode);
@@ -543,6 +552,34 @@ TCLIST *tcfdbrange4(TCFDB *fdb, const void *ibuf, int isiz, int max);
 TCLIST *tcfdbrange5(TCFDB *fdb, const void *istr, int max);
 
 
+/* Add an integer to a record in a fixed-length database object.
+   `fdb' specifies the fixed-length database object connected as a writer.
+   `id' specifies the ID number.  It should be more than 0.  If it is `FDBIDMIN', the minimum ID
+   number of existing records is specified.  If it is `FDBIDPREV', the number less by one than
+   the minimum ID number of existing records is specified.  If it is `FDBIDMAX', the maximum ID
+   number of existing records is specified.  If it is `FDBIDNEXT', the number greater by one than
+   the maximum ID number of existing records is specified.
+   `num' specifies the additional value.
+   If successful, the return value is the summation value, else, it is `INT_MIN'.
+   If the corresponding record exists, the value is treated as an integer and is added to.  If no
+   record corresponds, a new record of the additional value is stored. */
+int tcfdbaddint(TCFDB *fdb, int64_t id, int num);
+
+
+/* Add a real number to a record in a fixed-length database object.
+   `fdb' specifies the fixed-length database object connected as a writer.
+   `id' specifies the ID number.  It should be more than 0.  If it is `FDBIDMIN', the minimum ID
+   number of existing records is specified.  If it is `FDBIDPREV', the number less by one than
+   the minimum ID number of existing records is specified.  If it is `FDBIDMAX', the maximum ID
+   number of existing records is specified.  If it is `FDBIDNEXT', the number greater by one than
+   the maximum ID number of existing records is specified.
+   `num' specifies the additional value.
+   If successful, the return value is the summation value, else, it is Not-a-Number.
+   If the corresponding record exists, the value is treated as a real number and is added to.  If
+   no record corresponds, a new record of the additional value is stored. */
+double tcfdbadddouble(TCFDB *fdb, int64_t id, double num);
+
+
 /* Synchronize updated contents of a fixed-length database object with the file and the device.
    `fdb' specifies the fixed-length database object connected as a writer.
    If successful, the return value is true, else, it is false.
@@ -578,6 +615,32 @@ bool tcfdbvanish(TCFDB *fdb);
 bool tcfdbcopy(TCFDB *fdb, const char *path);
 
 
+/* Begin the transaction of a fixed-length database object.
+   `fdb' specifies the fixed-length database object connected as a writer.
+   If successful, the return value is true, else, it is false.
+   The database is locked by the thread while the transaction so that only one transaction can be
+   activated with a database object at the same time.  Thus, the serializable isolation level is
+   assumed if every database operation is performed in the transaction.  All updated regions are
+   kept track of by write ahead logging while the transaction.  If the database is closed during
+   transaction, the transaction is aborted implicitly. */
+bool tcfdbtranbegin(TCFDB *fdb);
+
+
+/* Commit the transaction of a fixed-length database object.
+   `fdb' specifies the fixed-length database object connected as a writer.
+   If successful, the return value is true, else, it is false.
+   Update in the transaction is fixed when it is committed successfully. */
+bool tcfdbtrancommit(TCFDB *fdb);
+
+
+/* Abort the transaction of a fixed-length database object.
+   `fdb' specifies the fixed-length database object connected as a writer.
+   If successful, the return value is true, else, it is false.
+   Update in the transaction is discarded when it is aborted.  The state of the database is
+   rollbacked to before transaction. */
+bool tcfdbtranabort(TCFDB *fdb);
+
+
 /* Get the file path of a fixed-length database object.
    `fdb' specifies the fixed-length database object.
    The return value is the path of the database file or `NULL' if the object does not connect to
@@ -626,6 +689,12 @@ void tcfdbsetdbgfd(TCFDB *fdb, int fd);
 int tcfdbdbgfd(TCFDB *fdb);
 
 
+/* Check whether mutual exclusion control is set to a fixed-length database object.
+   `fdb' specifies the fixed-length database object.
+   If mutual exclusion control is set, it is true, else it is false. */
+bool tcfdbhasmutex(TCFDB *fdb);
+
+
 /* Synchronize updating contents on memory of a fixed-length database object.
    `fdb' specifies the fixed-length database object connected as a writer.
    `phys' specifies whether to synchronize physically.
@@ -651,7 +720,7 @@ uint64_t tcfdbmax(TCFDB *fdb);
    `fdb' specifies the fixed-length database object.
    The return value is the width of the value of each record or 0 if the object does not connect
    to any database file. */
-uint64_t tcfdbwidth(TCFDB *fdb);
+uint32_t tcfdbwidth(TCFDB *fdb);
 
 
 /* Get the limit file size of a fixed-length database object.
@@ -670,15 +739,15 @@ uint64_t tcfdblimid(TCFDB *fdb);
 
 /* Get the inode number of the database file of a fixed-length database object.
    `fdb' specifies the fixed-length database object.
-   The return value is the inode number of the database file or 0 the object does not connect to
-   any database file. */
+   The return value is the inode number of the database file or 0 if the object does not connect
+   to any database file. */
 uint64_t tcfdbinode(TCFDB *fdb);
 
 
 /* Get the modification time of the database file of a fixed-length database object.
    `fdb' specifies the fixed-length database object.
-   The return value is the inode number of the database file or 0 the object does not connect to
-   any database file. */
+   The return value is the inode number of the database file or 0 if the object does not connect
+   to any database file. */
 time_t tcfdbmtime(TCFDB *fdb);
 
 
@@ -706,6 +775,86 @@ uint8_t tcfdbflags(TCFDB *fdb);
 char *tcfdbopaque(TCFDB *fdb);
 
 
+/* Store a record into a fixed-length database object with a duplication handler.
+   `fdb' specifies the fixed-length database object connected as a writer.
+   `id' specifies the ID number.  It should be more than 0.  If it is `FDBIDMIN', the minimum ID
+   number of existing records is specified.  If it is `FDBIDPREV', the number less by one than
+   the minimum ID number of existing records is specified.  If it is `FDBIDMAX', the maximum ID
+   number of existing records is specified.  If it is `FDBIDNEXT', the number greater by one than
+   the maximum ID number of existing records is specified.
+   `vbuf' specifies the pointer to the region of the value.  `NULL' means that record addition is
+   ommited if there is no corresponding record.
+   `vsiz' specifies the size of the region of the value.  If the size of the value is greater
+   than the width tuning parameter of the database, the size is cut down to the width.
+   `proc' specifies the pointer to the callback function to process duplication.  It receives
+   four parameters.  The first parameter is the pointer to the region of the value.  The second
+   parameter is the size of the region of the value.  The third parameter is the pointer to the
+   variable into which the size of the region of the return value is assigned.  The fourth
+   parameter is the pointer to the optional opaque object.  It returns the pointer to the result
+   object allocated with `malloc'.  It is released by the caller.  If it is `NULL', the record is
+   not modified.  If it is `(void *)-1', the record is removed.
+   `op' specifies an arbitrary pointer to be given as a parameter of the callback function.  If
+   it is not needed, `NULL' can be specified.
+   If successful, the return value is true, else, it is false.
+   Note that the callback function can not perform any database operation because the function
+   is called in the critical section guarded by the same locks of database operations. */
+bool tcfdbputproc(TCFDB *fdb, int64_t id, const void *vbuf, int vsiz, TCPDPROC proc, void *op);
+
+
+/* Move the iterator to the record corresponding a key of a fixed-length database object.
+   `fdb' specifies the fixed-length database object.
+   `id' specifies the ID number.  It should be more than 0.  If it is `FDBIDMIN', the minimum ID
+   number of existing records is specified.  If it is `FDBIDMAX', the maximum ID number of
+   existing records is specified.
+   If successful, the return value is true, else, it is false.  False is returned if there is
+   no record corresponding the condition. */
+bool tcfdbiterinit2(TCFDB *fdb, int64_t id);
+
+
+/* Move the iterator to the decimal record of a fixed-length database object.
+   `fdb' specifies the fixed-length database object.
+   `kbuf' specifies the pointer to the region of the decimal key.  It should be more than 0.  If
+   it is "min", the minimum ID number of existing records is specified.  If it is "max", the
+   maximum ID number of existing records is specified.
+   `ksiz' specifies the size of the region of the key.
+   If successful, the return value is true, else, it is false.  False is returned if there is
+   no record corresponding the condition. */
+bool tcfdbiterinit3(TCFDB *fdb, const void *kbuf, int ksiz);
+
+
+/* Move the iterator to the decimal string record of a fixed-length database object.
+   `fdb' specifies the fixed-length database object.
+   `kstr' specifies the string of the decimal key.  It should be more than 0.  If it is "min",
+   the minimum ID number of existing records is specified.  If it is "max", the maximum ID number
+   of existing records is specified.
+   If successful, the return value is true, else, it is false.  False is returned if there is
+   no record corresponding the condition. */
+bool tcfdbiterinit4(TCFDB *fdb, const char *kstr);
+
+
+/* Process each record atomically of a fixed-length database object.
+   `fdb' specifies the fixed-length database object.
+   `iter' specifies the pointer to the iterator function called for each record.  It receives
+   five parameters.  The first parameter is the pointer to the region of the key.  The second
+   parameter is the size of the region of the key.  The third parameter is the pointer to the
+   region of the value.  The fourth parameter is the size of the region of the value.  The fifth
+   parameter is the pointer to the optional opaque object.  It returns true to continue iteration
+   or false to stop iteration.
+   `op' specifies an arbitrary pointer to be given as a parameter of the iterator function.  If
+   it is not needed, `NULL' can be specified.
+   If successful, the return value is true, else, it is false.
+   Note that the callback function can not perform any database operation because the function
+   is called in the critical section guarded by the same locks of database operations. */
+bool tcfdbforeach(TCFDB *fdb, TCITER iter, void *op);
+
+
+/* Generate the ID number from arbitrary binary data.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   The return value is the ID number. */
+int64_t tcfdbkeytoid(const char *kbuf, int ksiz);
+
+
 
 __TCFDB_CLINKAGEEND
 #endif                                   /* duplication check */
