@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The command line utility of the fixed-length database API
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -29,7 +29,6 @@ int main(int argc, char **argv);
 static void usage(void);
 static void printerr(TCFDB *fdb);
 static int printdata(const char *ptr, int size, bool px);
-static char *hextoobj(const char *str, int *sp);
 static char *mygetline(FILE *ifp);
 static int runcreate(int argc, char **argv);
 static int runinform(int argc, char **argv);
@@ -58,7 +57,7 @@ int main(int argc, char **argv){
   g_progname = argv[0];
   g_dbgfd = -1;
   const char *ebuf = getenv("TCDBGFD");
-  if(ebuf) g_dbgfd = atoi(ebuf);
+  if(ebuf) g_dbgfd = tcatoix(ebuf);
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "create")){
@@ -93,7 +92,7 @@ static void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s create path [width [limsiz]]\n", g_progname);
   fprintf(stderr, "  %s inform [-nl|-nb] path\n", g_progname);
-  fprintf(stderr, "  %s put [-nl|-nb] [-sx] [-dk|-dc] path key value\n", g_progname);
+  fprintf(stderr, "  %s put [-nl|-nb] [-sx] [-dk|-dc|-dai|-dad] path key value\n", g_progname);
   fprintf(stderr, "  %s out [-nl|-nb] [-sx] path key\n", g_progname);
   fprintf(stderr, "  %s get [-nl|-nb] [-sx] [-px] [-pz] path key\n", g_progname);
   fprintf(stderr, "  %s list [-nl|-nb] [-m num] [-pv] [-px] [-rb lkey ukey] [-ri str] path\n",
@@ -131,27 +130,6 @@ static int printdata(const char *ptr, int size, bool px){
 }
 
 
-/* create a binary object from a hexadecimal string */
-static char *hextoobj(const char *str, int *sp){
-  int len = strlen(str);
-  char *buf = tcmalloc(len + 1);
-  int j = 0;
-  for(int i = 0; i < len; i += 2){
-    while(strchr(" \n\r\t\f\v", str[i])){
-      i++;
-    }
-    char mbuf[3];
-    if((mbuf[0] = str[i]) == '\0') break;
-    if((mbuf[1] = str[i+1]) == '\0') break;
-    mbuf[2] = '\0';
-    buf[j++] = (char)strtol(mbuf, NULL, 16);
-  }
-  buf[j] = '\0';
-  *sp = j;
-  return buf;
-}
-
-
 /* read a line from a file descriptor */
 static char *mygetline(FILE *ifp){
   int len = 0;
@@ -198,8 +176,8 @@ static int runcreate(int argc, char **argv){
     }
   }
   if(!path) usage();
-  int width = wstr ? atoi(wstr) : -1;
-  int64_t limsiz = lstr ? strtoll(lstr, NULL, 10) : -1;
+  int width = wstr ? tcatoix(wstr) : -1;
+  int64_t limsiz = lstr ? tcatoix(lstr) : -1;
   int rv = proccreate(path, width, limsiz);
   return rv;
 }
@@ -248,6 +226,10 @@ static int runput(int argc, char **argv){
         dmode = -1;
       } else if(!strcmp(argv[i], "-dc")){
         dmode = 1;
+      } else if(!strcmp(argv[i], "-dai")){
+        dmode = 10;
+      } else if(!strcmp(argv[i], "-dad")){
+        dmode = 11;
       } else if(!strcmp(argv[i], "-sx")){
         sx = true;
       } else {
@@ -264,11 +246,11 @@ static int runput(int argc, char **argv){
     }
   }
   if(!path || !key || !value) usage();
-  int ksiz, vsiz;
   char *kbuf, *vbuf;
+  int ksiz, vsiz;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
-    vbuf = hextoobj(value, &vsiz);
+    kbuf = tchexdecode(key, &ksiz);
+    vbuf = tchexdecode(value, &vsiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
@@ -311,7 +293,7 @@ static int runout(int argc, char **argv){
   int ksiz;
   char *kbuf;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
+    kbuf = tchexdecode(key, &ksiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
@@ -357,7 +339,7 @@ static int runget(int argc, char **argv){
   int ksiz;
   char *kbuf;
   if(sx){
-    kbuf = hextoobj(key, &ksiz);
+    kbuf = tchexdecode(key, &ksiz);
   } else {
     ksiz = strlen(key);
     kbuf = tcmemdup(key, ksiz);
@@ -386,7 +368,7 @@ static int runlist(int argc, char **argv){
         omode |= FDBOLCKNB;
       } else if(!strcmp(argv[i], "-m")){
         if(++i >= argc) usage();
-        max = atoi(argv[i]);
+        max = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-pv")){
         pv = true;
       } else if(!strcmp(argv[i], "-px")){
@@ -440,8 +422,8 @@ static int runoptimize(int argc, char **argv){
     }
   }
   if(!path) usage();
-  int width = wstr ? atoi(wstr) : -1;
-  int64_t limsiz = lstr ? strtoll(lstr, NULL, 10) : -1;
+  int width = wstr ? tcatoix(wstr) : -1;
+  int64_t limsiz = lstr ? tcatoix(lstr) : -1;
   int rv = procoptimize(path, width, limsiz, omode);
   return rv;
 }
@@ -527,6 +509,7 @@ static int procinform(const char *path, int omode){
   case TCDBTHASH: type = "hash"; break;
   case TCDBTBTREE: type = "btree"; break;
   case TCDBTFIXED: type = "fixed"; break;
+  case TCDBTTABLE: type = "table"; break;
   }
   printf("database type: %s\n", type);
   uint8_t flags = tcfdbflags(fdb);
@@ -578,6 +561,18 @@ static int procput(const char *path, const char *kbuf, int ksiz, const char *vbu
       err = true;
     }
     break;
+  case 10:
+    if(tcfdbaddint(fdb, tcfdbkeytoid(kbuf, ksiz), tcatoi(vbuf)) == INT_MIN){
+      printerr(fdb);
+      err = true;
+    }
+    break;
+  case 11:
+    if(isnan(tcfdbadddouble(fdb, tcfdbkeytoid(kbuf, ksiz), tcatof(vbuf)))){
+      printerr(fdb);
+      err = true;
+    }
+    break;
   default:
     if(!tcfdbput2(fdb, kbuf, ksiz, vbuf, vsiz)){
       printerr(fdb);
@@ -755,7 +750,7 @@ static int procimporttsv(const char *path, const char *file, int omode, bool sc)
     }
     *pv = '\0';
     if(sc) tcstrtolower(line);
-    if(!tcfdbput3(fdb, line, pv + 1) && tcfdbecode(fdb) != TCEKEEP){
+    if(!tcfdbput3(fdb, line, pv + 1)){
       printerr(fdb);
       err = true;
     }
@@ -780,8 +775,9 @@ static int procimporttsv(const char *path, const char *file, int omode, bool sc)
 
 /* perform version command */
 static int procversion(void){
-  printf("Tokyo Cabinet version %s (%d:%s)\n", tcversion, _TC_LIBVER, _TC_FORMATVER);
-  printf("Copyright (C) 2006-2008 Mikio Hirabayashi\n");
+  printf("Tokyo Cabinet version %s (%d:%s) for %s\n",
+         tcversion, _TC_LIBVER, _TC_FORMATVER, TCSYSNAME);
+  printf("Copyright (C) 2006-2009 Mikio Hirabayashi\n");
   return 0;
 }
 
