@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The test cases of the fixed-length database API
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -18,7 +18,7 @@
 #include <tcfdb.h>
 #include "myconf.h"
 
-#define RECBUFSIZ      32                // buffer for records
+#define RECBUFSIZ      48                // buffer for records
 #define EXHEADSIZ      256               // expected header size
 
 typedef struct {                         // type of structure for write thread
@@ -62,6 +62,7 @@ typedef struct {                         // type of structure for typical thread
 
 /* global variables */
 const char *g_progname;                  // program name
+unsigned int g_randseed;                 // random seed
 int g_dbgfd;                             // debugging output
 
 
@@ -69,10 +70,13 @@ int g_dbgfd;                             // debugging output
 int main(int argc, char **argv);
 static void usage(void);
 static void iprintf(const char *format, ...);
+static void iputchar(int c);
 static void eprint(TCFDB *fdb, const char *func);
 static void mprint(TCFDB *fdb);
+static void sysprint(void);
 static int myrand(int range);
 static int myrandnd(int range);
+static bool iterfunc(const void *kbuf, int ksiz, const void *vbuf, int vsiz, void *op);
 static int runwrite(int argc, char **argv);
 static int runread(int argc, char **argv);
 static int runremove(int argc, char **argv);
@@ -95,10 +99,11 @@ static void *threadtypical(void *targ);
 /* main routine */
 int main(int argc, char **argv){
   g_progname = argv[0];
-  g_dbgfd = -1;
-  const char *ebuf = getenv("TCDBGFD");
-  if(ebuf) g_dbgfd = atoi(ebuf);
-  srand((unsigned int)(tctime() * 1000) % UINT_MAX);
+  const char *ebuf = getenv("TCRNDSEED");
+  g_randseed = ebuf ? tcatoix(ebuf) : tctime() * 1000;
+  srand(g_randseed);
+  ebuf = getenv("TCDBGFD");
+  g_dbgfd = ebuf ? tcatoix(ebuf) : UINT16_MAX;
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "write")){
@@ -114,6 +119,13 @@ int main(int argc, char **argv){
   } else {
     usage();
   }
+  if(rv != 0){
+    printf("FAILED:");
+    for(int i = 0; i < argc; i++){
+      printf(" %s", argv[i]);
+    }
+    printf("\n\n");
+  }
   return rv;
 }
 
@@ -145,6 +157,13 @@ static void iprintf(const char *format, ...){
 }
 
 
+/* print a character and flush the buffer */
+static void iputchar(int c){
+  putchar(c);
+  fflush(stdout);
+}
+
+
 /* print error message of fixed-length database */
 static void eprint(TCFDB *fdb, const char *func){
   const char *path = tcfdbpath(fdb);
@@ -168,9 +187,27 @@ static void mprint(TCFDB *fdb){
 }
 
 
+/* print system information */
+static void sysprint(void){
+  TCMAP *info = tcsysinfo();
+  if(info){
+    tcmapiterinit(info);
+    const char *kbuf;
+    while((kbuf = tcmapiternext2(info)) != NULL){
+      iprintf("sys_%s: %s\n", kbuf, tcmapiterval2(kbuf));
+    }
+    tcmapdel(info);
+  }
+}
+
+
 /* get a random number */
 static int myrand(int range){
-  return (int)((double)range * rand() / (RAND_MAX + 1.0));
+  if(range < 2) return 0;
+  int high = (unsigned int)rand() >> 4;
+  int low = range * (rand() / (RAND_MAX + 1.0));
+  low &= (unsigned int)INT_MAX >> 4;
+  return (high + low) % range;
 }
 
 
@@ -181,6 +218,19 @@ static int myrandnd(int range){
 }
 
 
+/* iterator function */
+static bool iterfunc(const void *kbuf, int ksiz, const void *vbuf, int vsiz, void *op){
+  unsigned int sum = 0;
+  while(--ksiz >= 0){
+    sum += ((char *)kbuf)[ksiz];
+  }
+  while(--vsiz >= 0){
+    sum += ((char *)vbuf)[vsiz];
+  }
+  return myrand(100 + (sum & 0xff)) > 0;
+}
+
+
 /* parse arguments of write command */
 static int runwrite(int argc, char **argv){
   char *path = NULL;
@@ -216,11 +266,11 @@ static int runwrite(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
-  int width = wstr ? atoi(wstr) : -1;
-  int64_t limsiz = lstr ? strtoll(lstr, NULL, 10) : -1;
+  int width = wstr ? tcatoix(wstr) : -1;
+  int64_t limsiz = lstr ? tcatoix(lstr) : -1;
   int rv = procwrite(path, tnum, rnum, width, limsiz, omode, rnd);
   return rv;
 }
@@ -255,7 +305,7 @@ static int runread(int argc, char **argv){
     }
   }
   if(!path || !tstr) usage();
-  int tnum = atoi(tstr);
+  int tnum = tcatoix(tstr);
   if(tnum < 1) usage();
   int rv = procread(path, tnum, omode, wb, rnd);
   return rv;
@@ -288,7 +338,7 @@ static int runremove(int argc, char **argv){
     }
   }
   if(!path || !tstr) usage();
-  int tnum = atoi(tstr);
+  int tnum = tcatoix(tstr);
   if(tnum < 1) usage();
   int rv = procremove(path, tnum, omode, rnd);
   return rv;
@@ -324,8 +374,8 @@ static int runwicked(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
   int rv = procwicked(path, tnum, rnum, omode, nc);
   return rv;
@@ -352,7 +402,7 @@ static int runtypical(int argc, char **argv){
         nc = true;
       } else if(!strcmp(argv[i], "-rr")){
         if(++i >= argc) usage();
-        rratio = atoi(argv[i]);
+        rratio = tcatoix(argv[i]);
       } else {
         usage();
       }
@@ -371,11 +421,11 @@ static int runtypical(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
-  int width = wstr ? atoi(wstr) : -1;
-  int64_t limsiz = lstr ? strtoll(lstr, NULL, 10) : -1;
+  int width = wstr ? tcatoix(wstr) : -1;
+  int64_t limsiz = lstr ? tcatoix(lstr) : -1;
   int rv = proctypical(path, tnum, rnum, width, limsiz, omode, nc, rratio);
   return rv;
 }
@@ -384,8 +434,9 @@ static int runtypical(int argc, char **argv){
 /* perform write command */
 static int procwrite(const char *path, int tnum, int rnum, int width, int64_t limsiz,
                      int omode, bool rnd){
-  iprintf("<Writing Test>\n  path=%s  tnum=%d  rnum=%d  width=%d  limsiz=%lld"
-          "  omode=%d  rnd=%d\n\n", path, tnum, rnum, width, (long long)limsiz, omode, rnd);
+  iprintf("<Writing Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  width=%d  limsiz=%lld"
+          "  omode=%d  rnd=%d\n\n",
+          g_randseed, path, tnum, rnum, width, (long long)limsiz, omode, rnd);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -436,6 +487,7 @@ static int procwrite(const char *path, int tnum, int rnum, int width, int64_t li
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -449,8 +501,8 @@ static int procwrite(const char *path, int tnum, int rnum, int width, int64_t li
 
 /* perform read command */
 static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
-  iprintf("<Reading Test>\n  path=%s  tnum=%d  omode=%d  wb=%d  rnd=%d\n\n",
-          path, tnum, omode, wb, rnd);
+  iprintf("<Reading Test>\n  seed=%u  path=%s  tnum=%d  omode=%d  wb=%d  rnd=%d\n\n",
+          g_randseed, path, tnum, omode, wb, rnd);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -500,6 +552,7 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -513,7 +566,8 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
 
 /* perform remove command */
 static int procremove(const char *path, int tnum, int omode, bool rnd){
-  iprintf("<Removing Test>\n  path=%s  tnum=%d  omode=%d  rnd=%d\n\n", path, tnum, omode, rnd);
+  iprintf("<Removing Test>\n  seed=%u  path=%s  tnum=%d  omode=%d  rnd=%d\n\n",
+          g_randseed, path, tnum, omode, rnd);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -561,6 +615,7 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -574,8 +629,8 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
 
 /* perform wicked command */
 static int procwicked(const char *path, int tnum, int rnum, int omode, bool nc){
-  iprintf("<Writing Test>\n  path=%s  tnum=%d  rnum=%d  omode=%d  nc=%d\n\n",
-          path, tnum, rnum, omode, nc);
+  iprintf("<Writing Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  omode=%d  nc=%d\n\n",
+          g_randseed, path, tnum, rnum, omode, nc);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -648,8 +703,8 @@ static int procwicked(const char *path, int tnum, int rnum, int omode, bool nc){
       int rsiz;
       char *rbuf = tcfdbget2(fdb, kbuf, ksiz, &rsiz);
       if(vbuf){
-        putchar('.');
-        if(vsiz > RECBUFSIZ) vsiz = RECBUFSIZ;
+        iputchar('.');
+        if(vsiz > tcfdbwidth(fdb)) vsiz = tcfdbwidth(fdb);
         if(!rbuf){
           eprint(fdb, "tcfdbget");
           err = true;
@@ -658,7 +713,7 @@ static int procwicked(const char *path, int tnum, int rnum, int omode, bool nc){
           err = true;
         }
       } else {
-        putchar('*');
+        iputchar('*');
         if(rbuf || tcfdbecode(fdb) != TCENOREC){
           eprint(fdb, "(validation)");
           err = true;
@@ -673,6 +728,7 @@ static int procwicked(const char *path, int tnum, int rnum, int omode, bool nc){
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -687,9 +743,9 @@ static int procwicked(const char *path, int tnum, int rnum, int omode, bool nc){
 /* perform typical command */
 static int proctypical(const char *path, int tnum, int rnum, int width, int64_t limsiz,
                        int omode, bool nc, int rratio){
-  iprintf("<Typical Access Test>\n  path=%s  tnum=%d  rnum=%d  width=%d  limsiz=%lld"
+  iprintf("<Typical Access Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  width=%d  limsiz=%lld"
           "  omode=%d  nc=%d  rratio=%d\n\n",
-          path, tnum, rnum, width, (long long)limsiz, omode, nc, rratio);
+          g_randseed, path, tnum, rnum, width, (long long)limsiz, omode, nc, rratio);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -742,6 +798,7 @@ static int proctypical(const char *path, int tnum, int rnum, int width, int64_t
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -770,8 +827,7 @@ static void *threadwrite(void *targ){
       break;
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -807,8 +863,7 @@ static void *threadread(void *targ){
       tcfree(vbuf);
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -833,8 +888,7 @@ static void *threadremove(void *targ){
       break;
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -862,7 +916,7 @@ static void *threadwicked(void *targ){
     if(!nc) tcglobalmutexlock();
     switch(myrand(16)){
     case 0:
-      if(id == 0) putchar('0');
+      if(id == 0) iputchar('0');
       if(!tcfdbput2(fdb, kbuf, ksiz, vbuf, vsiz)){
         eprint(fdb, "tcfdbput2");
         err = true;
@@ -870,7 +924,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapput(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 1:
-      if(id == 0) putchar('1');
+      if(id == 0) iputchar('1');
       if(!tcfdbput3(fdb, kbuf, vbuf)){
         eprint(fdb, "tcfdbput3");
         err = true;
@@ -878,7 +932,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapput2(map, kbuf, vbuf);
       break;
     case 2:
-      if(id == 0) putchar('2');
+      if(id == 0) iputchar('2');
       if(!tcfdbputkeep2(fdb, kbuf, ksiz, vbuf, vsiz) && tcfdbecode(fdb) != TCEKEEP){
         eprint(fdb, "tcfdbputkeep2");
         err = true;
@@ -886,7 +940,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputkeep(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 3:
-      if(id == 0) putchar('3');
+      if(id == 0) iputchar('3');
       if(!tcfdbputkeep3(fdb, kbuf, vbuf) && tcfdbecode(fdb) != TCEKEEP){
         eprint(fdb, "tcfdbputkeep3");
         err = true;
@@ -894,7 +948,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputkeep2(map, kbuf, vbuf);
       break;
     case 4:
-      if(id == 0) putchar('4');
+      if(id == 0) iputchar('4');
       if(!tcfdbputcat2(fdb, kbuf, ksiz, vbuf, vsiz)){
         eprint(fdb, "tcfdbputcat2");
         err = true;
@@ -902,7 +956,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputcat(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 5:
-      if(id == 0) putchar('5');
+      if(id == 0) iputchar('5');
       if(!tcfdbputcat3(fdb, kbuf, vbuf)){
         eprint(fdb, "tcfdbputcat3");
         err = true;
@@ -910,8 +964,8 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputcat2(map, kbuf, vbuf);
       break;
     case 6:
-      if(id == 0) putchar('6');
-      if(myrand(10) == 0){
+      if(id == 0) iputchar('6');
+      if(myrand(2) == 0){
         if(!tcfdbout2(fdb, kbuf, ksiz) && tcfdbecode(fdb) != TCENOREC){
           eprint(fdb, "tcfdbout2");
           err = true;
@@ -920,8 +974,8 @@ static void *threadwicked(void *targ){
       }
       break;
     case 7:
-      if(id == 0) putchar('7');
-      if(myrand(10) == 0){
+      if(id == 0) iputchar('7');
+      if(myrand(2) == 0){
         if(!tcfdbout3(fdb, kbuf) && tcfdbecode(fdb) != TCENOREC){
           eprint(fdb, "tcfdbout3");
           err = true;
@@ -930,7 +984,7 @@ static void *threadwicked(void *targ){
       }
       break;
     case 8:
-      if(id == 0) putchar('8');
+      if(id == 0) iputchar('8');
       if(!(rbuf = tcfdbget2(fdb, kbuf, ksiz, &vsiz))){
         if(tcfdbecode(fdb) != TCENOREC){
           eprint(fdb, "tcfdbget2");
@@ -953,7 +1007,7 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 9:
-      if(id == 0) putchar('9');
+      if(id == 0) iputchar('9');
       if(!(rbuf = tcfdbget2(fdb, kbuf, ksiz, &vsiz)) && tcfdbecode(fdb) != TCENOREC){
         eprint(fdb, "tcfdbget2");
         err = true;
@@ -961,7 +1015,7 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 10:
-      if(id == 0) putchar('A');
+      if(id == 0) iputchar('A');
       if(!(rbuf = tcfdbget3(fdb, kbuf)) && tcfdbecode(fdb) != TCENOREC){
         eprint(fdb, "tcfdbge3");
         err = true;
@@ -969,7 +1023,7 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 11:
-      if(id == 0) putchar('B');
+      if(id == 0) iputchar('B');
       if(myrand(1) == 0) vsiz = 1;
       if((vsiz = tcfdbget4(fdb, kid, vbuf, vsiz)) < 0 && tcfdbecode(fdb) != TCENOREC){
         eprint(fdb, "tcfdbget4");
@@ -977,7 +1031,7 @@ static void *threadwicked(void *targ){
       }
       break;
     case 14:
-      if(id == 0) putchar('E');
+      if(id == 0) iputchar('E');
       if(myrand(rnum / 50) == 0){
         if(!tcfdbiterinit(fdb)){
           eprint(fdb, "tcfdbiterinit");
@@ -995,7 +1049,42 @@ static void *threadwicked(void *targ){
       }
       break;
     default:
-      if(id == 0) putchar('@');
+      if(id == 0) iputchar('@');
+      if(tcfdbtranbegin(fdb)){
+        if(myrand(2) == 0){
+          if(!tcfdbput2(fdb, kbuf, ksiz, vbuf, vsiz)){
+            eprint(fdb, "tcfdbput");
+            err = true;
+          }
+          if(!nc) tcmapput(map, kbuf, ksiz, vbuf, vsiz);
+        } else {
+          if(!tcfdbout2(fdb, kbuf, ksiz) && tcfdbecode(fdb) != TCENOREC){
+            eprint(fdb, "tcfdbout");
+            err = true;
+          }
+          if(!nc) tcmapout(map, kbuf, ksiz);
+        }
+        if(nc && myrand(2) == 0){
+          if(!tcfdbtranabort(fdb)){
+            eprint(fdb, "tcfdbtranabort");
+            err = true;
+          }
+        } else {
+          if(!tcfdbtrancommit(fdb)){
+            eprint(fdb, "tcfdbtrancommit");
+            err = true;
+          }
+        }
+      } else {
+        eprint(fdb, "tcfdbtranbegin");
+        err = true;
+      }
+      if(myrand(1000) == 0){
+        if(!tcfdbforeach(fdb, iterfunc, NULL)){
+          eprint(fdb, "tcfdbforeach");
+          err = true;
+        }
+      }
       if(myrand(10000) == 0) srand((unsigned int)(tctime() * 1000) % UINT_MAX);
       break;
     }
@@ -1003,7 +1092,7 @@ static void *threadwicked(void *targ){
     if(id == 0){
       if(i % 50 == 0) iprintf(" (%08d)\n", i);
       if(id == 0 && i == rnum / 4){
-        if(!tcfdboptimize(fdb, RECBUFSIZ, -1)){
+        if(!tcfdboptimize(fdb, RECBUFSIZ, -1) && tcfdbecode(fdb) != TCEINVALID){
           eprint(fdb, "tcfdboptimize");
           err = true;
         }
@@ -1096,8 +1185,7 @@ static void *threadtypical(void *targ){
       }
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
