@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The test cases of the fixed-length database API
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -18,12 +18,13 @@
 #include <tcfdb.h>
 #include "myconf.h"
 
-#define RECBUFSIZ      32                // buffer for records
+#define RECBUFSIZ      48                // buffer for records
 #define EXHEADSIZ      256               // expected header size
 
 
 /* global variables */
 const char *g_progname;                  // program name
+unsigned int g_randseed;                 // random seed
 int g_dbgfd;                             // debugging output
 
 
@@ -31,9 +32,13 @@ int g_dbgfd;                             // debugging output
 int main(int argc, char **argv);
 static void usage(void);
 static void iprintf(const char *format, ...);
+static void iputchar(int c);
 static void eprint(TCFDB *fdb, const char *func);
 static void mprint(TCFDB *fdb);
+static void sysprint(void);
 static int myrand(int range);
+static void *pdprocfunc(const void *vbuf, int vsiz, int *sp, void *op);
+static bool iterfunc(const void *kbuf, int ksiz, const void *vbuf, int vsiz, void *op);
 static int runwrite(int argc, char **argv);
 static int runread(int argc, char **argv);
 static int runremove(int argc, char **argv);
@@ -45,7 +50,7 @@ static int procwrite(const char *path, int rnum, int width, int64_t limsiz,
 static int procread(const char *path, bool mt, int omode, bool wb, bool rnd);
 static int procremove(const char *path, bool mt, int omode, bool rnd);
 static int procrcat(const char *path, int rnum, int width, int64_t limsiz,
-                    bool mt, int omode, int pnum, bool rl);
+                    bool mt, int omode, int pnum, bool dai, bool dad, bool rl, bool ru);
 static int procmisc(const char *path, int rnum, bool mt, int omode);
 static int procwicked(const char *path, int rnum, bool mt, int omode);
 
@@ -53,10 +58,11 @@ static int procwicked(const char *path, int rnum, bool mt, int omode);
 /* main routine */
 int main(int argc, char **argv){
   g_progname = argv[0];
-  g_dbgfd = -1;
-  const char *ebuf = getenv("TCDBGFD");
-  if(ebuf) g_dbgfd = atoi(ebuf);
-  srand((unsigned int)(tctime() * 1000) % UINT_MAX);
+  const char *ebuf = getenv("TCRNDSEED");
+  g_randseed = ebuf ? tcatoix(ebuf) : tctime() * 1000;
+  srand(g_randseed);
+  ebuf = getenv("TCDBGFD");
+  g_dbgfd = ebuf ? tcatoix(ebuf) : UINT16_MAX;
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "write")){
@@ -74,6 +80,13 @@ int main(int argc, char **argv){
   } else {
     usage();
   }
+  if(rv != 0){
+    printf("FAILED:");
+    for(int i = 0; i < argc; i++){
+      printf(" %s", argv[i]);
+    }
+    printf("\n\n");
+  }
   return rv;
 }
 
@@ -87,8 +100,8 @@ static void usage(void){
   fprintf(stderr, "  %s write [-mt] [-nl|-nb] [-rnd] path rnum [width [limsiz]]\n", g_progname);
   fprintf(stderr, "  %s read [-mt] [-nl|-nb] [-wb] [-rnd] path\n", g_progname);
   fprintf(stderr, "  %s remove [-mt] [-nl|-nb] [-rnd] path\n", g_progname);
-  fprintf(stderr, "  %s rcat [-mt] [-nl|-nb] [-pn num] [-rl] path rnum [width [limsiz]]\n",
-          g_progname);
+  fprintf(stderr, "  %s rcat [-mt] [-nl|-nb] [-pn num] [-dai|-dad|-rl|-ru]"
+          " path rnum [width [limsiz]]\n", g_progname);
   fprintf(stderr, "  %s misc [-mt] [-nl|-nb] path rnum\n", g_progname);
   fprintf(stderr, "  %s wicked [-mt] [-nl|-nb] path rnum\n", g_progname);
   fprintf(stderr, "\n");
@@ -106,6 +119,13 @@ static void iprintf(const char *format, ...){
 }
 
 
+/* print a character and flush the buffer */
+static void iputchar(int c){
+  putchar(c);
+  fflush(stdout);
+}
+
+
 /* print error message of fixed-length database */
 static void eprint(TCFDB *fdb, const char *func){
   const char *path = tcfdbpath(fdb);
@@ -129,9 +149,68 @@ static void mprint(TCFDB *fdb){
 }
 
 
+/* print system information */
+static void sysprint(void){
+  TCMAP *info = tcsysinfo();
+  if(info){
+    tcmapiterinit(info);
+    const char *kbuf;
+    while((kbuf = tcmapiternext2(info)) != NULL){
+      iprintf("sys_%s: %s\n", kbuf, tcmapiterval2(kbuf));
+    }
+    tcmapdel(info);
+  }
+}
+
+
 /* get a random number */
 static int myrand(int range){
-  return (int)((double)range * rand() / (RAND_MAX + 1.0));
+  if(range < 2) return 0;
+  int high = (unsigned int)rand() >> 4;
+  int low = range * (rand() / (RAND_MAX + 1.0));
+  low &= (unsigned int)INT_MAX >> 4;
+  return (high + low) % range;
+}
+
+
+/* duplication callback function */
+static void *pdprocfunc(const void *vbuf, int vsiz, int *sp, void *op){
+  if(op){
+    char *buf = NULL;
+    int len = 0;
+    switch((int)(intptr_t)op){
+    case 1:
+      len = vsiz + 1;
+      buf = tcmalloc(len + 1);
+      memset(buf, '*', len);
+      break;
+    case 2:
+      buf = (void *)-1;
+      break;
+    }
+    *sp = len;
+    return buf;
+  }
+  if(myrand(4) == 0) return (void *)-1;
+  if(myrand(2) == 0) return NULL;
+  int len = myrand(RECBUFSIZ);
+  char buf[RECBUFSIZ];
+  memset(buf, '*', len);
+  *sp = len;
+  return tcmemdup(buf, len);
+}
+
+
+/* iterator function */
+static bool iterfunc(const void *kbuf, int ksiz, const void *vbuf, int vsiz, void *op){
+  unsigned int sum = 0;
+  while(--ksiz >= 0){
+    sum += ((char *)kbuf)[ksiz];
+  }
+  while(--vsiz >= 0){
+    sum += ((char *)vbuf)[vsiz];
+  }
+  return myrand(100 + (sum & 0xff)) > 0;
 }
 
 
@@ -170,10 +249,10 @@ static int runwrite(int argc, char **argv){
     }
   }
   if(!path || !rstr) usage();
-  int rnum = atoi(rstr);
+  int rnum = tcatoix(rstr);
   if(rnum < 1) usage();
-  int width = wstr ? atoi(wstr) : -1;
-  int64_t limsiz = lstr ? strtoll(lstr, NULL, 10) : -1;
+  int width = wstr ? tcatoix(wstr) : -1;
+  int64_t limsiz = lstr ? tcatoix(lstr) : -1;
   int rv = procwrite(path, rnum, width, limsiz, mt, omode, rnd);
   return rv;
 }
@@ -253,7 +332,10 @@ static int runrcat(int argc, char **argv){
   bool mt = false;
   int omode = 0;
   int pnum = 0;
+  bool dai = false;
+  bool dad = false;
   bool rl = false;
+  bool ru = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-mt")){
@@ -264,9 +346,15 @@ static int runrcat(int argc, char **argv){
         omode |= FDBOLCKNB;
       } else if(!strcmp(argv[i], "-pn")){
         if(++i >= argc) usage();
-        pnum = atoi(argv[i]);
+        pnum = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-dai")){
+        dai = true;
+      } else if(!strcmp(argv[i], "-dad")){
+        dad = true;
       } else if(!strcmp(argv[i], "-rl")){
         rl = true;
+      } else if(!strcmp(argv[i], "-ru")){
+        ru = true;
       } else {
         usage();
       }
@@ -283,11 +371,11 @@ static int runrcat(int argc, char **argv){
     }
   }
   if(!path || !rstr) usage();
-  int rnum = atoi(rstr);
+  int rnum = tcatoix(rstr);
   if(rnum < 1) usage();
-  int width = wstr ? atoi(wstr) : -1;
-  int64_t limsiz = lstr ? strtoll(lstr, NULL, 10) : -1;
-  int rv = procrcat(path, rnum, width, limsiz, mt, omode, pnum, rl);
+  int width = wstr ? tcatoix(wstr) : -1;
+  int64_t limsiz = lstr ? tcatoix(lstr) : -1;
+  int rv = procrcat(path, rnum, width, limsiz, mt, omode, pnum, dai, dad, rl, ru);
   return rv;
 }
 
@@ -318,7 +406,7 @@ static int runmisc(int argc, char **argv){
     }
   }
   if(!path || !rstr) usage();
-  int rnum = atoi(rstr);
+  int rnum = tcatoix(rstr);
   if(rnum < 1) usage();
   int rv = procmisc(path, rnum, mt, omode);
   return rv;
@@ -351,7 +439,7 @@ static int runwicked(int argc, char **argv){
     }
   }
   if(!path || !rstr) usage();
-  int rnum = atoi(rstr);
+  int rnum = tcatoix(rstr);
   if(rnum < 1) usage();
   int rv = procwicked(path, rnum, mt, omode);
   return rv;
@@ -361,8 +449,8 @@ static int runwicked(int argc, char **argv){
 /* perform write command */
 static int procwrite(const char *path, int rnum, int width, int64_t limsiz,
                      bool mt, int omode, bool rnd){
-  iprintf("<Writing Test>\n  path=%s  rnum=%d  width=%d  limsiz=%lld  mt=%d  omode=%d"
-          "  rnd=%d\n\n", path, rnum, width, (long long)limsiz, mt, omode, rnd);
+  iprintf("<Writing Test>\n  seed=%u  path=%s  rnum=%d  width=%d  limsiz=%lld  mt=%d  omode=%d"
+          "  rnd=%d\n\n", g_randseed, path, rnum, width, (long long)limsiz, mt, omode, rnd);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -389,14 +477,14 @@ static int procwrite(const char *path, int rnum, int width, int64_t limsiz,
       break;
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -410,8 +498,8 @@ static int procwrite(const char *path, int rnum, int width, int64_t limsiz,
 
 /* perform read command */
 static int procread(const char *path, bool mt, int omode, bool wb, bool rnd){
-  iprintf("<Reading Test>\n  path=%s  mt=%d  omode=%d  wb=%d  rnd=%d\n\n",
-          path, mt, omode, wb, rnd);
+  iprintf("<Reading Test>\n  seed=%u  path=%s  mt=%d  omode=%d  wb=%d  rnd=%d\n\n",
+          g_randseed, path, mt, omode, wb, rnd);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -447,14 +535,14 @@ static int procread(const char *path, bool mt, int omode, bool wb, bool rnd){
       tcfree(vbuf);
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -468,7 +556,8 @@ static int procread(const char *path, bool mt, int omode, bool wb, bool rnd){
 
 /* perform remove command */
 static int procremove(const char *path, bool mt, int omode, bool rnd){
-  iprintf("<Removing Test>\n  path=%s  mt=%d  omode=%d  rnd=%d\n\n", path, mt, omode, rnd);
+  iprintf("<Removing Test>\n  seed=%u  path=%s  mt=%d  omode=%d  rnd=%d\n\n",
+          g_randseed, path, mt, omode, rnd);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -491,14 +580,14 @@ static int procremove(const char *path, bool mt, int omode, bool rnd){
       break;
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -512,10 +601,11 @@ static int procremove(const char *path, bool mt, int omode, bool rnd){
 
 /* perform rcat command */
 static int procrcat(const char *path, int rnum, int width, int64_t limsiz,
-                    bool mt, int omode, int pnum, bool rl){
+                    bool mt, int omode, int pnum, bool dai, bool dad, bool rl, bool ru){
   iprintf("<Random Concatenating Test>\n"
-          "  path=%s  rnum=%d  width=%d  limsiz=%lld  mt=%d  omode=%d  pnum=%d  rl=%d\n\n",
-          path, rnum, width, (long long)limsiz, mt, omode, pnum, rl);
+          "  seed=%u  path=%s  rnum=%d  width=%d  limsiz=%lld  mt=%d  omode=%d  pnum=%d"
+          "  dai=%d  dad=%d  rl=%d  ru=%d\n\n",
+          g_randseed, path, rnum, width, (long long)limsiz, mt, omode, pnum, dai, dad, rl, ru);
   if(pnum < 1) pnum = rnum;
   bool err = false;
   double stime = tctime();
@@ -536,7 +626,19 @@ static int procrcat(const char *path, int rnum, int width, int64_t limsiz,
   for(int i = 1; i <= rnum; i++){
     char kbuf[RECBUFSIZ];
     int ksiz = sprintf(kbuf, "%d", myrand(pnum) + 1);
-    if(rl){
+    if(dai){
+      if(tcfdbaddint(fdb, tcfdbkeytoid(kbuf, ksiz), myrand(3)) == INT_MIN){
+        eprint(fdb, "tcfdbaddint");
+        err = true;
+        break;
+      }
+    } else if(dad){
+      if(isnan(tcfdbadddouble(fdb, tcfdbkeytoid(kbuf, ksiz), myrand(30) / 10.0))){
+        eprint(fdb, "tcfdbadddouble");
+        err = true;
+        break;
+      }
+    } else if(rl){
       char vbuf[PATH_MAX];
       int vsiz = myrand(PATH_MAX);
       for(int j = 0; j < vsiz; j++){
@@ -547,6 +649,62 @@ static int procrcat(const char *path, int rnum, int width, int64_t limsiz,
         err = true;
         break;
       }
+    } else if(ru){
+      int id = myrand(pnum) + 1;
+      switch(myrand(8)){
+      case 0:
+        if(!tcfdbput(fdb, id, kbuf, ksiz)){
+          eprint(fdb, "tcfdbput");
+          err = true;
+        }
+        break;
+      case 1:
+        if(!tcfdbputkeep(fdb, id, kbuf, ksiz) && tcfdbecode(fdb) != TCEKEEP){
+          eprint(fdb, "tcfdbputkeep");
+          err = true;
+        }
+        break;
+      case 2:
+        if(!tcfdbout(fdb, id) && tcfdbecode(fdb) != TCENOREC){
+          eprint(fdb, "tcfdbout");
+          err = true;
+        }
+        break;
+      case 3:
+        if(tcfdbaddint(fdb, id, 1) == INT_MIN && tcfdbecode(fdb) != TCEKEEP){
+          eprint(fdb, "tcfdbaddint");
+          err = true;
+        }
+        break;
+      case 4:
+        if(isnan(tcfdbadddouble(fdb, id, 1.0)) && tcfdbecode(fdb) != TCEKEEP){
+          eprint(fdb, "tcfdbadddouble");
+          err = true;
+        }
+        break;
+      case 5:
+        if(myrand(2) == 0){
+          if(!tcfdbputproc(fdb, id, kbuf, ksiz, pdprocfunc, NULL) &&
+             tcfdbecode(fdb) != TCEKEEP){
+            eprint(fdb, "tcfdbputproc");
+            err = true;
+          }
+        } else {
+          if(!tcfdbputproc(fdb, id, NULL, 0, pdprocfunc, NULL) &&
+             tcfdbecode(fdb) != TCEKEEP && tcfdbecode(fdb) != TCENOREC){
+            eprint(fdb, "tcfdbputproc");
+            err = true;
+          }
+        }
+        break;
+      default:
+        if(!tcfdbputcat(fdb, id, kbuf, ksiz)){
+          eprint(fdb, "tcfdbputcat");
+          err = true;
+        }
+        break;
+      }
+      if(err) break;
     } else {
       if(!tcfdbputcat2(fdb, kbuf, ksiz, kbuf, ksiz)){
         eprint(fdb, "tcfdbputcat");
@@ -555,14 +713,14 @@ static int procrcat(const char *path, int rnum, int width, int64_t limsiz,
       }
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -576,7 +734,8 @@ static int procrcat(const char *path, int rnum, int width, int64_t limsiz,
 
 /* perform misc command */
 static int procmisc(const char *path, int rnum, bool mt, int omode){
-  iprintf("<Miscellaneous Test>\n  path=%s  rnum=%d  mt=%d  omode=%d\n\n", path, rnum, mt, omode);
+  iprintf("<Miscellaneous Test>\n  seed=%u  path=%s  rnum=%d  mt=%d  omode=%d\n\n",
+          g_randseed, path, rnum, mt, omode);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -603,8 +762,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
       break;
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -626,8 +784,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
     }
     tcfree(vbuf);
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -661,8 +818,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
       break;
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
     tcfree(rbuf);
@@ -677,8 +833,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
       break;
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -706,8 +861,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
       }
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -728,8 +882,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
       }
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -762,8 +915,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
     }
     tcfree(vbuf);
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -789,8 +941,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
       break;
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -804,8 +955,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
       break;
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -827,12 +977,11 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
     }
     tcfree(vbuf);
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
-  iprintf("iterator checking:\n");
+  iprintf("checking iterator:\n");
   if(!tcfdbiterinit(fdb)){
     eprint(fdb, "tcfdbiterinit");
     err = true;
@@ -852,8 +1001,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
     tcfree(vbuf);
     tcfree(kbuf);
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -885,8 +1033,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
     }
     tcfree(kbuf);
     if(rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -894,7 +1041,7 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
     eprint(fdb, "(validation)");
     err = true;
   }
-  if(!tcfdbsync(fdb)){
+  if(myrand(10) == 0 && !tcfdbsync(fdb)){
     eprint(fdb, "tcfdbsync");
     err = true;
   }
@@ -902,9 +1049,302 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
     eprint(fdb, "tcfdbvanish");
     err = true;
   }
+  TCMAP *map = tcmapnew();
+  iprintf("random writing:\n");
+  for(int i = 1; i <= rnum; i++){
+    char kbuf[RECBUFSIZ];
+    int ksiz = sprintf(kbuf, "%d", myrand(rnum) + 1);
+    char vbuf[RECBUFSIZ];
+    int vsiz = sprintf(vbuf, "%d", myrand(rnum) + 1);
+    switch(myrand(7)){
+    case 0:
+      if(!tcfdbput2(fdb, kbuf, ksiz, vbuf, vsiz)){
+        eprint(fdb, "tcfdbput2");
+        err = true;
+      }
+      tcmapput(map, kbuf, ksiz, vbuf, vsiz);
+      break;
+    case 1:
+      if(!tcfdbputkeep2(fdb, kbuf, ksiz, vbuf, vsiz) && tcfdbecode(fdb) != TCEKEEP){
+        eprint(fdb, "tcfdbputkeep2");
+        err = true;
+      }
+      tcmapputkeep(map, kbuf, ksiz, vbuf, vsiz);
+      break;
+    case 2:
+      if(!tcfdbputcat2(fdb, kbuf, ksiz, vbuf, vsiz) && tcfdbecode(fdb) != TCEKEEP){
+        eprint(fdb, "tcfdbputcat2");
+        err = true;
+      }
+      tcmapputcat(map, kbuf, ksiz, vbuf, vsiz);
+      break;
+    case 3:
+      if(!tcfdbout2(fdb, kbuf, ksiz) && tcfdbecode(fdb) != TCENOREC){
+        eprint(fdb, "tcfdbout2");
+        err = true;
+      }
+      tcmapout(map, kbuf, ksiz);
+      break;
+    }
+    if(rnum > 250 && i % (rnum / 250) == 0){
+      iputchar('.');
+      if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
+    }
+  }
+  iprintf("checking transaction commit:\n");
+  if(!tcfdbtranbegin(fdb)){
+    eprint(fdb, "tcfdbtranbegin");
+    err = true;
+  }
+  for(int i = 1; i <= rnum; i++){
+    char kbuf[RECBUFSIZ];
+    int ksiz = sprintf(kbuf, "%d", myrand(rnum) + 1);
+    char vbuf[RECBUFSIZ];
+    int vsiz = sprintf(vbuf, "[%d]", myrand(rnum) + 1);
+    switch(myrand(7)){
+    case 0:
+      if(!tcfdbput2(fdb, kbuf, ksiz, vbuf, vsiz)){
+        eprint(fdb, "tcfdbput2");
+        err = true;
+      }
+      tcmapput(map, kbuf, ksiz, vbuf, vsiz);
+      break;
+    case 1:
+      if(!tcfdbputkeep2(fdb, kbuf, ksiz, vbuf, vsiz) && tcfdbecode(fdb) != TCEKEEP){
+        eprint(fdb, "tcfdbputkeep2");
+        err = true;
+      }
+      tcmapputkeep(map, kbuf, ksiz, vbuf, vsiz);
+      break;
+    case 2:
+      if(!tcfdbputcat2(fdb, kbuf, ksiz, vbuf, vsiz)){
+        eprint(fdb, "tcfdbputcat2");
+        err = true;
+      }
+      tcmapputcat(map, kbuf, ksiz, vbuf, vsiz);
+      break;
+    case 3:
+      if(tcfdbaddint(fdb, tcfdbkeytoid(kbuf, ksiz), 1) == INT_MIN && tcfdbecode(fdb) != TCEKEEP){
+        eprint(fdb, "tcfdbaddint");
+        err = true;
+      }
+      tcmapaddint(map, kbuf, ksiz, 1);
+      break;
+    case 4:
+      if(isnan(tcfdbadddouble(fdb, tcfdbkeytoid(kbuf, ksiz), 1.0)) && tcfdbecode(fdb) != TCEKEEP){
+        eprint(fdb, "tcfdbadddouble");
+        err = true;
+      }
+      tcmapadddouble(map, kbuf, ksiz, 1.0);
+      break;
+    case 5:
+      if(myrand(2) == 0){
+        void *op = (void *)(intptr_t)(myrand(3) + 1);
+        if(!tcfdbputproc(fdb, tcfdbkeytoid(kbuf, ksiz), vbuf, vsiz, pdprocfunc, op) &&
+           tcfdbecode(fdb) != TCEKEEP){
+          eprint(fdb, "tcfdbputproc");
+          err = true;
+        }
+        tcmapputproc(map, kbuf, ksiz, vbuf, vsiz, pdprocfunc, op);
+      } else {
+        vsiz = myrand(10);
+        void *op = (void *)(intptr_t)(myrand(3) + 1);
+        if(!tcfdbputproc(fdb, tcfdbkeytoid(kbuf, ksiz), NULL, vsiz, pdprocfunc, op) &&
+           tcfdbecode(fdb) != TCEKEEP && tcfdbecode(fdb) != TCENOREC){
+          eprint(fdb, "tcfdbputproc");
+          err = true;
+        }
+        tcmapputproc(map, kbuf, ksiz, NULL, vsiz, pdprocfunc, op);
+      }
+      break;
+    case 6:
+      if(!tcfdbout2(fdb, kbuf, ksiz) && tcfdbecode(fdb) != TCENOREC){
+        eprint(fdb, "tcfdbout2");
+        err = true;
+      }
+      tcmapout(map, kbuf, ksiz);
+      break;
+    }
+    if(rnum > 250 && i % (rnum / 250) == 0){
+      iputchar('.');
+      if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
+    }
+  }
+  if(!tcfdbtrancommit(fdb)){
+    eprint(fdb, "tcfdbtrancommit");
+    err = true;
+  }
+  iprintf("checking transaction abort:\n");
+  uint64_t ornum = tcfdbrnum(fdb);
+  uint64_t ofsiz = tcfdbfsiz(fdb);
+  if(!tcfdbtranbegin(fdb)){
+    eprint(fdb, "tcfdbtranbegin");
+    err = true;
+  }
+  for(int i = 1; i <= rnum; i++){
+    char kbuf[RECBUFSIZ];
+    int ksiz = sprintf(kbuf, "%d", myrand(rnum) + 1);
+    char vbuf[RECBUFSIZ];
+    int vsiz = sprintf(vbuf, "[%d]", myrand(rnum) + 1);
+    switch(myrand(7)){
+    case 0:
+      if(!tcfdbput2(fdb, kbuf, ksiz, vbuf, vsiz)){
+        eprint(fdb, "tcfdbput2");
+        err = true;
+      }
+      break;
+    case 1:
+      if(!tcfdbputkeep2(fdb, kbuf, ksiz, vbuf, vsiz) && tcfdbecode(fdb) != TCEKEEP){
+        eprint(fdb, "tcfdbputkeep2");
+        err = true;
+      }
+      break;
+    case 2:
+      if(!tcfdbputcat2(fdb, kbuf, ksiz, vbuf, vsiz)){
+        eprint(fdb, "tcfdbputcat2");
+        err = true;
+      }
+      break;
+    case 3:
+      if(tcfdbaddint(fdb, tcfdbkeytoid(kbuf, ksiz), 1) == INT_MIN && tcfdbecode(fdb) != TCEKEEP){
+        eprint(fdb, "tcfdbaddint");
+        err = true;
+      }
+      break;
+    case 4:
+      if(isnan(tcfdbadddouble(fdb, tcfdbkeytoid(kbuf, ksiz), 1.0)) && tcfdbecode(fdb) != TCEKEEP){
+        eprint(fdb, "tcfdbadddouble");
+        err = true;
+      }
+      break;
+    case 5:
+      if(myrand(2) == 0){
+        void *op = (void *)(intptr_t)(myrand(3) + 1);
+        if(!tcfdbputproc(fdb, tcfdbkeytoid(kbuf, ksiz), vbuf, vsiz, pdprocfunc, op) &&
+           tcfdbecode(fdb) != TCEKEEP){
+          eprint(fdb, "tcfdbputproc");
+          err = true;
+        }
+      } else {
+        vsiz = myrand(10);
+        void *op = (void *)(intptr_t)(myrand(3) + 1);
+        if(!tcfdbputproc(fdb, tcfdbkeytoid(kbuf, ksiz), NULL, vsiz, pdprocfunc, op) &&
+           tcfdbecode(fdb) != TCEKEEP && tcfdbecode(fdb) != TCENOREC){
+          eprint(fdb, "tcfdbputproc");
+          err = true;
+        }
+      }
+      break;
+    case 6:
+      if(!tcfdbout2(fdb, kbuf, ksiz) && tcfdbecode(fdb) != TCENOREC){
+        eprint(fdb, "tcfdbout2");
+        err = true;
+      }
+      break;
+    }
+    if(rnum > 250 && i % (rnum / 250) == 0){
+      iputchar('.');
+      if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
+    }
+  }
+  if(!tcfdbtranabort(fdb)){
+    eprint(fdb, "tcfdbtranabort");
+    err = true;
+  }
+  iprintf("checking consistency:\n");
+  if(tcfdbrnum(fdb) != ornum || tcfdbfsiz(fdb) != ofsiz || tcfdbrnum(fdb) != tcmaprnum(map)){
+    eprint(fdb, "(validation)");
+    err = true;
+  }
+  inum = 0;
+  tcmapiterinit(map);
+  const char *tkbuf;
+  int tksiz;
+  for(int i = 1; (tkbuf = tcmapiternext(map, &tksiz)) != NULL; i++, inum++){
+    int tvsiz;
+    const char *tvbuf = tcmapiterval(tkbuf, &tvsiz);
+    if(tvsiz > RECBUFSIZ) tvsiz = RECBUFSIZ;
+    int rsiz;
+    char *rbuf = tcfdbget2(fdb, tkbuf, tksiz, &rsiz);
+    if(!rbuf || rsiz != tvsiz || memcmp(rbuf, tvbuf, rsiz)){
+      eprint(fdb, "(validation)");
+      err = true;
+      tcfree(rbuf);
+      break;
+    }
+    tcfree(rbuf);
+    if(rnum > 250 && i % (rnum / 250) == 0){
+      iputchar('.');
+      if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
+    }
+  }
+  if(rnum > 250) iprintf(" (%08d)\n", inum);
+  inum = 0;
+  if(!tcfdbiterinit(fdb)){
+    eprint(fdb, "tcfdbiterinit");
+    err = true;
+  }
+  for(int i = 1; (kbuf = tcfdbiternext2(fdb, &ksiz)) != NULL; i++, inum++){
+    int vsiz;
+    char *vbuf = tcfdbget2(fdb, kbuf, ksiz, &vsiz);
+    int rsiz;
+    const char *rbuf = tcmapget(map, kbuf, ksiz, &rsiz);
+    if(rsiz > RECBUFSIZ) rsiz = RECBUFSIZ;
+    if(!rbuf || rsiz != vsiz || memcmp(rbuf, vbuf, rsiz)){
+      eprint(fdb, "(validation)");
+      err = true;
+      tcfree(vbuf);
+      tcfree(kbuf);
+      break;
+    }
+    tcfree(vbuf);
+    tcfree(kbuf);
+    if(rnum > 250 && i % (rnum / 250) == 0){
+      iputchar('.');
+      if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
+    }
+  }
+  if(rnum > 250) iprintf(" (%08d)\n", inum);
+  tcmapdel(map);
+  if(!tcfdbvanish(fdb)){
+    eprint(fdb, "tcfdbvanish");
+    err = true;
+  }
+  if(rnum >= 100){
+    if(!tcfdbtranbegin(fdb)){
+      eprint(fdb, "tcfdbtranbegin");
+      err = true;
+    }
+    if(!tcfdbput3(fdb, "99", "hirabayashi")){
+      eprint(fdb, "tcfdbput3");
+      err = true;
+    }
+    for(int i = 0; i < 10; i++){
+      char buf[RECBUFSIZ];
+      int size = sprintf(buf, "%d", myrand(rnum) + 1);
+      if(!tcfdbput2(fdb, buf, size, buf, size)){
+        eprint(fdb, "tcfdbput2");
+        err = true;
+      }
+    }
+    for(int i = myrand(3) + 1; i < PATH_MAX; i = i * 2 + myrand(3)){
+      char vbuf[i];
+      memset(vbuf, '@', i - 1);
+      vbuf[i-1] = '\0';
+      if(!tcfdbput3(fdb, "99", vbuf)){
+        eprint(fdb, "tcfdbput3");
+        err = true;
+      }
+    }
+    if(!tcfdbforeach(fdb, iterfunc, NULL)){
+      eprint(fdb, "tcfdbforeach");
+      err = true;
+    }
+  }
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
     err = true;
@@ -918,8 +1358,8 @@ static int procmisc(const char *path, int rnum, bool mt, int omode){
 
 /* perform wicked command */
 static int procwicked(const char *path, int rnum, bool mt, int omode){
-  iprintf("<Wicked Writing Test>\n  path=%s  rnum=%d  mt=%d  omode=%d\n\n",
-          path, rnum, mt, omode);
+  iprintf("<Wicked Writing Test>\n  seed=%u  path=%s  rnum=%d  mt=%d  omode=%d\n\n",
+          g_randseed, path, rnum, mt, omode);
   bool err = false;
   double stime = tctime();
   TCFDB *fdb = tcfdbnew();
@@ -952,7 +1392,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
     char *rbuf;
     switch(myrand(16)){
     case 0:
-      putchar('0');
+      iputchar('0');
       if(!tcfdbput2(fdb, kbuf, ksiz, vbuf, vsiz)){
         eprint(fdb, "tcfdbput2");
         err = true;
@@ -960,7 +1400,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcmapput(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 1:
-      putchar('1');
+      iputchar('1');
       if(!tcfdbput3(fdb, kbuf, vbuf)){
         eprint(fdb, "tcfdbput3");
         err = true;
@@ -968,7 +1408,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcmapput2(map, kbuf, vbuf);
       break;
     case 2:
-      putchar('2');
+      iputchar('2');
       if(!tcfdbputkeep2(fdb, kbuf, ksiz, vbuf, vsiz) && tcfdbecode(fdb) != TCEKEEP){
         eprint(fdb, "tcfdbputkeep2");
         err = true;
@@ -976,7 +1416,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcmapputkeep(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 3:
-      putchar('3');
+      iputchar('3');
       if(!tcfdbputkeep3(fdb, kbuf, vbuf) && tcfdbecode(fdb) != TCEKEEP){
         eprint(fdb, "tcfdbputkeep3");
         err = true;
@@ -984,7 +1424,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcmapputkeep2(map, kbuf, vbuf);
       break;
     case 4:
-      putchar('4');
+      iputchar('4');
       if(!tcfdbputcat2(fdb, kbuf, ksiz, vbuf, vsiz)){
         eprint(fdb, "tcfdbputcat2");
         err = true;
@@ -992,7 +1432,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcmapputcat(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 5:
-      putchar('5');
+      iputchar('5');
       if(!tcfdbputcat3(fdb, kbuf, vbuf)){
         eprint(fdb, "tcfdbputcat3");
         err = true;
@@ -1000,7 +1440,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcmapputcat2(map, kbuf, vbuf);
       break;
     case 6:
-      putchar('6');
+      iputchar('6');
       if(myrand(10) == 0){
         if(!tcfdbout2(fdb, kbuf, ksiz) && tcfdbecode(fdb) != TCENOREC){
           eprint(fdb, "tcfdbout2");
@@ -1010,7 +1450,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       }
       break;
     case 7:
-      putchar('7');
+      iputchar('7');
       if(myrand(10) == 0){
         if(!tcfdbout3(fdb, kbuf) && tcfdbecode(fdb) != TCENOREC){
           eprint(fdb, "tcfdbout3");
@@ -1020,7 +1460,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       }
       break;
     case 8:
-      putchar('8');
+      iputchar('8');
       if(!(rbuf = tcfdbget2(fdb, kbuf, ksiz, &vsiz))){
         if(tcfdbecode(fdb) != TCENOREC){
           eprint(fdb, "tcfdbget2");
@@ -1042,7 +1482,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcfree(rbuf);
       break;
     case 9:
-      putchar('9');
+      iputchar('9');
       if(!(rbuf = tcfdbget2(fdb, kbuf, ksiz, &vsiz)) && tcfdbecode(fdb) != TCENOREC){
         eprint(fdb, "tcfdbget2");
         err = true;
@@ -1050,7 +1490,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcfree(rbuf);
       break;
     case 10:
-      putchar('A');
+      iputchar('A');
       if(!(rbuf = tcfdbget3(fdb, kbuf)) && tcfdbecode(fdb) != TCENOREC){
         eprint(fdb, "tcfdbget3");
         err = true;
@@ -1058,7 +1498,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       tcfree(rbuf);
       break;
     case 11:
-      putchar('B');
+      iputchar('B');
       if(myrand(1) == 0) vsiz = 1;
       if((vsiz = tcfdbget4(fdb, id, vbuf, vsiz)) < 0 && tcfdbecode(fdb) != TCENOREC){
         eprint(fdb, "tcfdbget4");
@@ -1066,11 +1506,18 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       }
       break;
     case 12:
-      putchar('C');
-      if(myrand(rnum / 50) == 0){
-        if(!tcfdbiterinit(fdb)){
-          eprint(fdb, "tcfdbiterinit");
-          err = true;
+      iputchar('C');
+      if(myrand(rnum / 128) == 0){
+        if(myrand(2) == 0){
+          if(!tcfdbiterinit(fdb)){
+            eprint(fdb, "tcfdbiterinit");
+            err = true;
+          }
+        } else {
+          if(!tcfdbiterinit2(fdb, myrand(rnum) + 1) && tcfdbecode(fdb) != TCENOREC){
+            eprint(fdb, "tcfdbiterinit2");
+            err = true;
+          }
         }
       }
       for(int j = myrand(rnum) / 1000 + 1; j >= 0; j--){
@@ -1084,7 +1531,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
       }
       break;
     default:
-      putchar('@');
+      iputchar('@');
       if(myrand(10000) == 0) srand((unsigned int)(tctime() * 1000) % UINT_MAX);
       if(myrand(rnum / 16 + 1) == 0){
         int cnt = myrand(30);
@@ -1093,7 +1540,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
           if(tcfdbout2(fdb, kbuf, ksiz)){
             cnt--;
           } else if(tcfdbecode(fdb) != TCENOREC){
-            eprint(fdb, "tcbdbout2");
+            eprint(fdb, "tcfdbout2");
             err = true;
           }
           tcmapout(map, kbuf, ksiz);
@@ -1134,6 +1581,16 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
         eprint(fdb, "tcfdbiterinit");
         err = true;
       }
+    } else if(i == rnum / 8){
+      if(!tcfdbtranbegin(fdb)){
+        eprint(fdb, "tcfdbtranbegin");
+        err = true;
+      }
+    } else if(i == rnum / 8 + rnum / 16){
+      if(!tcfdbtrancommit(fdb)){
+        eprint(fdb, "tcfdbtrancommit");
+        err = true;
+      }
     }
   }
   if(rnum % 50 > 0) iprintf(" (%08d)\n", rnum);
@@ -1153,7 +1610,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
     int rsiz;
     char *rbuf = tcfdbget2(fdb, kbuf, ksiz, &rsiz);
     if(vbuf){
-      putchar('.');
+      iputchar('.');
       if(vsiz > RECBUFSIZ) vsiz = RECBUFSIZ;
       if(!rbuf){
         eprint(fdb, "tcfdbget2");
@@ -1163,7 +1620,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
         err = true;
       }
     } else {
-      putchar('*');
+      iputchar('*');
       if(rbuf || tcfdbecode(fdb) != TCENOREC){
         eprint(fdb, "(validation)");
         err = true;
@@ -1177,10 +1634,10 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
   int ksiz;
   const char *kbuf;
   for(int i = 1; (kbuf = tcmapiternext(map, &ksiz)) != NULL; i++){
-    putchar('+');
+    iputchar('+');
     int vsiz;
     const char *vbuf = tcmapiterval(kbuf, &vsiz);
-    if(vsiz > RECBUFSIZ) vsiz = RECBUFSIZ;
+    if(vsiz > tcfdbwidth(fdb)) vsiz = tcfdbwidth(fdb);
     int rsiz;
     char *rbuf = tcfdbget2(fdb, kbuf, ksiz, &rsiz);
     if(!rbuf){
@@ -1206,6 +1663,7 @@ static int procwicked(const char *path, int rnum, bool mt, int omode){
   iprintf("record number: %llu\n", (unsigned long long)tcfdbrnum(fdb));
   iprintf("size: %llu\n", (unsigned long long)tcfdbfsiz(fdb));
   mprint(fdb);
+  sysprint();
   tcmapdel(map);
   if(!tcfdbclose(fdb)){
     eprint(fdb, "tcfdbclose");
