@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The hash database API of Tokyo Cabinet
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -31,6 +31,8 @@ __TCHDB_CLINKAGEBEGIN
 #include <stdbool.h>
 #include <stdint.h>
 #include <time.h>
+#include <limits.h>
+#include <math.h>
 #include <tcutil.h>
 
 
@@ -40,21 +42,12 @@ __TCHDB_CLINKAGEBEGIN
  *************************************************************************************************/
 
 
-/* type of the pointer to a encoding or decoding function.
-   `ptr' specifies the pointer to the region.
-   `size' specifies the size of the region.
-   `sp' specifies the pointer to the variable into which the size of the region of the return
-   value is assigned.
-   `op' specifies the pointer to the optional opaque object.
-   If successful, the return value is the pointer to the result object allocated with `malloc'
-   call, else, it is `NULL'. */
-typedef void *(*HDBCODEC)(const void *ptr, int size, int *sp, void *op);
-
 typedef struct {                         /* type of structure for a hash database */
   void *mmtx;                            /* mutex for method */
   void *rmtxs;                           /* mutexes for records */
-  void *amtx;                            /* mutex for attributes */
-  void *dmtx;                            /* mutex for DRP */
+  void *dmtx;                            /* mutex for the while database */
+  void *tmtx;                            /* mutex for transaction */
+  void *wmtx;                            /* mutex for write ahead logging */
   void *eckey;                           /* key for thread specific error code */
   uint8_t type;                          /* database type */
   uint8_t flags;                         /* additional flags */
@@ -64,21 +57,22 @@ typedef struct {                         /* type of structure for a hash databas
   uint8_t opts;                          /* options */
   char *path;                            /* path of the database file */
   int fd;                                /* file descriptor of the database file */
-  uint32_t omode;                        /* connection mode */
+  uint32_t omode;                        /* open mode */
   uint64_t rnum;                         /* number of the records */
   uint64_t fsiz;                         /* size of the database file */
   uint64_t frec;                         /* offset of the first record */
-  uint64_t lrec;                         /* offset of the last record */
+  uint64_t dfcur;                        /* offset of the cursor for defragmentation */
   uint64_t iter;                         /* offset of the iterator */
   char *map;                             /* pointer to the mapped memory */
   uint64_t msiz;                         /* size of the mapped memory */
+  uint64_t xmsiz;                        /* size of the extra mapped memory */
+  uint64_t xfsiz;                        /* extra size of the file for mapped memory */
   uint32_t *ba32;                        /* 32-bit bucket array */
   uint64_t *ba64;                        /* 64-bit bucket array */
   uint32_t align;                        /* record alignment */
   uint32_t runit;                        /* record reading unit */
   bool zmode;                            /* whether compression is used */
   int32_t fbpmax;                        /* maximum number of the free block pool */
-  int32_t fbpsiz;                        /* size of the free block pool */
   void *fbpool;                          /* free block pool */
   int32_t fbpnum;                        /* number of the free block pool */
   int32_t fbpmis;                        /* number of missing retrieval of the free block pool */
@@ -87,15 +81,20 @@ typedef struct {                         /* type of structure for a hash databas
   TCXSTR *drpdef;                        /* deferred records of the delayed record pool */
   uint64_t drpoff;                       /* offset of the delayed record pool */
   TCMDB *recc;                           /* cache for records */
-  uint32_t rcnum;                        /* max number of cached records */
-  HDBCODEC enc;                          /* pointer to the encoding function */
+  uint32_t rcnum;                        /* maximum number of cached records */
+  TCCODEC enc;                           /* pointer to the encoding function */
   void *encop;                           /* opaque object for the encoding functions */
-  HDBCODEC dec;                          /* pointer to the decoding function */
+  TCCODEC dec;                           /* pointer to the decoding function */
   void *decop;                           /* opaque object for the decoding functions */
   int ecode;                             /* last happened error code */
   bool fatal;                            /* whether a fatal error occured */
   uint64_t inode;                        /* inode number */
   time_t mtime;                          /* modification time */
+  uint32_t dfunit;                       /* unit step number of auto defragmentation */
+  uint32_t dfcnt;                        /* counter of auto defragmentation */
+  bool tran;                             /* whether in the transaction */
+  int walfd;                             /* file descriptor of write ahead logging */
+  uint64_t walend;                       /* end offset of write ahead logging */
   int dbgfd;                             /* file descriptor for debugging */
   int64_t cnt_writerec;                  /* tesing counter for record write times */
   int64_t cnt_reuserec;                  /* tesing counter for record reuse times */
@@ -111,6 +110,9 @@ typedef struct {                         /* type of structure for a hash databas
   int64_t cnt_deferdrp;                  /* tesing counter for DRP defer times */
   int64_t cnt_flushdrp;                  /* tesing counter for DRP flush times */
   int64_t cnt_adjrecc;                   /* tesing counter for record cache adjust times */
+  int64_t cnt_defrag;                    /* tesing counter for defragmentation times */
+  int64_t cnt_shiftrec;                  /* tesing counter for record shift times */
+  int64_t cnt_trunc;                     /* tesing counter for truncation times */
 } TCHDB;
 
 enum {                                   /* enumeration for additional flags */
@@ -132,7 +134,8 @@ enum {                                   /* enumeration for open modes */
   HDBOCREAT = 1 << 2,                    /* writer creating */
   HDBOTRUNC = 1 << 3,                    /* writer truncating */
   HDBONOLCK = 1 << 4,                    /* open without locking */
-  HDBOLCKNB = 1 << 5                     /* lock without blocking */
+  HDBOLCKNB = 1 << 5,                    /* lock without blocking */
+  HDBOTSYNC = 1 << 6                     /* synchronize every transaction */
 };
 
 
@@ -157,7 +160,7 @@ void tchdbdel(TCHDB *hdb);
 /* Get the last happened error code of a hash database object.
    `hdb' specifies the hash database object.
    The return value is the last happened error code.
-   The following error code is defined: `TCESUCCESS' for success, `TCETHREAD' for threading
+   The following error codes are defined: `TCESUCCESS' for success, `TCETHREAD' for threading
    error, `TCEINVALID' for invalid operation, `TCENOFILE' for file not found, `TCENOPERM' for no
    permission, `TCEMETA' for invalid meta data, `TCERHEAD' for invalid record header, `TCEOPEN'
    for open error, `TCECLOSE' for close error, `TCETRUNC' for trunc error, `TCESYNC' for sync
@@ -180,13 +183,13 @@ bool tchdbsetmutex(TCHDB *hdb);
 /* Set the tuning parameters of a hash database object.
    `hdb' specifies the hash database object which is not opened.
    `bnum' specifies the number of elements of the bucket array.  If it is not more than 0, the
-   default value is specified.  The default value is 16381.  Suggested size of the bucket array
+   default value is specified.  The default value is 131071.  Suggested size of the bucket array
    is about from 0.5 to 4 times of the number of all records to be stored.
    `apow' specifies the size of record alignment by power of 2.  If it is negative, the default
    value is specified.  The default value is 4 standing for 2^4=16.
    `fpow' specifies the maximum number of elements of the free block pool by power of 2.  If it
    is negative, the default value is specified.  The default value is 10 standing for 2^10=1024.
-   `opts' specifies options by bitwise or: `HDBTLARGE' specifies that the size of the database
+   `opts' specifies options by bitwise-or: `HDBTLARGE' specifies that the size of the database
    can be larger than 2GB by using 64-bit bucket array, `HDBTDEFLATE' specifies that each record
    is compressed with Deflate encoding, `HDBTBZIP' specifies that each record is compressed with
    BZIP2 encoding, `HDBTTCBS' specifies that each record is compressed with TCBS encoding.
@@ -204,14 +207,33 @@ bool tchdbtune(TCHDB *hdb, int64_t bnum, int8_t apow, int8_t fpow, uint8_t opts)
 bool tchdbsetcache(TCHDB *hdb, int32_t rcnum);
 
 
+/* Set the size of the extra mapped memory of a hash database object.
+   `hdb' specifies the hash database object which is not opened.
+   `xmsiz' specifies the size of the extra mapped memory.  If it is not more than 0, the extra
+   mapped memory is disabled.  The default size is 67108864.
+   If successful, the return value is true, else, it is false.
+   Note that the mapping parameters should be set before the database is opened. */
+bool tchdbsetxmsiz(TCHDB *hdb, int64_t xmsiz);
+
+
+/* Set the unit step number of auto defragmentation of a hash database object.
+   `hdb' specifies the hash database object which is not opened.
+   `dfunit' specifie the unit step number.  If it is not more than 0, the auto defragmentation
+   is disabled.  It is disabled by default.
+   If successful, the return value is true, else, it is false.
+   Note that the defragmentation parameters should be set before the database is opened. */
+bool tchdbsetdfunit(TCHDB *hdb, int32_t dfunit);
+
+
 /* Open a database file and connect a hash database object.
    `hdb' specifies the hash database object which is not opened.
    `path' specifies the path of the database file.
    `omode' specifies the connection mode: `HDBOWRITER' as a writer, `HDBOREADER' as a reader.
-   If the mode is `HDBOWRITER', the following may be added by bitwise or: `HDBOCREAT', which
+   If the mode is `HDBOWRITER', the following may be added by bitwise-or: `HDBOCREAT', which
    means it creates a new database if not exist, `HDBOTRUNC', which means it creates a new
-   database regardless if one exists.  Both of `HDBOREADER' and `HDBOWRITER' can be added to by
-   bitwise or: `HDBONOLCK', which means it opens the database file without file locking, or
+   database regardless if one exists, `HDBOTSYNC', which means every transaction synchronizes
+   updated contents with the device.  Both of `HDBOREADER' and `HDBOWRITER' can be added to by
+   bitwise-or: `HDBONOLCK', which means it opens the database file without file locking, or
    `HDBOLCKNB', which means locking is performed without blocking.
    If successful, the return value is true, else, it is false. */
 bool tchdbopen(TCHDB *hdb, const char *path, int omode);
@@ -451,6 +473,28 @@ TCLIST *tchdbfwmkeys(TCHDB *hdb, const void *pbuf, int psiz, int max);
 TCLIST *tchdbfwmkeys2(TCHDB *hdb, const char *pstr, int max);
 
 
+/* Add an integer to a record in a hash database object.
+   `hdb' specifies the hash database object connected as a writer.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `num' specifies the additional value.
+   If successful, the return value is the summation value, else, it is `INT_MIN'.
+   If the corresponding record exists, the value is treated as an integer and is added to.  If no
+   record corresponds, a new record of the additional value is stored. */
+int tchdbaddint(TCHDB *hdb, const void *kbuf, int ksiz, int num);
+
+
+/* Add a real number to a record in a hash database object.
+   `hdb' specifies the hash database object connected as a writer.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `num' specifies the additional value.
+   If successful, the return value is the summation value, else, it is Not-a-Number.
+   If the corresponding record exists, the value is treated as a real number and is added to.  If
+   no record corresponds, a new record of the additional value is stored. */
+double tchdbadddouble(TCHDB *hdb, const void *kbuf, int ksiz, double num);
+
+
 /* Synchronize updated contents of a hash database object with the file and the device.
    `hdb' specifies the hash database object connected as a writer.
    If successful, the return value is true, else, it is false.
@@ -466,7 +510,7 @@ bool tchdbsync(TCHDB *hdb);
    setting is not changed.
    `fpow' specifies the maximum number of elements of the free block pool by power of 2.  If it
    is negative, the current setting is not changed.
-   `opts' specifies options by bitwise or: `HDBTLARGE' specifies that the size of the database
+   `opts' specifies options by bitwise-or: `HDBTLARGE' specifies that the size of the database
    can be larger than 2GB by using 64-bit bucket array, `HDBTDEFLATE' specifies that each record
    is compressed with Deflate encoding, `HDBTBZIP' specifies that each record is compressed with
    BZIP2 encoding, `HDBTTCBS' specifies that each record is compressed with TCBS encoding.  If it
@@ -495,6 +539,32 @@ bool tchdbvanish(TCHDB *hdb);
 bool tchdbcopy(TCHDB *hdb, const char *path);
 
 
+/* Begin the transaction of a hash database object.
+   `hdb' specifies the hash database object connected as a writer.
+   If successful, the return value is true, else, it is false.
+   The database is locked by the thread while the transaction so that only one transaction can be
+   activated with a database object at the same time.  Thus, the serializable isolation level is
+   assumed if every database operation is performed in the transaction.  All updated regions are
+   kept track of by write ahead logging while the transaction.  If the database is closed during
+   transaction, the transaction is aborted implicitly. */
+bool tchdbtranbegin(TCHDB *hdb);
+
+
+/* Commit the transaction of a hash database object.
+   `hdb' specifies the hash database object connected as a writer.
+   If successful, the return value is true, else, it is false.
+   Update in the transaction is fixed when it is committed successfully. */
+bool tchdbtrancommit(TCHDB *hdb);
+
+
+/* Abort the transaction of a hash database object.
+   `hdb' specifies the hash database object connected as a writer.
+   If successful, the return value is true, else, it is false.
+   Update in the transaction is discarded when it is aborted.  The state of the database is
+   rollbacked to before transaction. */
+bool tchdbtranabort(TCHDB *hdb);
+
+
 /* Get the file path of a hash database object.
    `hdb' specifies the hash database object.
    The return value is the path of the database file or `NULL' if the object does not connect to
@@ -549,6 +619,12 @@ void tchdbsetdbgfd(TCHDB *hdb, int fd);
 int tchdbdbgfd(TCHDB *hdb);
 
 
+/* Check whether mutual exclusion control is set to a hash database object.
+   `hdb' specifies the hash database object.
+   If mutual exclusion control is set, it is true, else it is false. */
+bool tchdbhasmutex(TCHDB *hdb);
+
+
 /* Synchronize updating contents on memory of a hash database object.
    `hdb' specifies the hash database object connected as a writer.
    `phys' specifies whether to synchronize physically.
@@ -557,7 +633,7 @@ bool tchdbmemsync(TCHDB *hdb, bool phys);
 
 
 /* Clear the cache of a hash tree database object.
-   `bdb' specifies the hash tree database object.
+   `hdb' specifies the hash tree database object.
    If successful, the return value is true, else, it is false. */
 bool tchdbcacheclear(TCHDB *hdb);
 
@@ -583,17 +659,24 @@ uint32_t tchdbalign(TCHDB *hdb);
 uint32_t tchdbfbpmax(TCHDB *hdb);
 
 
+/* Get the size of the extra mapped memory of a hash database object.
+   `hdb' specifies the hash database object.
+   The return value is the size of the extra mapped memory or 0 if the object does not connect to
+   any database file. */
+uint64_t tchdbxmsiz(TCHDB *hdb);
+
+
 /* Get the inode number of the database file of a hash database object.
    `hdb' specifies the hash database object.
-   The return value is the inode number of the database file or 0 the object does not connect to
-   any database file. */
+   The return value is the inode number of the database file or 0 if the object does not connect
+   to any database file. */
 uint64_t tchdbinode(TCHDB *hdb);
 
 
 /* Get the modification time of the database file of a hash database object.
    `hdb' specifies the hash database object.
-   The return value is the inode number of the database file or 0 the object does not connect to
-   any database file. */
+   The return value is the inode number of the database file or 0 if the object does not connect
+   to any database file. */
 time_t tchdbmtime(TCHDB *hdb);
 
 
@@ -629,14 +712,19 @@ char *tchdbopaque(TCHDB *hdb);
 
 /* Get the number of used elements of the bucket array of a hash database object.
    `hdb' specifies the hash database object.
-   The return value is the number of used elements of the bucket array or 0 if the object does not
-   connect to any database file. */
+   The return value is the number of used elements of the bucket array or 0 if the object does
+   not connect to any database file. */
 uint64_t tchdbbnumused(TCHDB *hdb);
 
 
 /* Set the custom codec functions of a hash database object.
    `hdb' specifies the hash database object.
-   `enc' specifies the pointer to the custom encoding function.
+   `enc' specifies the pointer to the custom encoding function.  It receives four parameters.
+   The first parameter is the pointer to the region.  The second parameter is the size of the
+   region.  The third parameter is the pointer to the variable into which the size of the region
+   of the return value is assigned.  The fourth parameter is the pointer to the optional opaque
+   object.  It returns the pointer to the result object allocated with `malloc' call if
+   successful, else, it returns `NULL'.
    `encop' specifies an arbitrary pointer to be given as a parameter of the encoding function.
    If it is not needed, `NULL' can be specified.
    `dec' specifies the pointer to the custom decoding function.
@@ -645,7 +733,7 @@ uint64_t tchdbbnumused(TCHDB *hdb);
    If successful, the return value is true, else, it is false.
    Note that the custom codec functions should be set before the database is opened and should be
    set every time the database is being opened. */
-bool tchdbsetcodecfunc(TCHDB *hdb, HDBCODEC enc, void *encop, HDBCODEC dec, void *decop);
+bool tchdbsetcodecfunc(TCHDB *hdb, TCCODEC enc, void *encop, TCCODEC dec, void *decop);
 
 
 /* Get the custom codec functions of a hash database object.
@@ -658,7 +746,44 @@ bool tchdbsetcodecfunc(TCHDB *hdb, HDBCODEC enc, void *encop, HDBCODEC dec, void
    function is assigned
    `dop' specifies the pointer to a variable into which the arbitrary pointer to be given to the
    decoding function is assigned. */
-void tchdbcodecfunc(TCHDB *hdb, HDBCODEC *ep, void **eop, HDBCODEC *dp, void **dop);
+void tchdbcodecfunc(TCHDB *hdb, TCCODEC *ep, void **eop, TCCODEC *dp, void **dop);
+
+
+/* Get the unit step number of auto defragmentation of a hash database object.
+   `hdb' specifies the hash database object.
+   The return value is the unit step number of auto defragmentation. */
+uint32_t tchdbdfunit(TCHDB *hdb);
+
+
+/* Perform dynamic defragmentation of a hash database object.
+   `hdb' specifies the hash database object connected as a writer.
+   `step' specifie the number of steps.  If it is not more than 0, the whole file is defragmented
+   gradually without keeping a continuous lock.
+   If successful, the return value is true, else, it is false. */
+bool tchdbdefrag(TCHDB *hdb, int64_t step);
+
+
+/* Store a record into a hash database object with a duplication handler.
+   `hdb' specifies the hash database object connected as a writer.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `vbuf' specifies the pointer to the region of the value.  `NULL' means that record addition is
+   ommited if there is no corresponding record.
+   `vsiz' specifies the size of the region of the value.
+   `proc' specifies the pointer to the callback function to process duplication.  It receives
+   four parameters.  The first parameter is the pointer to the region of the value.  The second
+   parameter is the size of the region of the value.  The third parameter is the pointer to the
+   variable into which the size of the region of the return value is assigned.  The fourth
+   parameter is the pointer to the optional opaque object.  It returns the pointer to the result
+   object allocated with `malloc'.  It is released by the caller.  If it is `NULL', the record is
+   not modified.  If it is `(void *)-1', the record is removed.
+   `op' specifies an arbitrary pointer to be given as a parameter of the callback function.  If
+   it is not needed, `NULL' can be specified.
+   If successful, the return value is true, else, it is false.
+   Note that the callback function can not perform any database operation because the function
+   is called in the critical section guarded by the same locks of database operations. */
+bool tchdbputproc(TCHDB *hdb, const void *kbuf, int ksiz, const void *vbuf, int vsiz,
+                  TCPDPROC proc, void *op);
 
 
 /* Retrieve the next record of a record in a hash database object.
@@ -668,7 +793,7 @@ void tchdbcodecfunc(TCHDB *hdb, HDBCODEC *ep, void **eop, HDBCODEC *dp, void **d
    `ksiz' specifies the size of the region of the key.
    `sp' specifies the pointer to the variable into which the size of the region of the return
    value is assigned.
-   If successful, the return value is the pointer to the region of the value of the next record.
+   If successful, the return value is the pointer to the region of the key of the next record.
    `NULL' is returned if no record corresponds.
    Because an additional zero code is appended at the end of the region of the return value,
    the return value can be treated as a character string.  Because the region of the return
@@ -677,16 +802,72 @@ void tchdbcodecfunc(TCHDB *hdb, HDBCODEC *ep, void **eop, HDBCODEC *dp, void **d
 void *tchdbgetnext(TCHDB *hdb, const void *kbuf, int ksiz, int *sp);
 
 
-/* Retrieve the next record of a string record in a hash database object.
+/* Retrieve the next string record in a hash database object.
    `hdb' specifies the hash database object.
    `kstr' specifies the string of the key.  If it is `NULL', the first record is retrieved.
-   If successful, the return value is the string of the value of the next record.  `NULL' is
+   If successful, the return value is the string of the key of the next record.  `NULL' is
    returned if no record corresponds.
    Because the region of the return value is allocated with the `malloc' call, it should be
    released with the `free' call when it is no longer in use. */
 char *tchdbgetnext2(TCHDB *hdb, const char *kstr);
 
 
+/* Retrieve the key and the value of the next record of a record in a hash database object.
+   `hdb' specifies the hash database object.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `sp' specifies the pointer to the variable into which the size of the region of the return
+   value is assigned.
+   `vbp' specifies the pointer to the variable into which the pointer to the value is assigned.
+   `vsp' specifies the pointer to the variable into which the size of the value is assigned.
+   If successful, the return value is the pointer to the region of the key of the next
+   record.
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call when it is no longer in use.  The retion pointed to by `vbp'
+   should not be released. */
+char *tchdbgetnext3(TCHDB *hdb, const char *kbuf, int ksiz, int *sp, const char **vbp, int *vsp);
+
+
+/* Move the iterator to the record corresponding a key of a hash database object.
+   `hdb' specifies the hash database object.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   If successful, the return value is true, else, it is false.  False is returned if there is
+   no record corresponding the condition. */
+bool tchdbiterinit2(TCHDB *hdb, const void *kbuf, int ksiz);
+
+
+/* Move the iterator to the record corresponding a key string of a hash database object.
+   `hdb' specifies the hash database object.
+   `kstr' specifies the string of the key.
+   If successful, the return value is true, else, it is false.  False is returned if there is
+   no record corresponding the condition. */
+bool tchdbiterinit3(TCHDB *hdb, const char *kstr);
+
+
+/* Process each record atomically of a hash database object.
+   `hdb' specifies the hash database object.
+   `iter' specifies the pointer to the iterator function called for each record.  It receives
+   five parameters.  The first parameter is the pointer to the region of the key.  The second
+   parameter is the size of the region of the key.  The third parameter is the pointer to the
+   region of the value.  The fourth parameter is the size of the region of the value.  The fifth
+   parameter is the pointer to the optional opaque object.  It returns true to continue iteration
+   or false to stop iteration.
+   `op' specifies an arbitrary pointer to be given as a parameter of the iterator function.  If
+   it is not needed, `NULL' can be specified.
+   If successful, the return value is true, else, it is false.
+   Note that the callback function can not perform any database operation because the function
+   is called in the critical section guarded by the same locks of database operations. */
+bool tchdbforeach(TCHDB *hdb, TCITER iter, void *op);
+
+
+/* Void the transaction of a hash database object.
+   `hdb' specifies the hash database object connected as a writer.
+   If successful, the return value is true, else, it is false.
+   This function should be called only when no update in the transaction. */
+bool tchdbtranvoid(TCHDB *hdb);
+
+
 
 __TCHDB_CLINKAGEEND
 #endif                                   /* duplication check */
