@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The test cases of the hash database API
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -18,7 +18,7 @@
 #include <tchdb.h>
 #include "myconf.h"
 
-#define RECBUFSIZ      32                // buffer for records
+#define RECBUFSIZ      48                // buffer for records
 
 typedef struct {                         // type of structure for write thread
   TCHDB *hdb;
@@ -59,9 +59,16 @@ typedef struct {                         // type of structure for typical thread
   int id;
 } TARGTYPICAL;
 
+typedef struct {                         // type of structure for race thread
+  TCHDB *hdb;
+  int rnum;
+  int id;
+} TARGRACE;
+
 
 /* global variables */
 const char *g_progname;                  // program name
+unsigned int g_randseed;                 // random seed
 int g_dbgfd;                             // debugging output
 
 
@@ -69,36 +76,47 @@ int g_dbgfd;                             // debugging output
 int main(int argc, char **argv);
 static void usage(void);
 static void iprintf(const char *format, ...);
+static void iputchar(int c);
 static void eprint(TCHDB *hdb, const char *func);
 static void mprint(TCHDB *hdb);
+static void sysprint(void);
 static int myrand(int range);
 static int myrandnd(int range);
+static bool iterfunc(const void *kbuf, int ksiz, const void *vbuf, int vsiz, void *op);
 static int runwrite(int argc, char **argv);
 static int runread(int argc, char **argv);
 static int runremove(int argc, char **argv);
 static int runwicked(int argc, char **argv);
 static int runtypical(int argc, char **argv);
+static int runrace(int argc, char **argv);
 static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
-                     int opts, int omode, bool as, bool rnd);
-static int procread(const char *path, int tnum, int rcnum, int omode, bool wb, bool rnd);
-static int procremove(const char *path, int tnum, int omode, bool rnd);
+                     int opts, int rcnum, int xmsiz, int dfunit, int omode, bool as, bool rnd);
+static int procread(const char *path, int tnum, int rcnum, int xmsiz, int dfunit, int omode,
+                    bool wb, bool rnd);
+static int procremove(const char *path, int tnum, int rcnum, int xmsiz, int dfunit, int omode,
+                      bool rnd);
 static int procwicked(const char *path, int tnum, int rnum, int opts, int omode, bool nc);
 static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
-                       int opts, int rcnum, int omode, bool nc, int rratio);
+                       int opts, int rcnum, int xmsiz, int dfunit, int omode,
+                       bool nc, int rratio);
+static int procrace(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
+                    int opts, int xmsiz, int dfunit, int omode);
 static void *threadwrite(void *targ);
 static void *threadread(void *targ);
 static void *threadremove(void *targ);
 static void *threadwicked(void *targ);
 static void *threadtypical(void *targ);
+static void *threadrace(void *targ);
 
 
 /* main routine */
 int main(int argc, char **argv){
   g_progname = argv[0];
-  g_dbgfd = -1;
-  const char *ebuf = getenv("TCDBGFD");
-  if(ebuf) g_dbgfd = atoi(ebuf);
-  srand((unsigned int)(tctime() * 1000) % UINT_MAX);
+  const char *ebuf = getenv("TCRNDSEED");
+  g_randseed = ebuf ? tcatoix(ebuf) : tctime() * 1000;
+  srand(g_randseed);
+  ebuf = getenv("TCDBGFD");
+  g_dbgfd = ebuf ? tcatoix(ebuf) : UINT16_MAX;
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "write")){
@@ -111,9 +129,18 @@ int main(int argc, char **argv){
     rv = runwicked(argc, argv);
   } else if(!strcmp(argv[1], "typical")){
     rv = runtypical(argc, argv);
+  } else if(!strcmp(argv[1], "race")){
+    rv = runrace(argc, argv);
   } else {
     usage();
   }
+  if(rv != 0){
+    printf("FAILED:");
+    for(int i = 0; i < argc; i++){
+      printf(" %s", argv[i]);
+    }
+    printf("\n\n");
+  }
   return rv;
 }
 
@@ -123,13 +150,17 @@ static void usage(void){
   fprintf(stderr, "%s: test cases of the hash database API of Tokyo Cabinet\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s write [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-as] [-rnd] path tnum rnum"
-          " [bnum [apow [fpow]]]\n", g_progname);
-  fprintf(stderr, "  %s read [-rc num] [-nl|-nb] [-wb] [-rnd] path tnum\n", g_progname);
-  fprintf(stderr, "  %s remove [-nl|-nb] [-rnd] path tnum\n", g_progname);
-  fprintf(stderr, "  %s wicked [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-nc] path tnum rnum\n",
+  fprintf(stderr, "  %s write [-tl] [-td|-tb|-tt|-tx] [-rc num] [-xm num] [-df num]"
+          " [-nl|-nb] [-as] [-rnd] path tnum rnum [bnum [apow [fpow]]]\n", g_progname);
+  fprintf(stderr, "  %s read [-rc num] [-xm num] [-df num] [-nl|-nb] [-wb] [-rnd] path tnum\n",
           g_progname);
-  fprintf(stderr, "  %s typical [-tl] [-td|-tb|-tt|-tx] [-rc num] [-nl|-nb] [-nc] [-rr num]"
+  fprintf(stderr, "  %s remove [-rc num] [-xm num] [-df num] [-nl|-nb] [-rnd] path tnum\n",
+          g_progname);
+  fprintf(stderr, "  %s wicked [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-nc]"
+          " path tnum rnum\n", g_progname);
+  fprintf(stderr, "  %s typical [-tl] [-td|-tb|-tt|-tx] [-rc num] [-xm num] [-df num]"
+          " [-nl|-nb] [-nc] [-rr num] path tnum rnum [bnum [apow [fpow]]]\n", g_progname);
+  fprintf(stderr, "  %s race [-tl] [-td|-tb|-tt|-tx] [-xm num] [-df num] [-nl|-nb]"
           " path tnum rnum [bnum [apow [fpow]]]\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
@@ -146,6 +177,13 @@ static void iprintf(const char *format, ...){
 }
 
 
+/* print a character and flush the buffer */
+static void iputchar(int c){
+  putchar(c);
+  fflush(stdout);
+}
+
+
 /* print error message of hash database */
 static void eprint(TCHDB *hdb, const char *func){
   const char *path = tchdbpath(hdb);
@@ -174,12 +212,33 @@ static void mprint(TCHDB *hdb){
   iprintf("cnt_deferdrp: %lld\n", (long long)hdb->cnt_deferdrp);
   iprintf("cnt_flushdrp: %lld\n", (long long)hdb->cnt_flushdrp);
   iprintf("cnt_adjrecc: %lld\n", (long long)hdb->cnt_adjrecc);
+  iprintf("cnt_defrag: %lld\n", (long long)hdb->cnt_defrag);
+  iprintf("cnt_shiftrec: %lld\n", (long long)hdb->cnt_shiftrec);
+  iprintf("cnt_trunc: %lld\n", (long long)hdb->cnt_trunc);
+}
+
+
+/* print system information */
+static void sysprint(void){
+  TCMAP *info = tcsysinfo();
+  if(info){
+    tcmapiterinit(info);
+    const char *kbuf;
+    while((kbuf = tcmapiternext2(info)) != NULL){
+      iprintf("sys_%s: %s\n", kbuf, tcmapiterval2(kbuf));
+    }
+    tcmapdel(info);
+  }
 }
 
 
 /* get a random number */
 static int myrand(int range){
-  return (int)((double)range * rand() / (RAND_MAX + 1.0));
+  if(range < 2) return 0;
+  int high = (unsigned int)rand() >> 4;
+  int low = range * (rand() / (RAND_MAX + 1.0));
+  low &= (unsigned int)INT_MAX >> 4;
+  return (high + low) % range;
 }
 
 
@@ -190,6 +249,19 @@ static int myrandnd(int range){
 }
 
 
+/* iterator function */
+static bool iterfunc(const void *kbuf, int ksiz, const void *vbuf, int vsiz, void *op){
+  unsigned int sum = 0;
+  while(--ksiz >= 0){
+    sum += ((char *)kbuf)[ksiz];
+  }
+  while(--vsiz >= 0){
+    sum += ((char *)vbuf)[vsiz];
+  }
+  return myrand(100 + (sum & 0xff)) > 0;
+}
+
+
 /* parse arguments of write command */
 static int runwrite(int argc, char **argv){
   char *path = NULL;
@@ -199,6 +271,9 @@ static int runwrite(int argc, char **argv){
   char *astr = NULL;
   char *fstr = NULL;
   int opts = 0;
+  int rcnum = 0;
+  int xmsiz = -1;
+  int dfunit = 0;
   int omode = 0;
   bool as = false;
   bool rnd = false;
@@ -214,6 +289,15 @@ static int runwrite(int argc, char **argv){
         opts |= HDBTTCBS;
       } else if(!strcmp(argv[i], "-tx")){
         opts |= HDBTEXCODEC;
+      } else if(!strcmp(argv[i], "-rc")){
+        if(++i >= argc) usage();
+        rcnum = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-nl")){
         omode |= HDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -242,13 +326,14 @@ static int runwrite(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
-  int bnum = bstr ? atoi(bstr) : -1;
-  int apow = astr ? atoi(astr) : -1;
-  int fpow = fstr ? atoi(fstr) : -1;
-  int rv = procwrite(path, tnum, rnum, bnum, apow, fpow, opts, omode, as, rnd);
+  int bnum = bstr ? tcatoix(bstr) : -1;
+  int apow = astr ? tcatoix(astr) : -1;
+  int fpow = fstr ? tcatoix(fstr) : -1;
+  int rv = procwrite(path, tnum, rnum, bnum, apow, fpow, opts, rcnum, xmsiz, dfunit, omode,
+                     as, rnd);
   return rv;
 }
 
@@ -258,6 +343,8 @@ static int runread(int argc, char **argv){
   char *path = NULL;
   char *tstr = NULL;
   int rcnum = 0;
+  int xmsiz = -1;
+  int dfunit = 0;
   int omode = 0;
   bool wb = false;
   bool rnd = false;
@@ -265,7 +352,13 @@ static int runread(int argc, char **argv){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-rc")){
         if(++i >= argc) usage();
-        rcnum = atoi(argv[i]);
+        rcnum = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-nl")){
         omode |= HDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -286,9 +379,9 @@ static int runread(int argc, char **argv){
     }
   }
   if(!path || !tstr) usage();
-  int tnum = atoi(tstr);
+  int tnum = tcatoix(tstr);
   if(tnum < 1) usage();
-  int rv = procread(path, tnum, rcnum, omode, wb, rnd);
+  int rv = procread(path, tnum, rcnum, xmsiz, dfunit, omode, wb, rnd);
   return rv;
 }
 
@@ -297,11 +390,23 @@ static int runread(int argc, char **argv){
 static int runremove(int argc, char **argv){
   char *path = NULL;
   char *tstr = NULL;
+  int rcnum = 0;
+  int xmsiz = -1;
+  int dfunit = 0;
   int omode = 0;
   bool rnd = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-nl")){
+      if(!strcmp(argv[i], "-rc")){
+        if(++i >= argc) usage();
+        rcnum = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-nl")){
         omode |= HDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
         omode |= HDBOLCKNB;
@@ -319,9 +424,9 @@ static int runremove(int argc, char **argv){
     }
   }
   if(!path || !tstr) usage();
-  int tnum = atoi(tstr);
+  int tnum = tcatoix(tstr);
   if(tnum < 1) usage();
-  int rv = procremove(path, tnum, omode, rnd);
+  int rv = procremove(path, tnum, rcnum, xmsiz, dfunit, omode, rnd);
   return rv;
 }
 
@@ -366,8 +471,8 @@ static int runwicked(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
   int rv = procwicked(path, tnum, rnum, opts, omode, nc);
   return rv;
@@ -384,6 +489,8 @@ static int runtypical(int argc, char **argv){
   char *fstr = NULL;
   int opts = 0;
   int rcnum = 0;
+  int xmsiz = -1;
+  int dfunit = 0;
   int omode = 0;
   int rratio = -1;
   bool nc = false;
@@ -401,7 +508,13 @@ static int runtypical(int argc, char **argv){
         opts |= HDBTEXCODEC;
       } else if(!strcmp(argv[i], "-rc")){
         if(++i >= argc) usage();
-        rcnum = atoi(argv[i]);
+        rcnum = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-nl")){
         omode |= HDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -410,7 +523,7 @@ static int runtypical(int argc, char **argv){
         nc = true;
       } else if(!strcmp(argv[i], "-rr")){
         if(++i >= argc) usage();
-        rratio = atoi(argv[i]);
+        rratio = tcatoix(argv[i]);
       } else {
         usage();
       }
@@ -431,23 +544,90 @@ static int runtypical(int argc, char **argv){
     }
   }
   if(!path || !tstr || !rstr) usage();
-  int tnum = atoi(tstr);
-  int rnum = atoi(rstr);
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
   if(tnum < 1 || rnum < 1) usage();
-  int bnum = bstr ? atoi(bstr) : -1;
-  int apow = astr ? atoi(astr) : -1;
-  int fpow = fstr ? atoi(fstr) : -1;
-  int rv = proctypical(path, tnum, rnum, bnum, apow, fpow, opts, rcnum, omode, nc, rratio);
+  int bnum = bstr ? tcatoix(bstr) : -1;
+  int apow = astr ? tcatoix(astr) : -1;
+  int fpow = fstr ? tcatoix(fstr) : -1;
+  int rv = proctypical(path, tnum, rnum, bnum, apow, fpow, opts, rcnum, xmsiz, dfunit, omode,
+                       nc, rratio);
+  return rv;
+}
+
+
+/* parse arguments of race command */
+static int runrace(int argc, char **argv){
+  char *path = NULL;
+  char *tstr = NULL;
+  char *rstr = NULL;
+  char *bstr = NULL;
+  char *astr = NULL;
+  char *fstr = NULL;
+  int opts = 0;
+  int xmsiz = -1;
+  int dfunit = 0;
+  int omode = 0;
+  for(int i = 2; i < argc; i++){
+    if(!path && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-tl")){
+        opts |= HDBTLARGE;
+      } else if(!strcmp(argv[i], "-td")){
+        opts |= HDBTDEFLATE;
+      } else if(!strcmp(argv[i], "-tb")){
+        opts |= HDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
+        opts |= HDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= HDBTEXCODEC;
+      } else if(!strcmp(argv[i], "-xm")){
+        if(++i >= argc) usage();
+        xmsiz = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-df")){
+        if(++i >= argc) usage();
+        dfunit = tcatoix(argv[i]);
+      } else if(!strcmp(argv[i], "-nl")){
+        omode |= HDBONOLCK;
+      } else if(!strcmp(argv[i], "-nb")){
+        omode |= HDBOLCKNB;
+      } else {
+        usage();
+      }
+    } else if(!path){
+      path = argv[i];
+    } else if(!tstr){
+      tstr = argv[i];
+    } else if(!rstr){
+      rstr = argv[i];
+    } else if(!bstr){
+      bstr = argv[i];
+    } else if(!astr){
+      astr = argv[i];
+    } else if(!fstr){
+      fstr = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!path || !tstr || !rstr) usage();
+  int tnum = tcatoix(tstr);
+  int rnum = tcatoix(rstr);
+  if(tnum < 1 || rnum < 1) usage();
+  int bnum = bstr ? tcatoix(bstr) : -1;
+  int apow = astr ? tcatoix(astr) : -1;
+  int fpow = fstr ? tcatoix(fstr) : -1;
+  int rv = procrace(path, tnum, rnum, bnum, apow, fpow, opts, xmsiz, dfunit, omode);
   return rv;
 }
 
 
 /* perform write command */
 static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
-                     int opts, int omode, bool as, bool rnd){
-  iprintf("<Writing Test>\n  path=%s  tnum=%d  rnum=%d  bnum=%d  apow=%d  fpow=%d"
-          "  opts=%d  omode=%d  as=%d  rnd=%d\n\n",
-          path, tnum, rnum, bnum, apow, fpow, opts, omode, as, rnd);
+                     int opts, int rcnum, int xmsiz, int dfunit, int omode, bool as, bool rnd){
+  iprintf("<Writing Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  bnum=%d  apow=%d  fpow=%d"
+          "  opts=%d  rcnum=%d  xmsiz=%d  dfunit=%d  omode=%d  as=%d  rnd=%d\n\n",
+          g_randseed, path, tnum, rnum, bnum, apow, fpow, opts, rcnum, xmsiz, dfunit, omode,
+          as, rnd);
   bool err = false;
   double stime = tctime();
   TCHDB *hdb = tchdbnew();
@@ -464,6 +644,18 @@ static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, i
     eprint(hdb, "tchdbtune");
     err = true;
   }
+  if(!tchdbsetcache(hdb, rcnum)){
+    eprint(hdb, "tchdbsetcache");
+    err = true;
+  }
+  if(xmsiz >= 0 && !tchdbsetxmsiz(hdb, xmsiz)){
+    eprint(hdb, "tchdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tchdbsetdfunit(hdb, dfunit)){
+    eprint(hdb, "tchdbsetdfunit");
+    err = true;
+  }
   if(!tchdbopen(hdb, path, HDBOWRITER | HDBOCREAT | HDBOTRUNC | omode)){
     eprint(hdb, "tchdbopen");
     err = true;
@@ -504,6 +696,7 @@ static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, i
   iprintf("record number: %llu\n", (unsigned long long)tchdbrnum(hdb));
   iprintf("size: %llu\n", (unsigned long long)tchdbfsiz(hdb));
   mprint(hdb);
+  sysprint();
   if(!tchdbclose(hdb)){
     eprint(hdb, "tchdbclose");
     err = true;
@@ -516,9 +709,10 @@ static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, i
 
 
 /* perform read command */
-static int procread(const char *path, int tnum, int rcnum, int omode, bool wb, bool rnd){
-  iprintf("<Reading Test>\n  path=%s  tnum=%d  rcnum=%d  omode=%d  wb=%d  rnd=%d\n\n",
-          path, tnum, rcnum, omode, wb, rnd);
+static int procread(const char *path, int tnum, int rcnum, int xmsiz, int dfunit, int omode,
+                    bool wb, bool rnd){
+  iprintf("<Reading Test>\n  seed=%u  path=%s  tnum=%d  rcnum=%d  xmsiz=%d  dfunit=%d  omode=%d"
+          "  wb=%d  rnd=%d\n\n", g_randseed, path, tnum, rcnum, xmsiz, dfunit, omode, wb, rnd);
   bool err = false;
   double stime = tctime();
   TCHDB *hdb = tchdbnew();
@@ -535,6 +729,14 @@ static int procread(const char *path, int tnum, int rcnum, int omode, bool wb, b
     eprint(hdb, "tchdbsetcache");
     err = true;
   }
+  if(xmsiz >= 0 && !tchdbsetxmsiz(hdb, xmsiz)){
+    eprint(hdb, "tchdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tchdbsetdfunit(hdb, dfunit)){
+    eprint(hdb, "tchdbsetdfunit");
+    err = true;
+  }
   if(!tchdbopen(hdb, path, HDBOREADER | omode)){
     eprint(hdb, "tchdbopen");
     err = true;
@@ -576,6 +778,7 @@ static int procread(const char *path, int tnum, int rcnum, int omode, bool wb, b
   iprintf("record number: %llu\n", (unsigned long long)tchdbrnum(hdb));
   iprintf("size: %llu\n", (unsigned long long)tchdbfsiz(hdb));
   mprint(hdb);
+  sysprint();
   if(!tchdbclose(hdb)){
     eprint(hdb, "tchdbclose");
     err = true;
@@ -588,8 +791,10 @@ static int procread(const char *path, int tnum, int rcnum, int omode, bool wb, b
 
 
 /* perform remove command */
-static int procremove(const char *path, int tnum, int omode, bool rnd){
-  iprintf("<Removing Test>\n  path=%s  tnum=%d  omode=%d  rnd=%d\n\n", path, tnum, omode, rnd);
+static int procremove(const char *path, int tnum, int rcnum, int xmsiz, int dfunit, int omode,
+                      bool rnd){
+  iprintf("<Removing Test>\n  seed=%u  path=%s  tnum=%d  rcnum=%d  xmsiz=%d  dfunit=%d  omode=%d"
+          "  rnd=%d\n\n", g_randseed, path, tnum, rcnum, xmsiz, dfunit, omode, rnd);
   bool err = false;
   double stime = tctime();
   TCHDB *hdb = tchdbnew();
@@ -602,6 +807,18 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
     eprint(hdb, "tchdbsetcodecfunc");
     err = true;
   }
+  if(!tchdbsetcache(hdb, rcnum)){
+    eprint(hdb, "tchdbsetcache");
+    err = true;
+  }
+  if(xmsiz >= 0 && !tchdbsetxmsiz(hdb, xmsiz)){
+    eprint(hdb, "tchdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tchdbsetdfunit(hdb, dfunit)){
+    eprint(hdb, "tchdbsetdfunit");
+    err = true;
+  }
   if(!tchdbopen(hdb, path, HDBOWRITER | omode)){
     eprint(hdb, "tchdbopen");
     err = true;
@@ -641,6 +858,7 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
   iprintf("record number: %llu\n", (unsigned long long)tchdbrnum(hdb));
   iprintf("size: %llu\n", (unsigned long long)tchdbfsiz(hdb));
   mprint(hdb);
+  sysprint();
   if(!tchdbclose(hdb)){
     eprint(hdb, "tchdbclose");
     err = true;
@@ -654,8 +872,8 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
 
 /* perform wicked command */
 static int procwicked(const char *path, int tnum, int rnum, int opts, int omode, bool nc){
-  iprintf("<Writing Test>\n  path=%s  tnum=%d  rnum=%d  opts=%d  omode=%d  nc=%d\n\n",
-          path, tnum, rnum, opts, omode, nc);
+  iprintf("<Writing Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  opts=%d  omode=%d  nc=%d\n\n",
+          g_randseed, path, tnum, rnum, opts, omode, nc);
   bool err = false;
   double stime = tctime();
   TCHDB *hdb = tchdbnew();
@@ -676,6 +894,14 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
     eprint(hdb, "tchdbsetcache");
     err = true;
   }
+  if(!tchdbsetxmsiz(hdb, rnum * sizeof(int))){
+    eprint(hdb, "tchdbsetxmsiz");
+    err = true;
+  }
+  if(!tchdbsetdfunit(hdb, 8)){
+    eprint(hdb, "tchdbsetdfunit");
+    err = true;
+  }
   if(!tchdbopen(hdb, path, HDBOWRITER | HDBOCREAT | HDBOTRUNC | omode)){
     eprint(hdb, "tchdbopen");
     err = true;
@@ -736,7 +962,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
       int rsiz;
       char *rbuf = tchdbget(hdb, kbuf, ksiz, &rsiz);
       if(vbuf){
-        putchar('.');
+        iputchar('.');
         if(!rbuf){
           eprint(hdb, "tchdbget");
           err = true;
@@ -745,7 +971,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
           err = true;
         }
       } else {
-        putchar('*');
+        iputchar('*');
         if(rbuf || tchdbecode(hdb) != TCENOREC){
           eprint(hdb, "(validation)");
           err = true;
@@ -760,6 +986,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
   iprintf("record number: %llu\n", (unsigned long long)tchdbrnum(hdb));
   iprintf("size: %llu\n", (unsigned long long)tchdbfsiz(hdb));
   mprint(hdb);
+  sysprint();
   if(!tchdbclose(hdb)){
     eprint(hdb, "tchdbclose");
     err = true;
@@ -773,10 +1000,12 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
 
 /* perform typical command */
 static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
-                       int opts, int rcnum, int omode, bool nc, int rratio){
-  iprintf("<Typical Access Test>\n  path=%s  tnum=%d  rnum=%d  bnum=%d  apow=%d  fpow=%d"
-          "  opts=%d  rcnum=%d  omode=%d  nc=%d  rratio=%d\n\n",
-          path, tnum, rnum, bnum, apow, fpow, opts, rcnum, omode, nc, rratio);
+                       int opts, int rcnum, int xmsiz, int dfunit, int omode,
+                       bool nc, int rratio){
+  iprintf("<Typical Access Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  bnum=%d  apow=%d"
+          "  fpow=%d  opts=%d  rcnum=%d  xmsiz=%d  dfunit=%d  omode=%d  nc=%d  rratio=%d\n\n",
+          g_randseed, path, tnum, rnum, bnum, apow, fpow, opts, rcnum, xmsiz, dfunit, omode,
+          nc, rratio);
   bool err = false;
   double stime = tctime();
   TCHDB *hdb = tchdbnew();
@@ -797,6 +1026,14 @@ static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow,
     eprint(hdb, "tchdbsetcache");
     err = true;
   }
+  if(xmsiz >= 0 && !tchdbsetxmsiz(hdb, xmsiz)){
+    eprint(hdb, "tchdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tchdbsetdfunit(hdb, dfunit)){
+    eprint(hdb, "tchdbsetdfunit");
+    err = true;
+  }
   if(!tchdbopen(hdb, path, HDBOWRITER | HDBOCREAT | HDBOTRUNC | omode)){
     eprint(hdb, "tchdbopen");
     err = true;
@@ -837,6 +1074,85 @@ static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow,
   iprintf("record number: %llu\n", (unsigned long long)tchdbrnum(hdb));
   iprintf("size: %llu\n", (unsigned long long)tchdbfsiz(hdb));
   mprint(hdb);
+  sysprint();
+  if(!tchdbclose(hdb)){
+    eprint(hdb, "tchdbclose");
+    err = true;
+  }
+  tchdbdel(hdb);
+  iprintf("time: %.3f\n", tctime() - stime);
+  iprintf("%s\n\n", err ? "error" : "ok");
+  return err ? 1 : 0;
+}
+
+
+/* perform race command */
+static int procrace(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
+                    int opts, int xmsiz, int dfunit, int omode){
+  iprintf("<Race Condition Test>\n  seed=%u  path=%s  tnum=%d  rnum=%d  bnum=%d  apow=%d"
+          "  fpow=%d  opts=%d  xmsiz=%d  dfunit=%d  omode=%d\n\n",
+          g_randseed, path, tnum, rnum, bnum, apow, fpow, opts, xmsiz, dfunit, omode);
+  bool err = false;
+  double stime = tctime();
+  TCHDB *hdb = tchdbnew();
+  if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  if(!tchdbsetmutex(hdb)){
+    eprint(hdb, "tchdbsetmutex");
+    err = true;
+  }
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(hdb, "tchdbsetcodecfunc");
+    err = true;
+  }
+  if(!tchdbtune(hdb, bnum, apow, fpow, opts)){
+    eprint(hdb, "tchdbtune");
+    err = true;
+  }
+  if(xmsiz >= 0 && !tchdbsetxmsiz(hdb, xmsiz)){
+    eprint(hdb, "tchdbsetxmsiz");
+    err = true;
+  }
+  if(dfunit >= 0 && !tchdbsetdfunit(hdb, dfunit)){
+    eprint(hdb, "tchdbsetdfunit");
+    err = true;
+  }
+  if(!tchdbopen(hdb, path, HDBOWRITER | HDBOCREAT | HDBOTRUNC | omode)){
+    eprint(hdb, "tchdbopen");
+    err = true;
+  }
+  TARGRACE targs[tnum];
+  pthread_t threads[tnum];
+  if(tnum == 1){
+    targs[0].hdb = hdb;
+    targs[0].rnum = rnum;
+    targs[0].id = 0;
+    if(threadrace(targs) != NULL) err = true;
+  } else {
+    for(int i = 0; i < tnum; i++){
+      targs[i].hdb = hdb;
+      targs[i].rnum = rnum;
+      targs[i].id = i;
+      if(pthread_create(threads + i, NULL, threadrace, targs + i) != 0){
+        eprint(hdb, "pthread_create");
+        targs[i].id = -1;
+        err = true;
+      }
+    }
+    for(int i = 0; i < tnum; i++){
+      if(targs[i].id == -1) continue;
+      void *rv;
+      if(pthread_join(threads[i], &rv) != 0){
+        eprint(hdb, "pthread_join");
+        err = true;
+      } else if(rv){
+        err = true;
+      }
+    }
+  }
+  iprintf("record number: %llu\n", (unsigned long long)tchdbrnum(hdb));
+  iprintf("size: %llu\n", (unsigned long long)tchdbfsiz(hdb));
+  mprint(hdb);
+  sysprint();
   if(!tchdbclose(hdb)){
     eprint(hdb, "tchdbclose");
     err = true;
@@ -874,8 +1190,7 @@ static void *threadwrite(void *targ){
       }
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -912,8 +1227,7 @@ static void *threadread(void *targ){
       tcfree(vbuf);
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -938,8 +1252,7 @@ static void *threadremove(void *targ){
       break;
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -966,7 +1279,7 @@ static void *threadwicked(void *targ){
     if(!nc) tcglobalmutexlock();
     switch(myrand(16)){
     case 0:
-      if(id == 0) putchar('0');
+      if(id == 0) iputchar('0');
       if(!tchdbput(hdb, kbuf, ksiz, vbuf, vsiz)){
         eprint(hdb, "tchdbput");
         err = true;
@@ -974,7 +1287,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapput(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 1:
-      if(id == 0) putchar('1');
+      if(id == 0) iputchar('1');
       if(!tchdbput2(hdb, kbuf, vbuf)){
         eprint(hdb, "tchdbput2");
         err = true;
@@ -982,7 +1295,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapput2(map, kbuf, vbuf);
       break;
     case 2:
-      if(id == 0) putchar('2');
+      if(id == 0) iputchar('2');
       if(!tchdbputkeep(hdb, kbuf, ksiz, vbuf, vsiz) && tchdbecode(hdb) != TCEKEEP){
         eprint(hdb, "tchdbputkeep");
         err = true;
@@ -990,7 +1303,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputkeep(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 3:
-      if(id == 0) putchar('3');
+      if(id == 0) iputchar('3');
       if(!tchdbputkeep2(hdb, kbuf, vbuf) && tchdbecode(hdb) != TCEKEEP){
         eprint(hdb, "tchdbputkeep2");
         err = true;
@@ -998,7 +1311,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputkeep2(map, kbuf, vbuf);
       break;
     case 4:
-      if(id == 0) putchar('4');
+      if(id == 0) iputchar('4');
       if(!tchdbputcat(hdb, kbuf, ksiz, vbuf, vsiz)){
         eprint(hdb, "tchdbputcat");
         err = true;
@@ -1006,7 +1319,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputcat(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 5:
-      if(id == 0) putchar('5');
+      if(id == 0) iputchar('5');
       if(!tchdbputcat2(hdb, kbuf, vbuf)){
         eprint(hdb, "tchdbputcat2");
         err = true;
@@ -1014,7 +1327,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapputcat2(map, kbuf, vbuf);
       break;
     case 6:
-      if(id == 0) putchar('6');
+      if(id == 0) iputchar('6');
       if(i > rnum / 4 * 3){
         if(!tchdbputasync(hdb, kbuf, ksiz, vbuf, vsiz)){
           eprint(hdb, "tchdbputasync");
@@ -1029,7 +1342,7 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapput(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 7:
-      if(id == 0) putchar('7');
+      if(id == 0) iputchar('7');
       if(i > rnum / 4 * 3){
         if(!tchdbputasync2(hdb, kbuf, vbuf)){
           eprint(hdb, "tchdbputasync2");
@@ -1044,8 +1357,8 @@ static void *threadwicked(void *targ){
       if(!nc) tcmapput2(map, kbuf, vbuf);
       break;
     case 8:
-      if(id == 0) putchar('8');
-      if(myrand(10) == 0){
+      if(id == 0) iputchar('8');
+      if(myrand(2) == 0){
         if(!tchdbout(hdb, kbuf, ksiz) && tchdbecode(hdb) != TCENOREC){
           eprint(hdb, "tchdbout");
           err = true;
@@ -1054,8 +1367,8 @@ static void *threadwicked(void *targ){
       }
       break;
     case 9:
-      if(id == 0) putchar('9');
-      if(myrand(10) == 0){
+      if(id == 0) iputchar('9');
+      if(myrand(2) == 0){
         if(!tchdbout2(hdb, kbuf) && tchdbecode(hdb) != TCENOREC){
           eprint(hdb, "tchdbout2");
           err = true;
@@ -1064,7 +1377,7 @@ static void *threadwicked(void *targ){
       }
       break;
     case 10:
-      if(id == 0) putchar('A');
+      if(id == 0) iputchar('A');
       if(!(rbuf = tchdbget(hdb, kbuf, ksiz, &vsiz))){
         if(tchdbecode(hdb) != TCENOREC){
           eprint(hdb, "tchdbget");
@@ -1087,7 +1400,7 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 11:
-      if(id == 0) putchar('B');
+      if(id == 0) iputchar('B');
       if(!(rbuf = tchdbget(hdb, kbuf, ksiz, &vsiz)) && tchdbecode(hdb) != TCENOREC){
         eprint(hdb, "tchdbget");
         err = true;
@@ -1095,7 +1408,7 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 12:
-      if(id == 0) putchar('C');
+      if(id == 0) iputchar('C');
       if(!(rbuf = tchdbget2(hdb, kbuf)) && tchdbecode(hdb) != TCENOREC){
         eprint(hdb, "tchdbget2");
         err = true;
@@ -1103,7 +1416,7 @@ static void *threadwicked(void *targ){
       tcfree(rbuf);
       break;
     case 13:
-      if(id == 0) putchar('D');
+      if(id == 0) iputchar('D');
       if(myrand(1) == 0) vsiz = 1;
       if((vsiz = tchdbget3(hdb, kbuf, ksiz, vbuf, vsiz)) < 0 && tchdbecode(hdb) != TCENOREC){
         eprint(hdb, "tchdbget3");
@@ -1111,7 +1424,7 @@ static void *threadwicked(void *targ){
       }
       break;
     case 14:
-      if(id == 0) putchar('E');
+      if(id == 0) iputchar('E');
       if(myrand(rnum / 50) == 0){
         if(!tchdbiterinit(hdb)){
           eprint(hdb, "tchdbiterinit");
@@ -1147,7 +1460,42 @@ static void *threadwicked(void *targ){
       tcxstrdel(ikey);
       break;
     default:
-      if(id == 0) putchar('@');
+      if(id == 0) iputchar('@');
+      if(tchdbtranbegin(hdb)){
+        if(myrand(2) == 0){
+          if(!tchdbput(hdb, kbuf, ksiz, vbuf, vsiz)){
+            eprint(hdb, "tchdbput");
+            err = true;
+          }
+          if(!nc) tcmapput(map, kbuf, ksiz, vbuf, vsiz);
+        } else {
+          if(!tchdbout(hdb, kbuf, ksiz) && tchdbecode(hdb) != TCENOREC){
+            eprint(hdb, "tchdbout");
+            err = true;
+          }
+          if(!nc) tcmapout(map, kbuf, ksiz);
+        }
+        if(nc && myrand(2) == 0){
+          if(!tchdbtranabort(hdb)){
+            eprint(hdb, "tchdbtranabort");
+            err = true;
+          }
+        } else {
+          if(!tchdbtrancommit(hdb)){
+            eprint(hdb, "tchdbtrancommit");
+            err = true;
+          }
+        }
+      } else {
+        eprint(hdb, "tchdbtranbegin");
+        err = true;
+      }
+      if(myrand(1000) == 0){
+        if(!tchdbforeach(hdb, iterfunc, NULL)){
+          eprint(hdb, "tchdbforeach");
+          err = true;
+        }
+      }
       if(myrand(10000) == 0) srand((unsigned int)(tctime() * 1000) % UINT_MAX);
       break;
     }
@@ -1155,7 +1503,7 @@ static void *threadwicked(void *targ){
     if(id == 0){
       if(i % 50 == 0) iprintf(" (%08d)\n", i);
       if(id == 0 && i == rnum / 4){
-        if(!tchdboptimize(hdb, rnum / 50, -1, -1, -1)){
+        if(!tchdboptimize(hdb, rnum / 50, -1, -1, -1) && tchdbecode(hdb) != TCEINVALID){
           eprint(hdb, "tchdboptimize");
           err = true;
         }
@@ -1262,8 +1610,7 @@ static void *threadtypical(void *targ){
       }
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
-      putchar('.');
-      fflush(stdout);
+      iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
   }
@@ -1293,5 +1640,118 @@ static void *threadtypical(void *targ){
 }
 
 
+/* thread the race function */
+static void *threadrace(void *targ){
+  TCHDB *hdb = ((TARGRACE *)targ)->hdb;
+  int rnum = ((TARGRACE *)targ)->rnum;
+  int id = ((TARGRACE *)targ)->id;
+  bool err = false;
+  int mid = rnum * 2;
+  for(int i = 1; !err && i <= rnum; i++){
+    char buf[RECBUFSIZ];
+    int len = sprintf(buf, "%d", myrandnd(i));
+    int rnd = myrand(100);
+    if(rnd < 10){
+      if(!tchdbputkeep(hdb, buf, len, buf, len) && tchdbecode(hdb) != TCEKEEP){
+        eprint(hdb, "tchdbputkeep");
+        err = true;
+      }
+    } else if(rnd < 20){
+      if(!tchdbputcat(hdb, buf, len, buf, len)){
+        eprint(hdb, "tchdbputcat");
+        err = true;
+      }
+    } else if(rnd < 30){
+      if(!tchdbout(hdb, buf, len) && tchdbecode(hdb) != TCENOREC){
+        eprint(hdb, "tchdbout");
+        err = true;
+      }
+    } else if(rnd < 31){
+      if(!tchdbputasync(hdb, buf, len, buf, len)){
+        eprint(hdb, "tchdbputasync");
+        err = true;
+      }
+    } else {
+      if(myrand(10) == 0){
+        int rsiz = myrand(256);
+        char *rbuf = tcmalloc(rsiz + 1);
+        for(int j = 0; j < rsiz; j++){
+          rbuf[j] = myrand('z' - 'a') + 'a';
+        }
+        if(myrand(2) == 0){
+          if(!tchdbput(hdb, buf, len, rbuf, rsiz)){
+            eprint(hdb, "tchdbputcat");
+            err = true;
+          }
+        } else {
+          if(!tchdbputcat(hdb, buf, len, rbuf, rsiz)){
+            eprint(hdb, "tchdbputcat");
+            err = true;
+          }
+        }
+        tcfree(rbuf);
+      } else {
+        if(!tchdbput(hdb, buf, len, buf, len)){
+          eprint(hdb, "tchdbput");
+          err = true;
+        }
+      }
+    }
+    if(id == 0){
+      if(myrand(mid) == 0){
+        iprintf("[v]");
+        if(!tchdbvanish(hdb)){
+          eprint(hdb, "tchdbvanish");
+          err = true;
+        }
+      }
+      if(myrand(mid) == 0){
+        iprintf("[o]");
+        if(!tchdboptimize(hdb, myrand(rnum) + 1, myrand(10), myrand(10), 0)){
+          eprint(hdb, "tchdboptimize");
+          err = true;
+        }
+      }
+      if(myrand(mid) == 0){
+        iprintf("[d]");
+        if(!tchdbdefrag(hdb, -1)){
+          eprint(hdb, "tchdbdefrag");
+          err = true;
+        }
+      }
+      if(myrand(mid) == 0){
+        iprintf("[i]");
+        if(!tchdbiterinit(hdb)){
+          eprint(hdb, "tchdbput");
+          err = true;
+        }
+        char *kbuf;
+        int ksiz;
+        while((kbuf = tchdbiternext(hdb, &ksiz)) != NULL){
+          int vsiz;
+          char *vbuf = tchdbget(hdb, kbuf, ksiz, &vsiz);
+          if(vbuf){
+            tcfree(vbuf);
+          } else if(tchdbecode(hdb) != TCENOREC){
+            eprint(hdb, "tchdbget");
+            err = true;
+          }
+          tcfree(kbuf);
+        }
+        if(tchdbecode(hdb) != TCENOREC){
+          eprint(hdb, "(validation)");
+          err = true;
+        }
+      }
+      if(rnum > 250 && i % (rnum / 250) == 0){
+        iputchar('.');
+        if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
+      }
+    }
+  }
+  return err ? "error" : NULL;
+}
+
+
 
 // END OF FILE
