@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Popular encoders and decoders of the utility API
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2009 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -30,6 +30,7 @@ static int runurl(int argc, char **argv);
 static int runbase(int argc, char **argv);
 static int runquote(int argc, char **argv);
 static int runmime(int argc, char **argv);
+static int runhex(int argc, char **argv);
 static int runpack(int argc, char **argv);
 static int runtcbs(int argc, char **argv);
 static int runzlib(int argc, char **argv);
@@ -44,6 +45,7 @@ static int procbase(const char *ibuf, int isiz, bool dec);
 static int procquote(const char *ibuf, int isiz, bool dec);
 static int procmime(const char *ibuf, int isiz, bool dec, const char *ename, bool qb, bool on,
                     bool hd, bool bd, int part);
+static int prochex(const char *ibuf, int isiz, bool dec);
 static int procpack(const char *ibuf, int isiz, bool dec, bool bwt);
 static int proctcbs(const char *ibuf, int isiz, bool dec);
 static int proczlib(const char *ibuf, int isiz, bool dec, bool gz);
@@ -68,6 +70,8 @@ int main(int argc, char **argv){
     rv = runquote(argc, argv);
   } else if(!strcmp(argv[1], "mime")){
     rv = runmime(argc, argv);
+  } else if(!strcmp(argv[1], "hex")){
+    rv = runhex(argc, argv);
   } else if(!strcmp(argv[1], "pack")){
     rv = runpack(argc, argv);
   } else if(!strcmp(argv[1], "tcbs")){
@@ -103,6 +107,7 @@ static void usage(void){
   fprintf(stderr, "  %s quote [-d] [file]\n", g_progname);
   fprintf(stderr, "  %s mime [-d] [-en name] [-q] [-on] [-hd] [-bd] [-part num] [file]\n",
           g_progname);
+  fprintf(stderr, "  %s hex [-d] [file]\n", g_progname);
   fprintf(stderr, "  %s pack [-d] [-bwt] [file]\n", g_progname);
   fprintf(stderr, "  %s tcbs [-d] [file]\n", g_progname);
   fprintf(stderr, "  %s zlib [-d] [-gz] [file]\n", g_progname);
@@ -270,7 +275,7 @@ static int runmime(int argc, char **argv){
         bd = true;
       } else if(!strcmp(argv[i], "-part")){
         if(++i >= argc) usage();
-        part = atoi(argv[i]);
+        part = tcatoix(argv[i]);
       } else {
         usage();
       }
@@ -300,6 +305,42 @@ static int runmime(int argc, char **argv){
 }
 
 
+/* parse arguments of hex command */
+static int runhex(int argc, char **argv){
+  char *path = NULL;
+  bool dec = false;
+  for(int i = 2; i < argc; i++){
+    if(!path && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-d")){
+        dec = true;
+      } else {
+        usage();
+      }
+    } else if(!path){
+      path = argv[i];
+    } else {
+      usage();
+    }
+  }
+  char *ibuf;
+  int isiz;
+  if(path && path[0] == '@'){
+    isiz = strlen(path) - 1;
+    ibuf = tcmemdup(path + 1, isiz);
+  } else {
+    ibuf = tcreadfile(path, -1, &isiz);
+  }
+  if(!ibuf){
+    eprintf("%s: cannot open", path ? path : "(stdin)");
+    return 1;
+  }
+  int rv = prochex(ibuf, isiz, dec);
+  if(path && path[0] == '@') printf("\n");
+  tcfree(ibuf);
+  return rv;
+}
+
+
 /* parse arguments of pack command */
 static int runpack(int argc, char **argv){
   char *path = NULL;
@@ -536,7 +577,7 @@ static int runhash(int argc, char **argv){
         crc = true;
       } else if(!strcmp(argv[i], "-ch")){
         if(++i >= argc) usage();
-        ch = atoi(argv[i]);
+        ch = tcatoix(argv[i]);
       } else {
         usage();
       }
@@ -577,7 +618,7 @@ static int rundate(int argc, char **argv){
         str = argv[i];
       } else if(!strcmp(argv[i], "-jl")){
         if(++i >= argc) usage();
-        jl = atoi(argv[i]);
+        jl = tcatoix(argv[i]);
       } else if(!strcmp(argv[i], "-wf")){
         wf = true;
       } else if(!strcmp(argv[i], "-rf")){
@@ -741,6 +782,22 @@ static int procmime(const char *ibuf, int isiz, bool dec, const char *ename, boo
 }
 
 
+/* perform hex command */
+static int prochex(const char *ibuf, int isiz, bool dec){
+  if(dec){
+    int osiz;
+    char *obuf = tchexdecode(ibuf, &osiz);
+    fwrite(obuf, 1, osiz, stdout);
+    tcfree(obuf);
+  } else {
+    char *obuf = tchexencode(ibuf, isiz);
+    fwrite(obuf, 1, strlen(obuf), stdout);
+    tcfree(obuf);
+  }
+  return 0;
+}
+
+
 /* perform pack command */
 static int procpack(const char *ibuf, int isiz, bool dec, bool bwt){
   if(dec){
@@ -990,6 +1047,7 @@ static int procconf(int mode){
     break;
   default:
     printf("myconf(version): %s\n", tcversion);
+    printf("myconf(sysname): %s\n", TCSYSNAME);
     printf("myconf(libver): %d\n", _TC_LIBVER);
     printf("myconf(formatver): %s\n", _TC_FORMATVER);
     printf("myconf(prefix): %s\n", _TC_PREFIX);
@@ -1001,6 +1059,7 @@ static int procconf(int mode){
     printf("myconf(applibs): %s\n", _TC_APPLIBS);
     printf("myconf(bigend): %d\n", TCBIGEND);
     printf("myconf(usezlib): %d\n", TCUSEZLIB);
+    printf("myconf(usebzip): %d\n", TCUSEBZIP);
     printf("sizeof(bool): %d\n", sizeof(bool));
     printf("sizeof(char): %d\n", sizeof(char));
     printf("sizeof(short): %d\n", sizeof(short));
@@ -1010,17 +1069,22 @@ static int procconf(int mode){
     printf("sizeof(float): %d\n", sizeof(float));
     printf("sizeof(double): %d\n", sizeof(double));
     printf("sizeof(long double): %d\n", sizeof(long double));
+    printf("sizeof(void *): %d\n", sizeof(void *));
+    printf("sizeof(intptr_t): %d\n", sizeof(intptr_t));
     printf("sizeof(size_t): %d\n", sizeof(size_t));
     printf("sizeof(time_t): %d\n", sizeof(time_t));
     printf("sizeof(off_t): %d\n", sizeof(off_t));
     printf("sizeof(ino_t): %d\n", sizeof(ino_t));
-    printf("sizeof(intptr_t): %d\n", sizeof(intptr_t));
-    printf("sizeof(void *): %d\n", sizeof(void *));
+    printf("sizeof(wchar_t): %d\n", sizeof(wchar_t));
+    printf("sizeof(sig_atomic_t): %d\n", sizeof(sig_atomic_t));
     printf("macro(CHAR_MAX): %llu\n", (unsigned long long)CHAR_MAX);
     printf("macro(SHRT_MAX): %llu\n", (unsigned long long)SHRT_MAX);
     printf("macro(INT_MAX): %llu\n", (unsigned long long)INT_MAX);
     printf("macro(LONG_MAX): %llu\n", (unsigned long long)LONG_MAX);
     printf("macro(LLONG_MAX): %llu\n", (unsigned long long)LLONG_MAX);
+    printf("macro(FLT_MAX): %g\n", (double)FLT_MAX);
+    printf("macro(DBL_MAX): %g\n", (double)DBL_MAX);
+    printf("macro(LDBL_MAX): %Lg\n", (long double)LDBL_MAX);
     printf("macro(PATH_MAX): %llu\n", (unsigned long long)PATH_MAX);
     printf("macro(RAND_MAX): %llu\n", (unsigned long long)RAND_MAX);
     printf("sysconf(_SC_CLK_TCK): %ld\n", sysconf(_SC_CLK_TCK));
